#if !defined(LV_SRE_h)
#define LV_SRE_h

//note: need to add more compiler support (ie. Borland ?)
#if defined(_MSC_VER) 
 #ifdef LV_SRE_EXPORTS
  #define LV_SRE_API __declspec(dllexport)
 #else
  #define LV_SRE_API __declspec(dllimport)
 #endif
#else //linux
 #define LV_SRE_API
#endif

#include "LV_SRE_ParseTree.h"
#include "LV_SRE_Semantic.h"
#include "LV_SRE_Grammar.h"

#if !defined(NULL)
#define NULL 0
#endif

typedef void* HPORT;//internal handle

//Standard Grammars
#define 	GRAMMAR_NONE			0		//do not use a standard grammar
#define 	GRAMMAR_DIGITS			1		//like a telephone number or pin code
#define		GRAMMAR_MONEY			2		//Monetary Value
#define		GRAMMAR_NUMERIC			3		//any number
#define		GRAMMAR_SPELLING		4		//Used for spelling - not implemented
#define		GRAMMAR_DATE			5		//Not implemented now
#define		GRAMMAR_ALPHA_NUMERIC	6		//Not implemented now
#define		GRAMMAR_ORDINAL			7		//Not implemented now

//Error return codes //////////////////////////////////////////////////////////
#define		LV_SUCCESS								 0
#define		LV_FAILURE								-1
#define		LV_SYSTEM_ERROR							-2
#define		LV_RESET								-3
#define		LV_BAD_SOUND_DATA						-4
#define		LV_INVALID_SOUND_FORMAT					-5
#define		LV_TIME_OUT								-6
#define		LV_GRAMMAR_SET_OUT_OF_RANGE				-7	
#define		LV_SOUND_CHANNEL_OUT_OF_RANGE			-8	
#define		LV_STANDARD_GRAMMAR_ALREADY_LOADED		-9
#define		LV_STANDARD_GRAMMAR_OUT_OF_RANGE	   -10
#define     LV_INVALID_PROPERTY_VALUE              -11
#define		LV_NOT_A_VALID_PROPERTY_VALUE          LV_INVALID_PROPERTY_VALUE //deprecated name
#define     LV_INVALID_HPORT                       -12
#define     LV_BAD_HPORT                           LV_INVALID_HPORT //deprecated name
#define     LV_NOT_IMPLEMENTED                     -13
#define     LV_SOCKETS_ERROR	                   -14
#define     LV_INVALID_PROPERTY_TARGET             -15
#define     LV_INVALID_PROPERTY_VALUE_TYPE         -16
#define     LV_INVALID_PROPERTY                    -17
#define     LV_INVALID_PROPERTY_TARGET_NDX         -18
#define     LV_STREAM_NOT_ACCEPTED				   -19
#define     LV_FUNCTION_NOT_FOUND				   -20
#define     LV_STRING_BUFFER_TOO_SMALL			   -21
#define		LV_NO_SERVER_AVAILABLE				   -22
#define		LV_GRAMMAR_SYNTAX_WARNING			   -23
#define     LV_GRAMMAR_WARNING                     LV_GRAMMAR_SYNTAX_WARNING //deprecated name
#define		LV_GRAMMAR_SYNTAX_ERROR                -24
#define     LV_GRAMMAR_ERROR                       LV_GRAMMAR_SYNTAX_ERROR //deprecated name 
                                                                           //(and changed meaning--used to mean syntax 
                                                                           // or loading error)
#define		LV_GRAMMAR_LOADING_ERROR				 -25

#define		LV_OPEN_PORT_FAILED__LICENSES_EXCEEDED				-26
#define		LV_OPEN_PORT_FAILED__PRIMARY_SERVER_NOT_RESPONDING	-27
#define		LV_OPEN_PORT_FAILED__PRIMARY_SERVER_NOT_COMPATIBLE	-28
#define		LV_OPEN_PORT_FAILED__OPEN_PORT_NOT_CALLED_YET		-29

#define		LV_NO_SERVER_RESPONDING								-30
				
                                                       
//used internally
#define     LV_LAST_ERROR LV_NO_SERVER_RESPONDING
#define     LV_NUM_ERRORS ((-(LV_LAST_ERROR))+1)

enum SOUND_FORMAT
{
    UNK_FORMAT = 0,
	ULAW_8KHZ,
	PCM_8KHZ,
	PCM_16KHZ,
	ALAW_8KHZ,
	SPX_8KHZ,
	SPX_16KHZ
};

//changed from enum to defines
//typedef 
//enum PROPERTIES
//{
//	PROP_TRIM_SILENCE_VALUE,		//Range of 0 - 100 ; smaller value means less aggressive trimming
//							//DEFAULT: 
//
//	//PROP_TOTAL_GRAMMAR_SIZE,		//Number of words that will be added to grammar while port is open 
//							//DEFAULT: 10000
//
//	PROP_SAVE_SOUND_FILES	//If 1 the sound files are stored to disk
//							//DEFAULT: 0
//};

//SetPropertyEx - properties
#define PROP_EX_TRIM_SILENCE_VALUE 1
#define PROP_EX_SAVE_SOUND_FILES   2
#define PROP_EX_LANGUAGE           3
#define PROP_EX_SRE_SERVERS        4	//this is a client level property (PROP_EX_TARGET_CLIENT) format "IP:Port;IP:Port"
//proerties for noise reduction
#define PROP_EX_NOISE_REDUCTION_ENABLE			5
#define PROP_EX_NOISE_REDUCTION_AUTO_ADAPTIVE	6
#define PROP_EX_NOISE_REDUCTION_ALPHA			7

#define PROP_EX_CHOOSE_MODEL	8

#define PROP_EX_SET_SERVER_IP	10	//server IP addresses to connect to (comma delimited list) first IP is master server
	//valid only for PROP_EX_TARGET_CLIENT (client process)
#define PROP_EX_SET_SERVER_PORT 11	//server Port addresses (to IPs of PROP_EX_SET_SERVER_IP)
	//valid only for PROP_EX_TARGET_CLIENT (client process)

#define PROP_EX_SEARCH_BEAM_WIDTH  12// during decode, beam width controls how many paths are searched.
                                     // narrower beams lead to faster searches, but with reduced accuracy.
                                     // the larger the beam value, the narrower the beam.  
                                     // a beam-width value of 0.0 corresponds to a completely
                                     // open search.  Default value is 1e-6

///////////////////////////////////////////////////////////////////////////////
/// \defgroup Concept Repetition variables
///
/// Control how many "concepts" in a grammar may appear.  Must be used in 
/// conjunction with the decode flag LV_DECODE_SRGS_GRAMMAR
///
/// \target       PROP_EX_TARGET_GRAMMAR
/// \type         PROP_EX_VALUE_TYPE_INT or PROP_EX_VALUE_TYPE_INT_PTR
///
/// \note         MIN <= MAX
/// \note         MAX == -1 means MAX == infinity
/// \note         values reset when a grammar is reset.
/// 
/// \default      MAX==infinity MIN==1
///
///////////////////////////////////////////////////////////////////////////////
#define PROP_EX_CONCEPT_REPETITION_MIN 13 
#define PROP_EX_CONCEPT_REPETITION_MAX 14 

                                          

///////////////////////////////////////////////////////////////////////////////
/// 
/// \def PROP_EX_ENABLE_LATTICE_CONFIDENCE_SCORE
///
/// \target PROP_EX_TARGET_CLIENT
/// \target PROP_EX_TARGET_PORT
/// \type   PROP_EX_VALUE_TYPE_INT
/// \type   PROP_EX_VALUE_TYPE_INT_PTR
///
/// \default 1 (on)
///  
///////////////////////////////////////////////////////////////////////////////                                         
#define PROP_EX_ENABLE_LATTICE_CONFIDENCE_SCORE 15 

#define USE_41_SRE

//SetProperty - map old names to new names
#define PROP_TRIM_SILENCE_VALUE PROP_EX_TRIM_SILENCE_VALUE
#define PROP_SAVE_SOUND_FILES   PROP_EX_SAVE_SOUND_FILES

//SetPropertyEx - port,channel, or grammar
#define PROP_EX_TARGET_PORT     1
#define PROP_EX_TARGET_CHANNEL  2
#define PROP_EX_TARGET_GRAMMAR  3
#define PROP_EX_TARGET_CLIENT   4	//all ports  //set hport to null when using this parm type

//SetPropertyEx - indicates how to typecast void *
#define PROP_EX_VALUE_TYPE_INT         1
#define PROP_EX_VALUE_TYPE_INT_PTR     2
#define PROP_EX_VALUE_TYPE_STRING      3
#define PROP_EX_VALUE_TYPE_FLOAT_PTR   4
#define PROP_EX_VALUE_TYPE_FLOAT PROP_EX_VALUE_TYPE_FLOAT_PTR /*misnamed.*/

//SetPropertyEx - Language names
#define PROP_EX_LANG_AMERICAN_ENGLISH "AmericanEnglish"/*"AmericanEnglish"*/
#define PROP_EX_LANG_SPANISH "Spanish" /*"Spanish"*/
#define PROP_EX_LANG_FRENCH "French" /*"French"*/

//bitflags - currently only used by Decode()
#define LV_DECODE_USE_OOV            1
#define LV_DECODE_BLOCK              2
#define LV_DECODE_GENDER_MALE        4
#define LV_DECODE_GENDER_FEMALE      8
#define LV_DECODE_FIRST_TIME_USER   16
#define LV_DECODE_RETURN_EACH_DIGIT 32
//64 and 128 are reserved.
#define LV_DECODE_SRGS_GRAMMAR 256 
#define LV_DECODE_USE_ABNF_GRAMMAR LV_DECODE_SRGS_GRAMMAR //deprecated name.
#define LV_DECODE_SEMANTIC_INTERPRETATION	512


//used to identify the SRGS ActiveGrammarSet in the Decode's "grammarset" argument.
#define LV_ACTIVE_GRAMMAR_SET -1

//used as voicechannel argument to get answers back.
#define LV_DTMF_CHANNEL -1


//Event defines
#define EVENT_DTMF "DTMF"
#define EVENT_TRANSFER "TRANSFER"
#define EVENT_NOINPUT "NO_INPUT"
#define EVENT_START_DECODE_SEQ "START_DECODE_SEQ"
#define EVENT_END_DECODE_SEQ "END_DECODE_SEQ"
#define EVENT_SRE "SRE"
#define EVENT_SRE_FAIL "SRE_FAIL"
#define EVENT_SRE_APPLICATION_ACCEPTED_CONCEPT "SRE_APP_CONCEPT"
#define EVENT_DTMF_DECODE "DTMF_DECODE"


#define STREAM_STATUS_NOT_READY		     0 //must call LV_SRE_StreamStart function to set up new stream
#define STREAM_STATUS_READY			     1 //port can accept streamed data
#define STREAM_STATUS_BARGE_IN		     2 //barge-in detected, now saving streamed data (user code can stop playback of audio data)
#define STREAM_STATUS_END_SPEECH	     3 //when returned , new stream data no longer accepted until one of the StartStream functions called
#define STREAM_STATUS_STOPPED		     4 //StreamStop was called, 
#define STREAM_STATUS_BARGE_IN_TIMEOUT   5 //barge-in not triggered before timeout, no audio will be saved.
#define STREAM_STATUS_END_SPEECH_TIMEOUT 6 //end-speech not detected.  call StreamStop to force a decode if in AutoDecode mode.

//BEGIN stream parameters used by LV_SRE_StreamSetParameter(), LV_SRE_StreamGetParameter() and LV_SRE_StreamSetParameterToDefault

#define STREAM_PARM_SOUND_FORMAT			1 //specifies sound data type stream handles - uses SOUND_FORMAT enum - default is ULAW_8KHZ

#define STREAM_PARM_VOICE_CHANNEL			2 //voice channel to load sound data to on stream completion
#define STREAM_PARM_GRAMMAR_SET				3 //grammar set to use with auto decode streams
#define STREAM_PARM_DECODE_FLAGS			4 //decode flags to send with auto decode streams
#define STREAM_PARM_USE_COMPRESSION			5 //use compression internally for sound data - default = 1
#define STREAM_PARM_STORE_PROMPT			6 //save streamed prompt data in log file - default = 0 


//BEGIN  stream type parameters  0 = false, 1 = true	//////////////////////////////////////////////////////////////

//if true will discard stream data until barge-in detected
#define STREAM_PARM_DETECT_BARGE_IN			101			

//if true will stop excepting stream data on end-of-speech detect and change stream status to STREAM_STATUS_END_SPEECH
//  if auto_decode true, will immediatly begin decoding
#define STREAM_PARM_DETECT_END_OF_SPEECH	102			

//if true will start decode immeditaly on end-of-speech detection or a call to StopStream()
#define STREAM_PARM_AUTO_DECODE				103

// amount of audio in milliseconds that can be fed without barge-in occuring, before the state of
// the stream changes to STREAM_STATUS_BARGE_IN_TIMEOUT
#define STREAM_PARM_BARGE_IN_TIMEOUT        104

// amount of recorded audio (post-bargein) in milliseconde that can be fed without end-of-speech 
// triggering, before the stream state changes to STREAM_STATUS_END_SPEECH_TIMEOUT
#define STREAM_PARM_END_OF_SPEECH_TIMEOUT   105

//END    stream type parameters 



//BEGIN barge-in adjustment values
#define STREAM_PARM_BARGE_IN_BEGIN_DELAY	201		//number of .1 seconds at beggining of stream to limit barge-in
#define STREAM_PARM_BARGE_IN_NOISE_COUNT_LOW_THRESHOLD 202 //strength of signal to trigger barge-in  (default is 75)
#define STREAM_PARM_BARGE_IN_DYNAMIC_ADJUST 203 //if value set to 1 will dynamically adjust barge-in threshold 
//END   barge-in adjustment values


//BEGIN end-of-speech detection types

//used to tune end-of-speech detection , use STREAM_END_OF_SPEECH_DETECTION_ value from below
//	this parameter is not activated until StartStream() is called (changing it on the fly has no effect)
#define STREAM_PARM_END_OF_SPEECH_DETECTION	210		
	
	//speech sent to SRE is a mix of single words and longer utterences (default setting)
	#define STREAM_END_OF_SPEECH_DETECTION_NORMAL				0  
	
	//speech is just single words, end-of-speech detection is faster 
	//	(SRE does not need to wait for more words to follow)
	#define STREAM_END_OF_SPEECH_DETECTION_SINGLE_WORDS			1 
	
	//speech is phrases or sentences, with potential pauses by speaker
	//	(SRE will wait slightly longer before determining end-of-speech)
	#define STREAM_END_OF_SPEECH_DETECTION_PHRASES_WITH_PAUSES	2 
	
	//speech is phrases or sentences, no pauses by speaker
	#define STREAM_END_OF_SPEECH_DETECTION_PHRASES_NO_PAUSES	3 

//END   end-of-speech detection types

//these are parameters for noise reduction and frequency domain VAD
#define STREAM_PARM_NOISE_REDUCTION	220
#define STREAM_PARM_USE_FREQ_VAD		221
#define STREAM_PARM_VAD_BARGEIN_LVL	222
#define STREAM_PARM_VAD_EOS_DELAY	223
#define STREAM_PARM_VAD_NOISE_FLOOR 224
#define STREAM_PARM_VAD_P2A_THLD	225
#define STREAM_PARM_VAD_WIND_BACK	226
#define STREAM_PARM_VAD_SAVE_ALL_AUDIO	227
#define STREAM_PARM_VAD_BURST_THLD	228
#define STREAM_PARM_VAD_INIT_TIME	229
#define STREAM_PARM_VAD_USE_ZERO_CROSSING	230
#define STREAM_PARM_VAD_USE_BAND_PASS		231

typedef void (*ExportLogMsg)(const char* String, void* p);

//*********************************************************************
//prevent name-mangling
#if defined(__cplusplus)
extern "C"
{
#endif

//optional system startup and shutdown calls.
LV_SRE_API void LV_SRE_Startup();
LV_SRE_API void LV_SRE_Shutdown();

LV_SRE_API HPORT LV_SRE_OpenPort(ExportLogMsg log,void *p,int verbosity);
LV_SRE_API int LV_SRE_ClosePort(HPORT hport);
LV_SRE_API int LV_SRE_LoadVoiceChannel(HPORT hport,int VoiceChannel,void* M,int Length,
                                       SOUND_FORMAT Format);//,const char* SoundFileName);
LV_SRE_API int LV_SRE_Decode(HPORT hport,int VoiceChannel,int GrammarSet,unsigned int flags);

LV_SRE_API int LV_SRE_GetNumberOfConceptsReturned(HPORT hport,int VoiceChannel);
LV_SRE_API const char* LV_SRE_GetConcept(HPORT hport,int VoiceChannel, int Index);
LV_SRE_API const char* LV_SRE_GetPhraseDecoded(HPORT hport,int VoiceChannel, int Index);
LV_SRE_API const char* LV_SRE_GetRawTextDecoded(HPORT hport,int VoiceChannel, int Index);

//deprecated; use LV_SRE_GetPhonemesDecoded
LV_SRE_API const char* LV_SRE_GetPhonemes(HPORT hport,int VoiceChannel, int Index);
LV_SRE_API const char* LV_SRE_GetPhonemesDecoded(HPORT hport, int VoiceChannel, int Index);
LV_SRE_API int LV_SRE_GetConceptScore(HPORT hport,int VoiceChannel, int Index);
LV_SRE_API int LV_SRE_AddPhrase(HPORT hport,int GrammarSet, const char* Concept, 
                                const char* Phrase);
LV_SRE_API int LV_SRE_LoadStandardGrammar(HPORT hport,int GrammarSet, int DefaultGrammar);
LV_SRE_API int LV_SRE_RemoveConcept(HPORT hport,int GrammarSet, const char* Concept);
LV_SRE_API int LV_SRE_ResetGrammar(HPORT hport,int GrammarSet);
LV_SRE_API int	LV_SRE_GetVoiceChannelData(HPORT hport,int VoiceChannel, short** PCM,  
                                           unsigned int* Samples);
//LV_SRE_API int LV_SRE_ActivateGrammar(HPORT hport,int GrammarSet, int UseOOV);
LV_SRE_API const char* LV_SRE_ReturnErrorString(int ReturnCode);

//note!! API param change, this function is deprecated in favor of LV_SRE_SetPropertyEx
//LV_SRE_API int LV_SRE_SetProperty(HPORT hport,PROPERTIES Property, int Value);
LV_SRE_API int LV_SRE_SetProperty(HPORT hport,int property,int value);

LV_SRE_API void LV_SRE_RegisterAppLogMsg(ExportLogMsg Log,void* p,int NewMsgVerbosity);

LV_SRE_API int LV_SRE_WaitForEngineToIdle(HPORT hport,int voicechannel,int ms);

#define LV_LOG_ANSWER 0
#define LV_LOG_REQUEST 1
LV_SRE_API int LV_SRE_GetLogFileName(HPORT hport,int voicechannel, char *FileName,int* FileNameStringSize);

LV_SRE_API int LV_SRE_SetPropertyEx(HPORT hport,int property,int valuetype,void *pvalue,
                                    int target,int ndx);

LV_SRE_API int LV_SRE_AddEvent(HPORT hport, const char* eventName);
LV_SRE_API int LV_SRE_AddEventEx(HPORT hport, const char* eventName, const char* eventData);

//example to set language for port
//LV_SRE_SetPropertyEx(HPort,
//                     PROP_EX_LANGUAGE,
//                     PROP_EX_VALUE_TYPE_STRING,//were passing in a null terminated string
//                     (void *)PSZLangStr, //ie., "AmericanEnglish" or "Spanish", etc.
//                     PROP_EX_TARGET_PORT,//modify the port's property
//                     0);

LV_SRE_API HPORT LV_SRE_OpenPort2(long* ErrorReturnCode, ExportLogMsg log,void *p,long verbosity);

LV_SRE_API int LV_SRE_GetDecodeMS(HPORT hport,int voicechannel);
LV_SRE_API int LV_SRE_GetTotalDecodeMS(HPORT hport,int voicechannel);

LV_SRE_API int LV_SRE_IsAcousticModelAvail(const char *pszmodel);//returns LV_SUCCESS|LV_FAILURE
LV_SRE_API int LV_SRE_GetDecodeAcousticModel(HPORT hport,int vc,char *pzmodel,int buffersize);

LV_SRE_API int LV_SRE_AddFieldToRequest(HPORT hport, const char* KeyName, const char* String);
LV_SRE_API int LV_SRE_ClearAllFieldsToRequest(HPORT hport);


//END stream parameters



//sets up new stream
LV_SRE_API int LV_SRE_StreamStart(HPORT hport); 

//call with each buffer of sound data
LV_SRE_API int LV_SRE_StreamSendData(HPORT hport, void* SoundData, int SoundDataLength);

//call with each buffer of sound data of the prompt (playing in parralell to recorded sound data)
LV_SRE_API int LV_SRE_StreamSendPromptData(HPORT hport, void* SoundData, int SoundDataLength);

//return status of stream (STREAM_STATUS_NOT_READY, STREAM_STATUS_READY, etc)
LV_SRE_API int LV_SRE_StreamGetStatus(HPORT hport);
  
//returns number of bytes of retained stream (does not count bytes before barge-in detection)
LV_SRE_API int LV_SRE_StreamGetLength(HPORT hport);


typedef void LV_SRE_StreamStateChangeFn(long NewState, unsigned long TotalBytes, unsigned long RecordedBytes, void* UserData);
//specifies a call-back for stream state changes
LV_SRE_API int LV_SRE_StreamSetStateChangeCallBack(HPORT hport, LV_SRE_StreamStateChangeFn* fn, void* UserData); 


//stops accepting stream data and loads sound channel(specified in parameters) with streamed data
LV_SRE_API int LV_SRE_StreamStop(HPORT hport);  

//stops accepting stream data, discards stream data 
LV_SRE_API int LV_SRE_StreamCancel(HPORT hport);  


LV_SRE_API int LV_SRE_StreamSetParameter(HPORT hport, int StreamParameter, unsigned long  StreamParameterValue); 
LV_SRE_API int LV_SRE_StreamGetParameter(HPORT hport, int StreamParameter, unsigned long* StreamParameterValue);
LV_SRE_API int LV_SRE_StreamSetParameterToDefault(HPORT hport, int StreamParameter);




//older version had this define with too many underscores
//this is here so old
#define STREAM__END_OF_SPEECH_DETECTION_PHRASES_NO_PAUSES	3 


/*
 * backward compatibility function, for controlling how a concept-phrase grammar gets implemented as an
 * SRGS grammar, by controlling how many concepts can be detected in an utterance:
 * Equivalent to the following:
 *		root $rootrule;
 *		$rootrule = ($concept1 | $concept2 | .... $conceptN)<from-to>;
 * If you set "to" == -1, then it will accept at least as many concepts as "from"
 * Equivalent to the following:
 *		root $rootrule;
 *		$rootrule = ($concept1 | $concept2 | .... $conceptN)<from->;
 *
 * The default settings are from=1, to=-1, so that the SRE is told to recognize one or more concepts.
 *
 * This function is only for use in grammars that are built using the old api 
 * (AddPhrase, RemoveConcept, ResetGrammar)
 * Also, you must put LV_DECODE_SRGS_GRAMMAR in your decode flags.
 */
LV_SRE_API int LV_SRE_SetConceptRepetition(HPORT hport, int GrammarSet, int from, int to);


/******************************************************************************************************************
 *
 *  Functions for loading and unloading grammars into the speechport.
 *
 ******************************************************************************************************************/
 

/*
 * Load an SRGS grammar file by its location. 
 * The grammars name is the uri (or the index value if you use the Idx version of the call)
 *
 * Input: hport		The speech port handle.
 * Input: uri		The location of the grammar file.
 */
LV_SRE_API int LV_SRE_LoadGrammar(HPORT hport, const char* label, const char* uri);
LV_SRE_API int LV_SRE_LoadGrammarIdx(HPORT hport, int index, const char* uri);

/*
 * Load an SRGS grammar from a null terminated string. 
 * The grammar's name is the uri (or the index value if you use the Idx version of the call)
 *
 * Input: hport		The speech port handle.
 * Input: gram		The name of the grammar. Can be an alpha-numeric string, or a uri (e.g. mygrammar://doo/dah) 
 *                  (or an integer, if you use the Idx form of the call)
 */
LV_SRE_API int LV_SRE_LoadGrammarFromBuffer(HPORT hport, const char* gram, const char* buffer);
LV_SRE_API int LV_SRE_LoadGrammarFromBufferIdx(HPORT hport, int gram, const char* buffer);

/*
 * Load an SRGS grammar from a grammar object.  Grammar objects allow the user to both load a grammar
 * from file, and then modify that grammar programatically.
 *
 * Input: hport		The speech port handle.
 * Input: grammar	The grammar handle.
 */
LV_SRE_API int LV_SRE_LoadGrammarFromObject(HPORT hport, const char* gram, HGRAMMAR grammar);
LV_SRE_API int LV_SRE_LoadGrammarFromObjectIdx(HPORT hport, int index, HGRAMMAR grammar);

/*
 * Unload a grammar from the Speech Port.
 *
 * Input: hport
 * Input: name/index	The name (index if you use the Idx call)
 *
 */
LV_SRE_API int LV_SRE_UnloadGrammar(HPORT hport, const char* name);
LV_SRE_API int LV_SRE_UnloadGrammarIdx(HPORT hport, int index);



 






/************************************************************************************************************
 *
 *	Functions for loading application-level grammars that may be used by all of your speechports.
 *                                                              
 ************************************************************************************************************/

/*
 * Load an SRGS grammar file into the grammar manager.
 *
 * Input: label The name you wish to give the grammar.
 * Input: uri	The location of the grammar you wish to load.
 * 
 */
LV_SRE_API int LV_SRE_LoadGlobalGrammar(const char* label, const char* uri);

/*
 * Load an SRGS grammar file into the grammar manager from a string containing the file contents.
 *
 * Input: label			The name the grammar will be given.  The label should be a uri(but it need not
 *						resolve to any location, or use any known protocol), or an alphanumeric string.
 *
 * Input: file_buffer	The contents of the grammar, as a null-terminated string.
 */
LV_SRE_API int LV_SRE_LoadGlobalGrammarFromBuffer(const char* label, const char* file_buffer);


LV_SRE_API int LV_SRE_LoadGlobalGrammarFromObject(const char* label, HGRAMMAR grammar);


/*
 * Unload an SRGS grammar file from grammar manager.
 *
 * Input: grammar_name	the location of the grammar when it was loaded into the manager.
 */
LV_SRE_API int LV_SRE_UnloadGlobalGrammar(const char* grammar_name);

LV_SRE_API int LV_SRE_SetBuiltinGrammarURI(const char* Name, bool DTMF, const char* URI);

LV_SRE_API int LV_SRE_IsGlobalGrammarLoaded(const char* grammar_name);

LV_SRE_API void LV_SRE_UnloadGlobalGrammars();

LV_SRE_API int LV_SRE_UnloadGrammars(HPORT hport);

LV_SRE_API int LV_SRE_IsGrammarLoaded(HPORT hport, const char* name);

LV_SRE_API int LV_SRE_IsGrammarLoadedIdx(HPORT hport, int index);



/************************************************************************************************************
 *
 *	Functions for Activating/Deactivating grammars for use in a decode.                                                               
 *                                                              
 ************************************************************************************************************/


/*
 * Add an active grammar to be used in the next decode.  The grammar is retrieved from the speechport's 
 * set of grammars.
 * 
 * Input: hport			The speechport that will use this active grammar.
 * Input: gram	        the name of the grammar to be activated.
 */
LV_SRE_API int LV_SRE_ActivateGrammar(HPORT hport, const char* gram);
LV_SRE_API int LV_SRE_ActivateGrammarIdx(HPORT hport, int gram);

/*
 * This activate function is only for specifying a global grammar when a local grammar of the 
 * same name is present in your list of grammars.  Otherwise, just use ActivateGrammar
 * 
 * Input: hport			The speechport that will use this active grammar set.
 * Input: uri			the uri descriptor of the grammar to be activated.
 */
LV_SRE_API int LV_SRE_ActivateGlobalGrammar(HPORT hport, const char* uri);

LV_SRE_API int LV_SRE_DeactivateGrammar(HPORT hport, const char* gram);
LV_SRE_API int LV_SRE_DeactivateGrammarIdx(HPORT hport, int gram);

/* Deprecated:  Just use LV_SRE_DeactivateGrammar */
LV_SRE_API int LV_SRE_DeactivateGlobalGrammar(HPORT hport, const char* gram);

/*
 * Clears out the active grammar set.
 * 
 * Input: hport			The speechport that owns the active grammar set.
 */
LV_SRE_API int LV_SRE_DeactivateGrammars(HPORT hport);


/************************************************************************************************************
 *
 *	Alternate Decode function, flags, etc.
 *
 ************************************************************************************************************/

/*
 *	Decode Flags (defined in LV_SRE.h):
 *
 *		LV_DECODE_SRGS_GRAMMAR		
 *		Tells the decode to use Lumenvox's implementation of SRGS grammars.  The return value of
 *		an SRGS decode is a parse string and parse tree compatible with the suggested return type laid out
 *		by the W3Cs Speech Recognition Grammar Specification.
 *
 *		LV_DECODE_SEMANTIC_INTERPRETATION
 *		If this is set, The speechport will use the parse tree and tag information in the tree to return
 *		an XML string compatible with W3Cs Semantic Interpretation working draft.  In the future, it will
 *		also return a C data structure representation of the semantic interpretation tree, to make it 
 *		easier to transform the semantic interpretation into results like ECMAScript source, EMMA, NLSML, etc.
 *
 *	Decode variables:
 *
 *		LV_ACTIVE_GRAMMAR_SET
 *		During the call to LV_SRE_Decode, if this variable is set for the "grammarset" variable, then your
 *		set of active grammars will be used.  Otherwise, only one of your speechports grammars will be used.
 *
 *
 *	e.g. decoding with all the bells and whistles:
 *		 LV_SRE_Decode(port, voicechannel, LV_ACTIVE_GRAMMAR_SET, LV_DECODE_SRGS_GRAMMAR | 
 *		                                                          LV_DECODE_SEMANTIC_INTERPRENTATION);
 */


/* Uses the active grammars that are dtmf grammars to decode your dtmf string.
 * Your decode results are stored in voicechannel=LV_DTMF_CHANNEL, so you can call:
 *
 * LV_SRE_GetNumberOfInterpretations(hport,LV_DTMF_CHANNEL);
 * LV_SRE_GetInterpretationString(hport,LV_DTMF_CHANNEL,index); 
 * 
 * etc.
 */
LV_SRE_API int LV_SRE_DTMFDecode(HPORT hport, const char* dtmf_str, unsigned int flags);




/************************************************************************************************************
 *
 *	functions for retrieving information after a decode that used an SRGS grammar.
 *
 ************************************************************************************************************/

/* 
 * returns the number of parses for the SRE output.
 * If the value is zero, then the decode didn't match the active grammar(s).
 * If the value is greater than one, then the active grammar(s) were ambiguous for the decode.
 */
LV_SRE_API int LV_SRE_GetNumberOfParses(HPORT hport, int VoiceChannel);

/*
 * Returns an utterance score for the decoded audio.  Higher scores are more likely to be correct than
 * lower scores.  The range of the score is 0 - 1000.
 *
 * Input: hport			The speechport used in the decode.
 * Input: VoiceChannel	The voice channel used in the decode.
 */
LV_SRE_API int LV_SRE_GetUtteranceScore(HPORT hport, int VoiceChannel);



/*
 * Returns a logical parse string for the decode.
 *
 * See "http://www.w3.org/TR/speech-grammar/#AppH"
 *
 * Input: hport			The speechport used in the decode.
 * Input: VoiceChannel	The voice channel used in the decode.
 * Input: Index			A value from 0 to GetNumberOfParses - 1, indexing the parse trees for the decode.
 */
LV_SRE_API const char* LV_SRE_GetParseTreeString(HPORT hport, int VoiceChannel, int Index);

/*
 * Logically equivalent to GetParseString, A parse tree handle allows traversal of the parse tree, 
 * letting the user see word confidence scores, start and end frames for each word, 
 * and phonetic pronunciations.
 *
 * See <LV_SRE_ParseTree.h>
 *
 * Input: hport			The speechport used in the decode.
 * Input: VoiceChannel	The voice channel used in the decode.
 * Input: Index			A value from 0 to GetNumberOfParses - 1
 */

LV_SRE_API H_PARSE_TREE LV_SRE_CreateParseTree(HPORT hport, int VoiceChannel, int Index);

/*DEPRECATED - Use LV_SRE_GetParseTree*/
LV_SRE_API H_PARSE_TREE LV_SRE_GetParseTreeHandle(HPORT hport, int VoiceChannel, int Index);

/* 
 * returns the number of interpretation for the SRE output.
 */
LV_SRE_API int LV_SRE_GetNumberOfInterpretations(HPORT hport, int VoiceChannel);


/* Creates a handle to interpretation structure.
 * When you are finished with it, release it with LVInterpretation_Release
 * contains: 
 *		language identifier
 *		interaction mode
 *		semantic process identifier (tag format)
 *      input sentence
 *      confidence score
 *      interpretation result (the semantic data structure)
 *      interpretation name (usually the name of the top level rule in the grammar that generated the result)
 *
 * Input:   hport			The port used in the decode.
 *			voicechannel    The audio (or dtmf) channel used during decode.
 *			index			The index of the interpretation. 
 *							Ranges from 0 through LV_SRE_GetNumberOfInterpretations - 1
 *
 * Note:  The flag LV_DECODE_SEMANTIC_INTERPRETATION must be set if you want to recieve 
 *        semantic interpretation results.
 */
LV_SRE_API H_SI LV_SRE_CreateInterpretation(HPORT hport, int voicechannel, int index);


/*
 * Get semantic interpretation result in XML format
 * See "http://www.w3.org/TR/semantic-interpretation/#SI7."
 *
 * Input:	hport			The port used in the decode.
 *			voicechannel	The audio channel that the port used during decode.
 *			index			The index of the interpretation.  
 *							Ranges from 0 through LV_SRE_GetNumberOfInterpretations - 1
 *
 * Note:  The flag LV_DECODE_SEMANTIC_INTERPRETATION must be set if you want to recieve 
 *        semantic interpretation results.
 *      
 */
LV_SRE_API const char* LV_SRE_GetInterpretationString(HPORT hport, int VoiceChannel, int Index);


/* DEPRECATED --- Use LV_SRE_CreateInterpretation
 * Get a handle to Semantic Data. 
 *
 * Input: hport			The port used in the decode.
 *        voicechannel	The audio channel that the port used during decode.
 *		  index			The index of the interpretation.  There is one semantic interpretation result
 *                      per parse tree, so the index ranges from 
 *					    0 to LV_SRE_GetNumberOfInterpretations - 1 
 *
 * Note:  The flag LV_DECODE_SEMANTIC_INTERPRETATION must be set if you want to recieve 
 *        semantic interpretation data.      
 */
LV_SRE_API H_SI_DATA LV_SRE_GetInterpretationData(HPORT hport, int VoiceChannel, int Index);

/* DEPRECATED --- LV_SRE_CreateInterpretation
 * Get the grammar label associated with a Semantic Data. 
 *
 * Input: hport			The port used in the decode.
 *        voicechannel	The audio channel that the port used during decode.
 *		  index			The index of the interpretation.  There is one semantic interpretation result
 *                      per parse tree, so the index ranges from 
 *					    0 to LV_SRE_GetNumberOfInterpretations - 1      
 */
LV_SRE_API const char* LV_SRE_GetInterpretationGrammarLabel(HPORT hport, int VoiceChannel, int Index);

/* DEPRECATED --- Use LV_SRE_CreateInterpretation
 * Get the language code of the grammar associated with a Semantic Data. 
 *
 * Input: hport			The port used in the decode.
 *        voicechannel	The audio channel that the port used during decode.
 *		  index			The index of the interpretation.  There is one semantic interpretation result
 *                      per parse tree, so the index ranges from 
 *					    0 to LV_SRE_GetNumberOfInterpretations - 1      
 */
LV_SRE_API const char* LV_SRE_GetInterpretationLanguage(HPORT hport, int VoiceChannel, int Index);

/* DEPRECATED --- LV_SRE_CreateInterpretation
 * Get the mode of the grammar associated with a Semantic Data. 
 *
 * Input: hport			The port used in the decode.
 *        voicechannel	The audio channel that the port used during decode.
 *		  index			The index of the interpretation.  There is one semantic interpretation result
 *                      per parse tree, so the index ranges from 
 *					    0 to LV_SRE_GetNumberOfInterpretations - 1      
 */
LV_SRE_API const char* LV_SRE_GetInterpretationMode(HPORT hport, int VoiceChannel, int Index);

/* DEPRECATED --- Use LV_SRE_CreateInterpretation */
LV_SRE_API const char* LV_SRE_GetInterpretationTopLevelRule(HPORT hport, int VoiceChannel, int Index);
/* DEPRECATED --- Use LV_SRE_CreateInterpretation */
LV_SRE_API const char* LV_SRE_GetInterpretationInputString(HPORT hport, int VoiceChannel, int Index);
/* DEPRECATED --- Use LV_SRE_CreateInterpretation */
LV_SRE_API int LV_SRE_GetInterpretationScore(HPORT hport, int VoiceChannel, int Index);



#if defined(__cplusplus)
}
#endif
//*********************************************************************
#endif
