/*********************************************************************
 *        FILE: prdslib.h
 * DESCRIPTION: Header File for DIALOGIC PRDS Library for Windows-NT
 *
 *   Copyright (c) 1997 Dialogic Corp. All Rights Reserved
 *
 *   THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Dialogic Corp.
 *   The copyright notice above does not evidence any actual or
 *   intended publication of such source code.
 ********************************************************************/

#ifndef __PRDSLIB_H__
#define __PRDSLIB_H__

//
// Definitions
//
#define MAXFIRMWAREBUFFERS   32
#define BUFFER_NOT_VALID     0xFFFF

//
// Buffer Header Structure Definition
// Header is followed by contiguous Firmware buffers
//
//
typedef struct BulkDataSharedBuffer_tag {
USHORT  SizeOfHeader;
USHORT  NumberOfFirmwareBuffers;
USHORT  FirmwareBufferSize;
USHORT  NextBufferToWriteTo;
USHORT  NextBufferToReadFrom;
USHORT  BufferNotReadyCount;
PCHAR   ApplBufferStartAddress;
PCHAR   DriverBufferStartAddress;
ULONG   TotalSize;
ULONG   TransferredByteCount;
ULONG   TotalByteCount;
USHORT  BufferCountStatus[MAXFIRMWAREBUFFERS];
USHORT  BufferOffset[MAXFIRMWAREBUFFERS];
} BulkDataSharedBuffer, *PBulkDataSharedBuffer;


//
// Host Side functions - Buffer Allocation
//
PBulkDataSharedBuffer AllocateBulkDataBuffer(
                                  USHORT NumberOfFirmwareBuffers,
                                  USHORT FirmwareBufferSize);

VOID FreeBulkDataBuffer( PBulkDataSharedBuffer pBulkDataBuffer );

VOID ClearBulkDataBuffer( PBulkDataSharedBuffer pBulkDataBuffer );


//
// Host Side functions - Play buffer manipulation
//
PCHAR GetNextPlayBuffer( PBulkDataSharedBuffer pBulkDataBuffer );

INT SetPlayBufferCountStatus( PBulkDataSharedBuffer pBulkDataBuffer,
                              USHORT BufferCountStatus );

ULONG GetNonTransferredPlayCount( PBulkDataSharedBuffer pBulkDataBuffer );

ULONG GetTransferredPlayCount( PBulkDataSharedBuffer pBulkDataBuffer );

INT StartPlayingFromBuffer( INT hDevice, PBulkDataSharedBuffer pBulkDataBuffer );

//
// Host Side functions - Record buffer manipulation
//
PCHAR GetNextRecordBuffer( PBulkDataSharedBuffer pBulkDataBuffer );

INT GetRecordBufferCountStatus( PBulkDataSharedBuffer pBulkDataBuffer,
                                USHORT * pBufferCountStatus );

INT SetRecordBufferCountStatus( PBulkDataSharedBuffer pBulkDataBuffer,
                                USHORT BufferCountStatus );

INT StartRecordingToBuffer( INT hDevice, PBulkDataSharedBuffer pBulkDataBuffer );

ULONG GetNonTransferredRecordCount( PBulkDataSharedBuffer pBulkDataBuffer );

ULONG GetBufferNotReadyCount( PBulkDataSharedBuffer pBulkDataBuffer );

#endif

