//
//  Copyright (c) 1999-2007 by AudioCodes USA, Inc.
//  All rights reserved.
//
//  NTiEvent.h
//      SmartWORKS API event definition file
//
//  AUDIOCODES USA, INC. PROPRIETARY.
//

#ifndef _INC_NTIEVENT_H
#define _INC_NTIEVENT_H

//
// event structure of MTAPI
//

#define MTDSP_EVENT         MT_EVENT        // NTi Events
#define PMTDSP_EVENT        PMT_EVENT

#define MIN_XTRABUFFER_LENGTH               512


typedef struct _MT_EVENT {
#if _WIN32
    LARGE_INTEGER   TimeStamp;              // In FILETIME format
#elif LINUX
    struct timeval  TimeStamp;              // In gettimeofday() format
#endif
    ULONG           UserStatus;             // Not used by API
    ULONG           EventCode;              // Event Code
    ULONG           SubReason;              // Event sub-reason
    ULONG           XtraInfo;               // Extra information, e.g. termination
    ULONG           FuncCode;               // Function active when this event occurred
    USHORT          Board;                  // Board ID
    USHORT          Channel;                // Global Channel ID (GCI)
    PVOID           ptrBuffer;              // Related play/record buffer pointer
    ULONG           DataLength;             // Byte length of data accessed (played/recorded)

    PVOID           ptrXtraBuffer;          // Pointer to xtra buffer
    ULONG           XtraBufferLength;       // Length of buffer pointed by ptrXtraBuffer
    ULONG           XtraDataLength;         // Length of data in buffer pointed by ptrXtraBuffer
    ULONG           EventFlag;              // Falgs of the following:
                                            // bit 0x00000001: 1 - Appl created the event
                                            //                 0 - NTi DLL created the event
                                            // bit 0x00000002: 1 - Appl allocated ptrtraBuffer
                                            //                 0 - NTi DLL allocated ptrXtraBuffer
} MT_EVENT, *PMT_EVENT;


//---------------------------------------------------------------------------
// Bit definitions for field MT_EVENT.EventFlag
//---------------------------------------------------------------------------
#define APPL_EVENT              0x00000001
#define APPL_XTRABUFFER         0x00000002
#define XTRADATA_TRUNCATED      0x00000004


#if 0   // Obsoleted
//
// Event types for MT_EVENT "UserEvent" when MTWaitForVoiceEvent is used
// with a channel which is associated with an event queue.
//

#define MT_EVENTSRC_MTDSP   0x0002          // MTDSP driver event source
#define MT_EVENTSRC_USER    0x00f0

#ifndef NO_COMPAT_CODES
#define SRC_MT800           MT_EVENTSRC_MTDSP   // historical
#define SRC_USER1           0x000000f0          // historical
#endif

#endif


//-----------------------------------------------------------------------------
//                Event type codes of MTAPI
//-----------------------------------------------------------------------------

//
// Regular events:
//
#define EVT_MAXDIGITS           1   // Maximum DTMF digits received
#define EVT_TERMDIGIT           2   // Terminating DTMF digit received
#define EVT_STOP                3   // User stopped
#define EVT_SYS_ERROR           4   // System reponse error
#define EVT_MAXBYTES            5   // Max bytes reached on play or record
#define EVT_ERROR               6   // Hardware failure
#define EVT_MAXTIME             7   // Max time reached
#define EVT_OFFHOOK             8   // Offhook complete
#define EVT_DIALED              9   // Dialing complete
#define EVT_MAX_SILENCE         10  // Maximum silence received
#define EVT_EOF                 11  // EOF reached on playback
#define EVT_LOOP_STOP           12  // Terminate by drop in loop current
#define EVT_DISK_FULL           13  // Disk full
#define EVT_ONHOOK              14  // Onhook complete
#define EVT_DISK_WRITE_ERROR    15  // File write error
#define EVT_BOARD_PANIC_ERROR   16  // Board Panic error
#define EVT_TERMSILENCE         17  // Termination silence reached on play or record

#define EVT_CPM_STOP            18  // Call analysis termination
#define EVT_LOOP_REVERSE        19  // Loop current battery reversal
#define EVT_LOOP_DROP           20  // Loop current dropped
#define EVT_RINGS               21  // Ring received
#define EVT_MON_ACTIVITY        22  // Silence off, activity on
#define EVT_MON_SILENCE         23  // Silence on, activity off

#define EVT_IDLE                24  // Channel back in IDLE state
#define EVT_FRAME_HEADER_ERROR  25  // Frame header error, function terminated

#define EVT_LOOP_ON             26  // Loop current on
#define EVT_FRAME_DATA_ERROR    27  // Frame data error, playback function terminated
#define EVT_LVOLTAGE_REVERSAL   28  // Loop voltage reversal
#define EVT_DIGITENABLE         29  // Terminated by enabled DTMF digit
#define EVT_INTERDIGIT          30  // Interdigit delay exceeded
#define EVT_MAX_ACTIVITY        31  // Terminated by a max non-silence
#define EVT_LVOLTAGE_MIDDLE     32  // Loop voltage between ThresholdHigh and ThresholdLow
#define EVT_LVOLTAGE_ABOVEORBELOW 33 // Loop voltage not in MIDDLE state
#define EVT_TERMLVOLTAGE_ABOVEORBELOW 34  // Termination on loop voltage state not MIDDLE
#define EVT_LVOLTAGE_OFFHOOK    EVT_LVOLTAGE_MIDDLE
#define EVT_LVOLTAGE_NOTOFFHOOK EVT_LVOLTAGE_ABOVEORBELOW
#define EVT_TERMLVOLTAGE_NOTOFFHOOK    EVT_TERMLVOLTAGE_ABOVEORBELOW
#define EVT_LVOLTAGE_ABOVE      35  // Loop voltage above the ThresholdHigh
#define EVT_LVOLTAGE_BELOW      36  // Loop voltage below the ThresholdLow
#define EVT_SETPAR_DONE         37  // Parameter updated
#define EVT_WINKDONE            38  // Wink protocol complete
#define EVT_DIGIT_DROPPED       39  // Received DTMF digit dropped due to queue full
#define EVT_DIGIT               40  // Received DTMF digit queued
#define EVT_UTONE_ON            41  // Tone on detected
#define EVT_UTONE_OFF           42  // Tone off detected
#define EVT_LCURRENT_REVERSAL   43  // Loop current reversal
#define EVT_UTONE_TON           44  // Terminated by Tone On
#define EVT_UTONE_TOFF          45  // Terminated by Tone Off
#define EVT_TONEPERR            47  // Tone generation template error
#define EVT_TONEP_DONE          48  // Tone generation complete
#define EVT_MFTONE_DROPPED      49  // Received MF(R2) tone dropped due to queue full
#define EVT_MFTONE              50  // Received MF(R2) tone queued
#define EVT_LVOLTAGE_RECORD     51  // Recording started on detection of loop voltage offhook
#define EVT_IO_COMPLETED        52  // IO complete, obsoleted
#define EVT_VOICE_RECORD        53  // Recording started on detection of activity/voice
#define EVT_LOOPON_RECORD       54  // Recording started on detection of loop current
#define EVT_STREAMIN_WATERMARK  55  // Data ENCODE streaming watermark reached
#define EVT_STREAMOUT_WATERMARK 56  // Data DECODE streaming watermark reached
#define EVT_STREAMIN_START      57  // Data ENCODE streaming started
#define EVT_STREAMOUT_START     58  // Data DECODE streaming started
#define EVT_STREAM_MAXSIL       59  // Obsoleted (Max silence during data streaming)
#define EVT_STREAMIN_STOP       60  // Data ENCODE streaming stopped
#define EVT_STREAMIN_COMP       61  // Obsoleted (Input streaming complete)
#define EVT_STREAMOUT_COMP      62  // Obsoleted (Output streaming complete)
#define EVT_STREAMIN_DROPPED    63  // Input streaming data dropped
#define EVT_STREAMOUT_STOP      64  // Data DECODE streaming stopped
#define EVT_STREAMOUT_EMPTY     65  // Output stream buffer empty
#define EVT_STREAMOUT_PAUSED    66  // Output streaming paused
#define EVT_STREAMOUT_RESUMED   67  // Output streaming resumed

#define EVT_INDEX_PLAYED        69  // Indexed byte length played

// EVT_FAX... are not supported in NTI product line
#define EVT_FAXRECVEOP          70  // Receive FAX page done
#define EVT_FAXSEND_DONE        71  // Send FAX page complete
#define EVT_FAXRECV_DONE        72  // Receive FAX document complete
#define EVT_FAX_ERROR           74  // FAX error
#define EVT_FAX_PHASEB_COMP     75  // Phase B complete
#define EVT_FAX_PHASED_COMP     76  // Phase D complete

#define EVT_DEVICE_WRITE_ERROR  77  // User device write error
#define EVT_DEVICE_READ_ERROR   78  // User device read error
#define EVT_DEVICE_SEEK_ERROR   79  // User device seek error

#define EVT_DSPRESET            80  // DSP status event
#define EVT_NOT_AVAILABLE       81  // Functionality not available or busy
#define EVT_NOT_SUPPORTED       82  // Functionality not supported
#define EVT_NO_EFFECT           83  // Functionality not necessary (already done)

#define EVT_LOSYNC_ALARM        84  // Loss Of Sync alarm
#define EVT_LOS_ALARM           85  // Loss Of Signal alarm
#define EVT_LOF_ALARM           86  // Loss Of Framer alarm
#define EVT_AIS_ALARM           87  // Alarm Indication alarm
#define EVT_YELLOW_ALARM        88  // Yellow Alarm
#define EVT_INCOMING_SIGCHANGE  89  // Change of Signaling Bits (ABCD) on incoming line
#define EVT_OUTGOING_SIGCHANGE  90  // Change of Signaling Bits (ABCD) on outgoing line

#define EVT_MVIP_ALARM          91  // MVIP error detected
#define EVT_MASTER_PLL_ALARM    92  // Master PLL error detected

#define EVT_LOSMF_ALARM         93  // E1: Loss Of Signaling Multiframe Alignment alarm
#define EVT_LOCRCMF_ALARM       94  // E1: Loss Of CRC Multiframe Alignment alarm
#define EVT_TS16RAI_ALARM       95  // E1: Timeslot 16 Remote Alarm Indication alarm

#if 0
#define EVT_RX_HDLC_ACTIVE      96  // ISDN Rx HDLC frame active
#define EVT_RX_HDLC_INACTIVE    97  // ISDN Rx HDLC frame inactive (abort)
#define EVT_RX_HDLC_OVERFLOW    98  // ISDN Rx HDLC frame overflow
#define EVT_RX_HDLC_VALID       99  // ISDN Rx HDLC frame valid
#define EVT_RX_HDLC_TOO_LONG    100 // ISDN Rx HDLC frame too long
#define EVT_RX_HDLC_INVALID     101 // ISDN Rx HDLC frame invalid
#endif

#define EVT_H100_MASTER_A_ALARM 102 // H100 Master A clock error detected
#define EVT_H100_MASTER_B_ALARM 103 // H100 Master B clock error detected

//
// Extended events:
//

// event code 104 - 128 are for CallControl events

//
// CPM monitor events (if enabled by MONI_... enable bits):
//
#define EVT_MON_BUSY1           129 // Monitored tone defined as CPM_BUSY1
#define EVT_MON_BUSY            EVT_MON_BUSY1 // Backward compatibility macro
#define EVT_MON_BUSY2           130 // Monitored tone defined as CPM_BUSY2
#define EVT_MON_BUSY_TRUNK      EVT_MON_BUSY2 // Backward compatibility macro
#define EVT_MON_SIT2            132 // Monitored tone defined as CPM_SIT2 (IC)
#define EVT_MON_SIT3            133 // Monitored tone defined as CPM_SIT3 (VC)
#define EVT_MON_SIT4            134 // Monitored tone defined as CPM_SIT4 (RO)
#define EVT_MON_SIT5            135 // Monitored tone defined as CPM_SIT5 (IO)
#define EVT_MON_DIAL1           136 // Monitored any tone defined as CPM_DIAL1
#define EVT_MON_DIAL            EVT_MON_DIAL1 // Backward compatibility macro
#define EVT_MON_DIAL2           137 // Monitored tone defined as CPM_DIAL2
#define EVT_MON_DIAL3           138 // Monitored tone defined as CPM_DIAL3
#define EVT_MON_CALLWAITING     140 // Monitored tone defined as CPM_CALLWAITING
#define EVT_MON_RECEIVEOFF      141 // Monitored tone defined as CPM_RECEIVEOFF
#define EVT_MON_RINGBACK1       142 // Monitored tone defined as CPM_RINGBACK1
#define EVT_MON_RINGBACK        EVT_MON_RINGBACK1 // Backward compatibility macro
#define EVT_MON_RINGBACK2       143 // Monitored tone defined as CPM_RINGBACK2
#define EVT_MON_DRINGBACK       EVT_MON_RINGBACK2 // Backward compatibility macro
#define EVT_MON_SIT1            144 // Monitored tone defined as CPM_SIT1 (NC)
#define EVT_MON_SIT             EVT_MON_SIT1      // Backward compatibility macro
#define EVT_MON_FAX1            145 // Monitored tone defined as CPM_FAX1
#define EVT_MON_FAX             EVT_MON_FAX1      // Backward compatibility macro
#define EVT_MON_REVERSAL        146 // Monitored polarity reversal, obsoleted
#define EVT_MON_VOICE           147 // Monitored voice (including human and machine)
#define EVT_MON_NOVOICE         148 // Monitored no voice
#define EVT_MON_MACHINE         149 // Monitored machine voice
#define EVT_MON_HUMAN           150 // Monitored human voice

#if 0   // Obsoleted
#define EVT_CALLID_START        150 // CallerID start detected, obsoleted
#endif
#define EVT_CALLID_DROPPED      151 // CallerID detected and dropped due to queue full
#define EVT_CALLID_STOP         152 // CallerID end detected
#if 0   // Obsoleted
#define EVT_CALLID_MSGLEN       153 // CallerID message length, obsoleted
#endif
#define EVT_CALLID_CSUMERR      154 // Indicates a checksum error in an 
                                    // incoming caller ID packet
#define EVT_RECOFFHOOK          159 // Recording offhoook
#define EVT_REVERSAL            160 // Loop current reversal

#define EVT_SIGNAL_CLIPPING     161 // Signal clipping
#if 0
#define EVT_FIFO_OVERFLOW       162 // FIFO overflow, to be removed
#define EVT_FIFO_ONE            163 // FIFO one, to be removed
#define EVT_FIFO_TWO            164 // FIFO two, to be removed
#define EVT_FIFO_THREE          165 // FIFO three, to be removed
#define EVT_PBX_DCH             166 // PBX
#define EVT_PHN_DCH             167 // PHONE
#endif

#define EVT_MON_SIGNAL_CYCLE    170 // Monitored CPM/User signal cycle
#define EVT_MON_FAX2            171 // Monitored tone defined as CPM_FAX2

// NetTAP event definitions
#define EVT_MEDIA_SESSION_STARTED     172 // An RTP stream was detected for a call session
#define EVT_MEDIA_SESSION_STOPPED     173 // An RTP stream has closed
#define EVT_STATION_ADDED             174 // A new station has been detected
#define EVT_STATION_REMOVED           175 // A station has been removed

#define EVT_AUX_MEDIA_SESSION_STARTED 176 // ??
#define EVT_AUX_MEDIA_SESSION_STOPPED 177 // ??

#define EVT_PHY_LINK_UP			      178  // An interface has shown up
#define EVT_PHY_LINK_DOWN		      179  // An interface has gone away
#define EVT_ETH_DEST_UNREACHABLE      181  // Destination has been unreachable
#define EVT_LICENSE_FILE_LOADED       182  // A license file has been loaded 
#define EVT_MEDIA_SESSION_QOS_REPORT  183 // Media Session QoS Report

// Dongle event definitions
#define EVT_DONGLE_ADDED              184 // A dongle has been detected
#define EVT_DONGLE_REMOVED            185 // A dongle has been removed

// Board Alarm event
#define EVT_BOARD_WATCHDOG_ALARM      186 // A board alarm event

// Mapped Station Events
#define EVT_MAPPED_STATION_DETECTED   187 // A new mapped station has been detected
#define EVT_MAPPED_STATION_TIMEOUT    188 // A mapped station has been removed due to timeout

// Licensing event definitions
#define EVT_QUOTA_EXCEEDED            180 // A licensed resource has been exhausted

// System event definition
#define EVT_SYS_SYNCTIME_OLD     200 // Resynchronization presented using time before resync
#define EVT_SYS_SYNCTIME_NEW     201 // Resynchronization presented using time after resync
#define EVT_SYS_BOARD_ADDED      202 // SmartWORKS board added
#define EVT_SYS_BOARD_REMOVED    203 // SmartWORKS board removed

#define EVT_SYS_QUEUE_OVERFLOW   204 // Generic Overflow Event

#define MAX_EVENT_CODE           204 // Max legal event code


// End of MTAPI event code definitions

//
// Call Progress termination types (for event EVT_CPM_STOP):
//   Enhanced and MTCallString() Specific Types
//

//
// Enhanced types (if enabled by TERM_... enable bits): (in MT_EVENT.SubReason)
//
#define CPM_MON_MAX_SILENCE      132    // Termination on max silence detection
#define CPM_MON_MAX_ACTIVITY     133    // Termination on max activity detection
#define CPM_MAX_SILENCE          CPM_MON_MAX_SILENCE
#define CPM_MAX_ACTIVITY         CPM_MON_MAX_ACTIVITY
#define CPM_MON_SILENCE          134    // Termination on silence detection
#define CPM_MON_ACTIVITY         135    // Termination on activity detection
#define CPM_MON_DIAL             136    // Termination on monitor ANY dial tone
#define CPM_MON_BUSY             137    // Termination on monitor ANY busy tone
#define CPM_MON_CONNECT          138
#define CPM_MON_VOICE            139    // Termination on monitor voice (human and machine inc.)
#define CPM_MON_MACHINE          140    // Termination on monitor answer machine intercept
#define CPM_MON_LOOP_DROP        141    // Termination on monitor loop current drop
#define CPM_MON_RINGS            142    // Termination on monitor rings
#define CPM_MON_LVOLTAGE_DROP    143    // Termination on monitor loop voltage not offhook
#define CPM_MON_HUMAN            144    // Termination on monitor human voice
#define CPM_MON_DIAL1            145    // Termination on monitor CPM_DIAL1 tone
#define CPM_MON_DIAL2            146    // Termination on monitor CPM_DIAL2 tone
#define CPM_MON_DIAL3            147    // Termination on monitor CPM_DIAL3 tone
#define CPM_MON_BUSY1            148    // Termination on monitor CPM_BUSY1 tone
#define CPM_MON_BUSY2            149    // Termination on monitor CPM_BUSY2 tone
#define CPM_MON_RINGBACK1        150    // Termination on monitor CPM_RINGBACK1 tone
#define CPM_MON_RINGBACK2        151    // Termination on monitor CPM_RINGBACK2 tone
#define CPM_MON_FAX1             152    // Termination on monitor CPM_FAX1 tone
#define CPM_MON_FAX2             153    // Termination on monitor CPM_FAX2 tone
#define CPM_MON_SIT1             154    // Termination on monitor CPM_SIT1 tone
#define CPM_MON_SIT2             155    // Termination on monitor CPM_SIT2 tone
#define CPM_MON_SIT3             156    // Termination on monitor CPM_SIT3 tone
#define CPM_MON_SIT4             157    // Termination on monitor CPM_SIT4 tone
#define CPM_MON_SIT5             158    // Termination on monitor CPM_SIT5 tone
#define CPM_MON_CALLWAITING      159    // Termination on monitor CPM_CALLWAITING tone
#define CPM_MON_RECEIVEOFF       160    // Termination on monitor CPM_RECEIVEOFF tone
#define CPM_MON_INTERCEPT        161    // Termination on monitor ANY CPM_SIT tone
#define CPM_MON_UTONE            162    // Termination on monitor USR_NONE tone


//
// MTCallString() Specific Termination types: (in MT_EVENT.SubReason)
//
#define CPM_NO_ANSWER              8    // No answer after number of rings or after timeout
#define CPM_NO_RINGBACK            9    // No ringback after timeout
#define CPM_CONNECT               10    // Call is answered
#define CPM_INTERCEPT             11    // ANY SIT tone detected
#define CPM_NO_DIAL               17    // No Dial tone after timeout



//
// MTCHAN_STATUS.CurrentEncFunction and MTCHAN_STATUS.CurrentDecFunction definition:
// MT_EVENT.FuncCode definition:
//
#define     ENCODE_BASE     0x1000      // Record function
#define     DECODE_BASE     0x2000      // Playback function
#define     BUFFER_BASE     0x0100      // Buffer I/O
#define     FILE_BASE       0x0200      // File I/O
#define     DEVICE_BASE     0x0400      // Device I/O
#define     STREAM_BASE     0x0800      // Streaming I/O

#define F_RECB          0x1100          // Record to buffer
#define F_RECF          0x1200          // Record to file
#define F_RECD          0x1400          // Record to device
#define F_RECS          0x1800          // Record to stream buffer
#define F_PLAYB         0x2100          // Playback from buffer
#define F_PLAYF         0x2200          // Playback from file
#define F_PLAYFI        0x2201          // Indexed Playback from file
#define F_PLAYD         0x2400          // Playback from device
#define F_PLAYS         0x2800          // Playback from stream buffer

#define F_RECS_START    0x1801          // Start record to stream buffer
#define F_PLAYS_START   0x2801          // Start playback from stream buffer

#define F_GETDTMFS      0x1001          // Get DTMF digits
#define F_PLAYDTMFS     0x2001          // Play DTMF digits in DialString
#define F_PLAYTONES     0x2002          // Play tones
#define F_CALLDTMFS     0x2004          // Play DTMF digits in CallString

#define F_IDLE          0x0000          // Idle

// Board Watchdog Alarm Conditions: (in MT_EVENT.SubReason)
#define WATCHDOG_ALARM_PROTOCOL_ENGINE_RESTARTED   0x1
#define WATCHDOG_ALARM_FW_COM_PROCESS_RESTARTED    0x2

#include "NTiEventCC.h"
// Currently, NTiEventCC uses events ranged from 104 to 128
#define CC_EVENT_BASE  104
#define CC_EVENT_MAX   128

#include "NTiEventDCC.h"
// Currently, NTiEventDCC uses events ranged from 0x1000 to 0x1fff
#define DCC_EVENT_BASE  0x1000
#define DCC_EVENT_MAX   0x1FFF

// Currently, NTiEvent reserves events ranged from 0x2000 to 0x2fff for user events
#define USR_EVENT_BASE  0x2000
#define USR_EVENT_MAX   0x2FFF
// Currently, NTiEvent also reserves events ranged from 0x8000 to 0x8fff for user events
#define START_EVENTCODE_RESERVED_FOR_USER      0x8000
#define END_EVENTCODE_RESERVED_FOR_USER        0x8FFF

#endif

// End of NTiEvent.h
