/* -------------------------------------------------------------------------
                Copyright (c) Engage Software Corporation 1994 - 1999.
                All rights reserved.
   ------------------------------------------------------------------------- */
#ifndef BRD_MT_CODE_H
#define BRD_MT_CODE_H

// Ai-Logix SmartWorks header files
// Ensure include path in project options has path to Ai-Logix 3rd party files
#include <NTiAPI.h>
#include <NTiEvent.h>

#include <fstream>
using std::fstream;

#include "..\TFServer\signalevt.h"
#include "brd_generic.h"

typedef struct AiLAudioBuffer
{
   SOUND_FORMAT format;
   // 200 ms * 8 samples/ms * 2 bytes/sample
   unsigned char buffer[200*8*2];
} AiLAudioBuffer;

class AiLAudioState : public AudioState
{
   public:

      AiLAudioState(CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>* commHub);

      unsigned long AudioBufferSize(SOUND_FORMAT format = TF_WAV_8KHZ_16BIT);

      void Reset(void);

      bool StartRecording(const char* fileName, SOUND_FORMAT format);
      void StopRecording(void);

      bool UpdateStreams(CHANNEL curChannel);

      bool IsVoiceRecorded(void);
      void SetVoiceRecorded(bool voiceRecorded = true);
      void ResetVoiceRecorded(void);

      bool IsDialing();
      void StartDialing();
      void StopDialing();

   protected:

      bool isVoiceRecorded;
      bool isDialing;
      fstream recordFile;
      AiLAudioBuffer audioBuffer;
};

typedef struct
{
	CHANNEL channel;
	SignalQueue<MT_EVENT>* eventQueue;
	SignalQueue<MT_EVENT>* callTriggerQueue;
   PIVRINFO pInfo;
   AiLAudioState* audioState;
} CALLBACK_INFO;

/* -------------------------------------------------------------------------
   Class:       MTBoardEvent

   Purpose:     Implements all the functions defined in the "BoardGeneric"
                class.
   ------------------------------------------------------------------------- */
class MTBoardEvent : public BoardEvent
{
	public:
 		MTBoardEvent(void);
 		virtual ~MTBoardEvent(void);

      bool SameEvent(BoardEvent* comparisonEvent);

   	MT_EVENT MTEvent;
};

/* -------------------------------------------------------------------------
   Class:       BoardMT

   Purpose:     Implements all the functions defined in the "BoardGeneric"
                class.
   ------------------------------------------------------------------------- */
class BoardMT : public BoardGeneric
{
	public:

      // Ai-logix can run exclusively active, or alternate active/passive based on TeleFlow app
      enum T_ACTIVITY_MODE { ACTIVE, PASSIVE, ACTIVE_ONLY };

   	BoardMT(PIVRINFO pIVRInfo);
	   virtual ~BoardMT(void);

      virtual void Reset(void);
      IV_STATUS StartReorderToneDetection(void);
      IV_STATUS StopReorderToneDetection(void);
      IV_STATUS Wait(int timeout, bool interruptable);

	   // ---- LINE FUNCTIONS -----------------------------------------------------

      // allows setting of specific hardware parameters
   	IV_STATUS SetParameter(const char* pszParm, DWORD dwParmValue);

      // timeout of 0 inidicates returns immediately with IVR_SUCCESS if an event
      // has already occurred or IVR_TIMEOUT if no event has occurred
      // timeout of FOREVER inidicates wait until event arrives
      IV_STATUS CallTrigger(const char* triggerList, const char*& trigger, char*& triggerInfo, long timeout);

      // flashTime of 0 skips flash
   	IV_STATUS FlashHook(long flashTime);

	   IV_STATUS Hangup(bool returnHangup = false, char* pszCause = NULL);
		IV_STATUS Pickup(void);
	   IV_STATUS PlaceCall(const char* digits, const char* callerID, int numrings,
         callMode mode, int flashdur, bool connectOnCED, bool connectOnSIT, int* pVoiceLength,
         int silenceTimeout, int maxVoiceTimeout,
         ISDNNumberPlan callingNumPlan = UNKNOWN_NUMBER_PLAN,
         ISDNNumberType callingNumType = UNKNOWN_NUMBER_TYPE,
         ISDNNumberPlan calledNumPlan = UNKNOWN_NUMBER_PLAN,
         ISDNNumberType calledNumType = UNKNOWN_NUMBER_TYPE);
      IV_STATUS WaitForCall(long timeout, int numrings, bool pickup, char* calledDigits,
         char* callingDigits, char* handsetDisplay, int numdigits = 0);

		IV_STATUS Switch(int newPort, bool bListen);
		IV_STATUS UnSwitch(void);
		virtual IV_STATUS QuerySwitch(void);

	   // ---- TT FUNCTIONS -------------------------------------------------------
   	IV_STATUS FlushDigits(void);
   	IV_STATUS GenerateDigits(const char* ttStr);
	   IV_STATUS QueryDigit(char& digit);

      // timeout of 0 inidicates returns immediately with IVR_SUCCESS if digit
      // has already occurred or IVR_TIMEOUT if no digit has occurred
      // count must be > 0 or INFINITE
      // digits buffer is populated by GetDigits and the client is repsonible for
      // deleting the buffer
      // Digits A,B,C, and D are in upper case
	   IV_STATUS GetDigits(
	      unsigned count,	// Number of digits to collect.
   	   char*& digits,		// Digit buffer returned from GetDigits
      	int newTime,		// Timeout between key presses.
	      const char* ttOkay,		// Digits that are valid.
   	   const char* ttDone);		// Digits that terminate TT collection.

   	// ---- AUDIO FUNCTIONS ----------------------------------------------------
	   IV_STATUS AudioBeep(bool interruptable);

      // maxRecTime of 0 records an empty or short as possible sound file
      // maxRecTime of FOREVER indicates record until interrupt or silence timeout
      IV_STATUS StartRecordFile(const char* filename, bool interruptable,
         SOUND_FORMAT format, int volLevel, int maxRecTime, int maxSilence,
         bool disableBeep, bool automaticGainControl);

		// Record to an audio buffer
		// Audio buffer must be created by the calling module
		// Maximum silence time is maxSilence or until buffer full, which ever comes first
      // There is no maximum record time, as this is determined by the size of the buffer
      /*
      IV_STATUS StartRecordBuffer(char* buffer, unsigned int bufsize,
         bool interruptable, SOUND_FORMAT format, int volLevel,
         int maxSilence, bool disableBeep, bool automaticGainControl);
      */
      // UNABLE TO MAKE THIS WORK - TF Jun.10/04
      // ENCOUNTERED PROBLEMS WITH STREAMING
      // MAY NEED HELP FROM SB
      // Override brd_generic::AudioRecordBuffer() instead...
      IV_STATUS AudioRecordBuffer(char* buffer, unsigned int bufsize,
         bool interruptable, SOUND_FORMAT format, int volLevel,
         int maxSilence, bool disableBeep, bool automaticGainControl);

      IV_STATUS AudioStopRecord(void);

      IV_STATUS AudioWaitForRecordStop(void);

		IV_STATUS AudioRecognise(int noVoiceTimeout, int endOfVoiceTimeout);

      // playForTime of 0 indicates do not play file
      // playForTime of FOREVER indicates play until end of file
	   IV_STATUS StartPlayFile(const char* filename, bool interruptable, SOUND_FORMAT format,
         int volLevel, unsigned int playFromTime = 0, unsigned int playForTime = FOREVER);

      IV_STATUS StartPlayBuffer(char* audioBuffer, unsigned bufferSize, bool interruptable,
         SOUND_FORMAT format, int volLevel);

	   IV_STATUS AudioStopPlay(void);

      IV_STATUS AudioWaitForPlayStop(void);

   	IV_STATUS AudioPlayEvalMessage(void);

      IV_STATUS HTTPRequest(HTTPTransaction& http, int timeout, int connectionTimeout,
         int connectionRetries, const char* filename, SOUND_FORMAT format, int volLevel);

      IV_STATUS StartStreaming(SOUND_FORMAT format);
      IV_STATUS StopStreaming(void);

      const char* GetEventName(int event);

      // Retrieve current mode: passive, active, active_only; see activityMode property
      T_ACTIVITY_MODE CurrentActivityMode(void);

	protected:

      // Eventually we should just use a 1 based curLine
      CHANNEL curChannel;
   	CHANNEL otherChannel;      // Other channel, if curChannel is initiated
		                           // a Switch connection; will be -1 if not.

      AiLAudioState* audioState;
      HANDLE hMTEventQueue;
      SignalQueue<char> ttQueue;			// software touch tone queue with depth of 128

	   SignalQueue<MT_EVENT> eventQueue;   // Structure to hold board events.
	   SignalQueue<MT_EVENT> callTriggerQueue;   // Structure to hold board events for Call Trigger step
      HANDLE hPollThread;							// Handle for event polling thread
      unsigned int pollThreadId;
      CALLBACK_INFO callbackParams;          // Info to pass to board event callback function

      unsigned short adapterType;
      unsigned short ANILength;
      map<int, const char*> eventNames;

	   char eventNameBuffer[256];

      // State of this port, one of active_only, active, or passive, based on TeleFlow app
      // This controls how BoardMT queues events and manages its event queues.
      // Passive applications require two queues, callTriggerQueue to manage all events
      // and eventQueue (aka "board queue") to manage events needed by board functions
      // to make active state changes. Passive applications can switch between PASSIVE
      // and ACTIVE. Applications that are ACTIVE_ONLY are not permitted to use the
      // CallTrigger step, and so that queue must not be used (it would continously grow)
      T_ACTIVITY_MODE activityMode;

	   // Prevent compiler from creating default constructor, default copy
   	// constructor, and assignment operator.
	   BoardMT(void);
   	BoardMT(const BoardMT&);
	   BoardMT& operator=(BoardMT&);

      int BuildDTMF(const char* filter);
		unsigned int GetEncoding(SOUND_FORMAT format);
      short GetVolumeLevel(int volLevel);
		IV_STATUS ProcessBoardEvent(BoardEvent* boardEvent, bool& bFound);
      bool CallerIDEvent(MT_EVENT& event, char* callingDigits);
      IV_STATUS AudioStopEvent(MT_EVENT* event);
      void InitialiseEventNames(void);
      void PrintMTError(void);

      // Sets the current activity mode; returns IVR_SUCCESS or IVR_FAIL.
      // If port is already set to ACTIVE_ONLY, attempting to change it will fail.
      IV_STATUS SetActivityMode(T_ACTIVITY_MODE requestedSetting);
};

#endif
