/* -------------------------------------------------------------------------
   Module:      BRD_SIMULATOR.CPP
   ------------------------------------------------------------------------- */

#include <winsock2.h>
#include <windows.h>
#include <io.h>	// access
#include <ctype.h>   // isdigit
#include <mmsystem.h>	// MCI stuff
#include <fstream>
using std::ofstream;

#include <string>
using std::string;

#include "..\escutil\iv_str.h"
#ifdef TF_CLOSED_SOURCE
#include "..\tts\tts_code.h"
#endif
#include "brd_simulator_communication.h"
#include "..\Voice\Voice.h"
#include "brd_simulator_code.h"

SimulatorBoardEvent::SimulatorBoardEvent(void)
{
	// Do nothing
}
//---------------------------------------------------------------------------

SimulatorBoardEvent::~SimulatorBoardEvent(void)
{
	// Do nothing
}
//---------------------------------------------------------------------------

bool SimulatorBoardEvent::SameEvent(BoardEvent* comparisonEvent)
{
	return(SimEvent == ((SimulatorBoardEvent*) comparisonEvent)->SimEvent);
}
//---------------------------------------------------------------------------

static const char* MMError(MMRESULT mmResult)
{
   switch(mmResult)
   {
      case MMSYSERR_ALLOCATED: return("MMSYSERR_ALLOCATED");
      case MMSYSERR_BADDEVICEID: return("MMSYSERR_BADDEVICEID");
      case MMSYSERR_INVALHANDLE: return("MMSYSERR_INVALHANDLE");
      case MMSYSERR_NODRIVER: return("MMSYSERR_NODRIVER");
      case MMSYSERR_NOMEM: return("MMSYSERR_NOMEM");
      case WAVERR_BADFORMAT: return("WAVERR_BADFORMAT");
      case WAVERR_STILLPLAYING: return("WAVERR_STILLPLAYING");
      case WAVERR_UNPREPARED: return("WAVERR_UNPREPARED");
      default: return("Unknown error");
   }
};

SimulatorAudioState::SimulatorAudioState(CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>* commHub, HANDLE hSimulator) : AudioState(commHub)
{
   memset(&mciOpenParms, 0, sizeof(MCI_OPEN_PARMS));
   wfx.wFormatTag = WAVE_FORMAT_PCM;
   wfx.nChannels = 1;
   wfx.nSamplesPerSec = 8000;
   wfx.nAvgBytesPerSec = 16000;
   wfx.nBlockAlign = 2;
   wfx.wBitsPerSample = 16;
   wfx.cbSize = 0;

   MMRESULT mmResult = waveInOpen(&hwi, WAVE_MAPPER, &wfx, (DWORD)(LPVOID) hSimulator, NULL, CALLBACK_WINDOW);
   if(mmResult != MMSYSERR_NOERROR)
   {
      printf("SimulatorAudioState: Unable to open MM handle: %d\n", mmResult);
      printf("         %s\n", MMError(mmResult));
   }

   memset(bufferHeaders, 0, 10*sizeof(WAVEHDR));
   for(currentBufferHeader = 0; currentBufferHeader < 10; currentBufferHeader++)
   {
      mmResult = waveInPrepareHeader(hwi, bufferHeaders + currentBufferHeader, sizeof(WAVEHDR));
      if(mmResult != MMSYSERR_NOERROR)
      {
         printf("SimulatorAudioState: Unable to prepare MM buffer: %d\n", mmResult);
         printf("         %s\n", MMError(mmResult));
      }
   }

   currentBufferHeader = 0;
}

WAVEHDR* SimulatorAudioState::AddBuffer(char* audioBuffer, unsigned int audioBufferSize)
{
   WAVEHDR* wh = bufferHeaders + currentBufferHeader;

   wh->lpData = audioBuffer;
   wh->dwBufferLength = audioBufferSize;

   waveInAddBuffer(hwi, wh, sizeof(WAVEHDR));
   currentBufferHeader = ++currentBufferHeader % 10;

   return(wh);
}

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::BoardSimulator                           (Constructor)

   Need to do:  - Write function to build list of lines available.
   ------------------------------------------------------------------------- */
BoardSimulator::BoardSimulator(PIVRINFO pIVRInfo) : BoardGeneric(pIVRInfo)
{
   // Disallow simulator to call system
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_NOT_READY_FOR_CALL);

   preferredFormat = TF_WAV_8KHZ_16BIT;
   
   // Warning: Be careful to delete base class' audioState before overwriting
   // it with the derived class' audioState
   delete BoardGeneric::audioState;
   BoardGeneric::audioState = audioState = new SimulatorAudioState(pIVRInfo->commHub, pIVRInfo->hSimulator);

   bBrdInit = true;
   blocked = false;

   // Clear TT Queue
   ttQueue.Clear();

   for(int counter = 0; counter < MAX_LINES; counter++)
   {
   	lines[counter].inUse = false;
   }

   lastLine = MAX_LINES - 1;

	// Initialise curLine and curChannel (curChannel = curLine + 1)
   // curChannel is for ease of use... curLine should probably eventually be
   // 1 based instead of 0 based
	curLine = pIVRInfo->iPort - 1;
   if(curLine >= 0)
   {
		// Set the wait object handle for our event queue
   	waitObjects[4] = eventQueue.GetSignal();
      trunkType = TF_TRUNK_TYPE_SIMULATOR;

  	   ivLog->Print("Driver successfully opened.\n");
   }
}
/*BoardSimulator::BoardSimulator*/

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::~BoardSimulator                           (Destructor)
   ------------------------------------------------------------------------- */
BoardSimulator::~BoardSimulator(void)
{
   ivLog->Print ("Shutting down...\n");

   // Disallow simulator to call system
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_NOT_READY_FOR_CALL);

   // Disconnect simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_ON_HOOK);
}
/*BoardSimulator::~BoardSimulator*/

IV_STATUS BoardSimulator::ProcessBoardMessage(BoardCommunicationPacket* boardMessage)
{
   BoardSimulatorCommunicationPacket* message = (BoardSimulatorCommunicationPacket*) boardMessage;
   IV_STATUS status = IVR_SUCCESS;

   switch(message->event)
   {
      case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
      {
         ttQueue.Add((char) message->eventInfo);
         break;
      }

      case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_CHANGE_SHORT_AUDIO_LENGTH:
      {
         shortAudioPromptLength = message->eventInfo;
         delete message;
         return(IVR_SUCCESS);
      }

//      case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_BARGEIN:
//      {
//         status = StartBargeInRecord();
//         break;
//      }

      case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_AUDIO_CHUNK_FINISHED:
      {
         if(pInfo->speechRecognition != NULL)
         {
            currentSRBuffer = audioState->AddBuffer(pInfo->speechRecognition->SRGetNextAudioChunk(), pInfo->speechRecognition->SRGetAudioChunkSize())->lpData;
         }

         delete message;
         return(IVR_SUCCESS);
      }
   }

   eventQueue.Add(message->event);
   delete message;

   return(status);
}

IV_STATUS BoardSimulator::ProcessBoardEvent(BoardEvent* boardEvent, bool& bFound)
{
	// All Simulator events are valid
	bFound = true;

	BoardSimulatorCommunicationPacket::BRD_SIMULATOR_EVENT* event = &(((SimulatorBoardEvent*) boardEvent)->SimEvent);

   eventQueue.Get(*event);

   switch(*event)
   {
      case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_ON_HOOK:
      {
//         DeactivateBargeIn();
        	lineOffHook = false;
      	callEnd.Update();
         FlushDigits();
         eventQueue.Clear();
         ivLog->Print("      Caller hung up\n");
         return(IVR_HANGUP);
      }

      case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_PLAYRECORD_DONE:
      {
         IV_STATUS status = IVR_SUCCESS;
         DWORD dwReturn;
         char string[1024];

         if(audioState->IsRecording())
         {
            MCI_SAVE_PARMS mciSaveParms;

            audioState->StopRecording();
            memset(&mciSaveParms, 0, sizeof(mciSaveParms));

            // Save the recording to a file named TEMPFILE.WAV. Wait for
            // the operation to complete before continuing.
            mciSaveParms.lpfilename = audioState->mciOpenParms.lpstrElementName;

            if((dwReturn = mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_SAVE, MCI_WAIT | MCI_SAVE_FILE, (DWORD)(LPVOID) &mciSaveParms))!= 0)
            {
               ivLog->Print("      mciSendCommand MCI_SAVE failed: %d\n", dwReturn);
               mciGetErrorStringA(dwReturn, string, 1023);
               ivLog->Print("         %s\n", string);
               status = IVR_FAIL;
            }

            delete[] audioState->mciOpenParms.lpstrElementName;
            audioState->mciOpenParms.lpstrElementName = NULL;

            if((dwReturn = mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_CLOSE, MCI_WAIT, NULL)) != 0)
            {
               ivLog->Print("      mciSendCommand MCI_CLOSE failed: %d\n", dwReturn);
               mciGetErrorStringA(dwReturn, string, 1023);
               ivLog->Print("         %s\n", string);
               status = IVR_FAIL;
            }
         }
         else if(audioState->IsPlaying())
         {
            audioState->StopPlaying();

            if((dwReturn = mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_CLOSE, MCI_WAIT, NULL)) != 0)
            {
           	   ivLog->Print("      mciSendCommand MCI_CLOSE failed: %d\n", dwReturn);
               mciGetErrorStringA(dwReturn, string, 1023);
               ivLog->Print("         %s\n", string);
               status = IVR_FAIL;
            }
         }

         return(status);
      }

      default:
      {
      	return(IVR_SUCCESS);
      }
   }
}

void BoardSimulator::SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_EVENT event, int eventInfo)
{
   TFSimulatorCommunicationPacket* message = new TFSimulatorCommunicationPacket(event, pInfo->iAppID);
   message->eventInfo = eventInfo;
   pInfo->commHub->SendMessage(TFServerCommunicationPacket::TF_SIMULATOR, message);
}

//---------------------------------------------------------------------------

IV_STATUS BoardSimulator::ProcessSystemEvent(void)
{
   IV_STATUS status = BoardGeneric::ProcessSystemEvent();

   if(status != IVR_SUCCESS)
   {
      lineOffHook = false;
   }

   return(status);
}

IV_STATUS BoardSimulator::StartReorderToneDetection(void)
{
   return(IVR_SUCCESS);
}

IV_STATUS BoardSimulator::StopReorderToneDetection(void)
{
   return(IVR_SUCCESS);
}

IV_STATUS BoardSimulator::Wait(int timeout, bool interruptable)
{
   LARGE_INTEGER waitTimeout;
   IV_STATUS status;
	SimulatorBoardEvent boardEvent;

   if(lineOffHook && interruptable)
   {
      char ttDig;

      if((status = QueryDigit(ttDig)) != IVR_SUCCESS)
      {
         return(status);
      }

      if((ttDig != '\0') || ((pInfo->speechRecognition != NULL)
         && (pInfo->speechRecognition->SRIsReceivingAudio() || pInfo->speechRecognition->SRIsPendingAudio())))
      {
         ivLog->Print("      Interrupted!\n");
         return(IVR_SUCCESS);
      }
   }

   if(timeout == 0)
   {
      return(IVR_SUCCESS);
   }

   waitTimeout.QuadPart = -(LONGLONG) timeout * 10000;
   SetWaitableTimer(hGenericTimer, &waitTimeout, 0, 0, 0, false);

   while((status = WaitForEvent(&boardEvent)) == IVR_SUCCESS)
   {
      switch(boardEvent.SimEvent)
      {
         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
         {
            if(interruptable)
            {
               ivLog->Print("      Interrupted by touch tone\n");
              	CancelWaitableTimer(hGenericTimer);
               return(IVR_SUCCESS);
            }

            break;
         }

//         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_BARGEIN:
//         {
//            ivLog->Print("      Interrupted by speech\n");
//            return(StartBargeInRecord());
//         }

         default:
         {
            // Do nothing
         }
      }
   }

  	CancelWaitableTimer(hGenericTimer);

   return(status == IVR_GENERIC_TIMEOUT ? IVR_SUCCESS : status);
}

// ****************************************************************************
// ************************** LINE FUNCTIONS **********************************
// ****************************************************************************

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::SetParameter                             (public)
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::SetParameter(const char*, DWORD)
{
   return(IVR_SUCCESS);
}
/*BoardSimulator::SetParameter*/

// ****************************************************************************
// ************************** LINE FUNCTIONS **********************************
// ****************************************************************************

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::BlockCalls                               (public)
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::BlockCalls(void)
{
	if(lineOffHook)
   {
      ivLog->Print("      ERROR: Port already in use\n");
   	return(IVR_FATAL);
   }

	blocked = true;
   return(IVR_SUCCESS);
}
/*BoardSimulator::BlockCalls*/

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::FlashHook                                (public)

   Purpose:     Used to perform a flash-hook during an active call.
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::FlashHook(long flashTime)
{
	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
   	return(IVR_FATAL);
   }
	ivLog->Print("      Flashing hook for %d ms\n", flashTime);
   
	if(flashTime > 0)
   {
	   PlaySound("FLASHHOOKON", pInfo->boardLibrary, SND_RESOURCE | SND_ASYNC);
   	Sleep(flashTime);
	   PlaySound("FLASHHOOKOFF", pInfo->boardLibrary, SND_RESOURCE | SND_ASYNC);
   }

	return(IVR_SUCCESS);
}
/*BoardSimulator::FlashHook*/

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::Hangup                                   (public)

   Purpose:     To disconnect the call currently in progress.
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::Hangup(bool returnHangup /* = false */, char*)
{
   // Disconnect simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_ON_HOOK);

//   DeactivateBargeIn();

   if(lineOffHook)
   {
      lineOffHook = false;

      ivLog->Print("      Hanging up\n");

      // Clear touch-tones from the buffer
      FlushDigits();
   }
   // Call deemed to start with incoming call event; lineOffHook may not have not been set;
   // therefore always update callEnd on Hangup.
   callEnd.Update();

   return(returnHangup ? IVR_HANGUP : IVR_SUCCESS);
}
/*BoardSimulator::Hangup*/

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::Pickup                                (public)
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::Pickup(void)
{
	SimulatorBoardEvent boardEvent;

	if(lineOffHook)
   {
      ivLog->Print("      ERROR: Port already in use\n");
   	return(IVR_FATAL);
   }

   lineOffHook = true;

   // Connect simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_OFF_HOOK);

   if(disableReorderToneDetection)
   {
      ivLog->Print("      Reorder tone detection disabled!\n");
   }

//   if(pInfo->speechRecognition != NULL)
//   {
//      pInfo->speechRecognition->SRClearAudio();
//      if(audioState->IsBargeInPermitted())
//      {
//         return(ActivateBargeIn());
//      }
//   }

   return(IVR_SUCCESS);
}
/*BoardSimulator::Pickup*/

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::PlaceCall                                (public)
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::PlaceCall(const char* digits, const char* , int, callMode mode,
   int flashdur, bool connectOnCED, bool connectOnSIT, int* pVoiceLength, int silenceTimeout,
   int, ISDNNumberPlan, ISDNNumberType, ISDNNumberPlan, ISDNNumberType)
{
	SimulatorBoardEvent boardEvent;
	IV_STATUS status;

   if(silenceTimeout > 0)
   {
		ivLog->Print("      Warning: Initial voice length detection is not supported in the simulator\n");
		ivLog->Print("         Initial voice length set to 0\n");
      if(pVoiceLength != NULL)
      {
         *pVoiceLength = 0;
      }
   }

   if(mode == newLine)
   {
      if(lineOffHook)
      {
         ivLog->Print("      ERROR: Port already in use\n");
       	return(IVR_FATAL);
      }
   }
   else if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
    	return(IVR_FATAL);
   }
   else // use current connection
   {
   	if((status = FlashHook(flashdur)) != IVR_SUCCESS)
      {
       	return(status);
      }
   }

 	lineOffHook = true;

   PlaySound("DIALTONE", pInfo->boardLibrary, SND_RESOURCE);

   if((status = GenerateDigits(digits)) != IVR_SUCCESS)
   {
     	lineOffHook = false;
      return(status);
   }

   // Send outgoing call signal to simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_OUTGOING_CALL);

   // Wait for board to signal dialing complete
   status = WaitForEvent(&boardEvent);

   if(status == IVR_SUCCESS)
   {
      switch(boardEvent.SimEvent)
      {
			case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_CED:
         {
            pInfo->variables->SetSystem("@CALLOUT_RESULT", connectOnCED ? "CONNECT" : "FAILURE");
            pInfo->variables->SetSystem("@CALLOUT_REASON", "CED");
            status = connectOnCED ? IVR_SUCCESS : IVR_FAIL;
            break;
         }

			case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_SIT:
         {
            pInfo->variables->SetSystem("@CALLOUT_RESULT", connectOnSIT ? "CONNECT" : "FAILURE");
            pInfo->variables->SetSystem("@CALLOUT_REASON", "SIT");
            status = connectOnSIT ? IVR_SUCCESS : IVR_FAIL;
            break;
         }

      	case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_OFF_HOOK:
         {
            pInfo->variables->SetSystem("@CALLOUT_RESULT", "CONNECT");
            pInfo->variables->SetSystem("@CALLOUT_REASON", "CONNECT");
            status = IVR_SUCCESS;
            break;
         }

		   case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_BUSY:
	      {
            pInfo->variables->SetSystem("@CALLOUT_RESULT", "BUSY");
            pInfo->variables->SetSystem("@CALLOUT_REASON", "BUSY");
	      	status = IVR_BUSY;
            break;
	      }

	      case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_NO_ANSWER:
	      {
            pInfo->variables->SetSystem("@CALLOUT_RESULT", "NOANSWER");
            pInfo->variables->SetSystem("@CALLOUT_REASON", "MAX_RINGS");
	      	status = IVR_TIMEOUT;
            break;
	      }

         default:
  	      {
            pInfo->variables->SetSystem("@CALLOUT_RESULT", "FAILURE");
            pInfo->variables->SetSystem("@CALLOUT_REASON", "UNKNOWN");
          	lineOffHook = false;
     	   	ivLog->Print("      ERROR: Unexpected Simulator board event %d:'%s'\n", boardEvent.SimEvent, BoardSimulatorCommunicationPacket::GetEventName(boardEvent.SimEvent));
  	         return(IVR_FAIL);
         }
      }
   }

   if(status == IVR_SUCCESS)
   {
      // Connect simulator
      SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_OFF_HOOK);
      callStart.Update();
      callEnd.Invalidate();

//      if(audioState->IsBargeInPermitted())
//      {
//         status = ActivateBargeIn();
//      }

      if(disableReorderToneDetection)
      {
         ivLog->Print("      Reorder tone detection disabled!\n");
      }
   }
   else if((status == IVR_FAIL) || (status == IVR_BUSY) || (status == IVR_TIMEOUT))
   {
      lineOffHook = false;
   }
   else
   {
      SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_OUTGOING_CALL);
   }

  	return(status);
}
/*BoardSimulator::PlaceCall*/

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::UnblockCalls                             (public)
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::UnblockCalls()
{
	if(lineOffHook)
   {
      ivLog->Print("      ERROR: Port already in use\n");
   	return(IVR_FATAL);
   }

	blocked = false;
   return(IVR_SUCCESS);
}
/*BoardSimulator::UnblockCalls*/

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::WaitForCall                              (public)

   Purpose:     Wait for an incoming call on the specified port, optionally
                returning the number called.  Assumes one port/dev.
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::WaitForCall(long timeout, int numrings, bool pickup,
   char* calledDigits, char* callingDigits, char* handsetDisplay, int)
{
	SimulatorBoardEvent boardEvent;
   IV_STATUS status = IVR_GENERIC_EVENT;
   LARGE_INTEGER waitTimeout;
   HANDLE hInterRingTimer;
   LARGE_INTEGER interRingTimeout;

	if(lineOffHook)
   {
      ivLog->Print("      ERROR: Port already in use\n");
   	return(IVR_FATAL);
   }

   if(calledDigits != NULL)
   {
      calledDigits[0] = '\0';
   }

   if(callingDigits != NULL)
   {
      callingDigits[0] = '\0';
   }

   if(handsetDisplay != NULL)
   {
      handsetDisplay[0] = '\0';
   }

   if(numrings == 0)
   {
      numrings = 1;
   }

	// Set timeout timer
   hInterRingTimer = CreateWaitableTimer(0, false, 0);
   if(timeout != FOREVER)
   {
      waitTimeout.QuadPart = -(LONGLONG) timeout * 10000000;
      SetWaitableTimer(hGenericTimer, &waitTimeout, 0, 0, 0, false);
   }

   // Allow simulator to call system
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_READY_FOR_CALL);

   // Set inter-ring timer to 8 seconds
   interRingTimeout.QuadPart = -80000000;
   SetGenericEvent(hInterRingTimer);

   while(status == IVR_GENERIC_EVENT)
   {
   	// Wait until enough rings have occurred
      for(int counter = 0; (counter < numrings) && ((status = WaitForEvent(&boardEvent)) == IVR_SUCCESS);)
      {
         switch(boardEvent.SimEvent)
         {
      	   case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_RING:
            {
               if(!blocked)
               {
                  SetWaitableTimer(hInterRingTimer, &interRingTimeout, 0, 0, 0, false);
                  counter++;
               }
               else
               {
                  break;
               }
            }

            case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
            {
               break;
            }

            default:
            {
               CancelWaitableTimer(hGenericTimer);
            	CancelWaitableTimer(hInterRingTimer);
               ClearGenericEvent();
               CloseHandle(hInterRingTimer);
     	   	   ivLog->Print("      ERROR: Unexpected Simulator board event %d:'%s'\n",boardEvent.SimEvent, BoardSimulatorCommunicationPacket::GetEventName(boardEvent.SimEvent));

               // Disallow simulator to call system
               SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_NOT_READY_FOR_CALL);

  	            return(IVR_FAIL);
            }
         }
      }

   	CancelWaitableTimer(hInterRingTimer);
   }

	CancelWaitableTimer(hGenericTimer);
   ClearGenericEvent();
   CloseHandle(hInterRingTimer);

   // Disallow simulator to call system
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_NOT_READY_FOR_CALL);

   if(status == IVR_SUCCESS)
   {
      // Set call start with the incoming call event
      callStart.Update();
      callEnd.Invalidate();

      return(pickup ? Pickup() : IVR_SUCCESS);
   }
   else if(status == IVR_GENERIC_TIMEOUT)
   {
      return(IVR_TIMEOUT);
   }

  	return(status);
}
/*BoardSimulator::WaitForCall*/

IV_STATUS BoardSimulator::Switch(int newPort, bool bListen)
{
	if(bListen)
   {
		ivLog->Print("      Switch current port (%d) to port %d (listen only)\n", pInfo->iPort, newPort);
   }
   else
   {
		ivLog->Print("      Switch current port (%d) to port %d\n", pInfo->iPort, newPort);
   }
	ivLog->Print("      WARNING: Switching cannot be simulated, assuming success.\n");
	ivLog->Print("      Functionality should be tested on telephony hardware.\n");
	return(IVR_SUCCESS);
}

IV_STATUS BoardSimulator::UnSwitch(void)
{
	ivLog->Print("      Disconnecting switch state(s)\n");
	ivLog->Print("      WARNING: Switching cannot be simulated, assuming success.\n");
	ivLog->Print("      Functionality should be tested on telephony hardware.\n");
	return(IVR_SUCCESS);
}

// ****************************************************************************
// *********************** TOUCH TONE FUNCTIONS *******************************
// ****************************************************************************

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::FlushDigits                              (public)

   Purpose:     Removes any Touch-Tones that the user has entered.

   Notes:       - Should check to ensure that 'ctahd' is valid.
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::FlushDigits(void)
{
   // Remove digits from the software touch tone queue
 	ttQueue.Clear();

   return(IVR_SUCCESS);
}
/*BoardSimulator::FlushDigits*/

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::GetDigits                                (public)

   Purpose:     Attempts to collect the specified number of DTMFs
                (Touch-Tones).  Collection will end when the maximum number
                of TT are entered, timeout occurs, or the user presses one
                of the termination TT.
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::GetDigits(
   unsigned  count,     // Number of digits to collect.
   char*&    digits,    // Digit buffer returned from GetDigits
   int       newTime,   // Timeout between key presses.
   const char* ttOkay,    // Digits that are valid.
   const char* ttDone)    // Digits that terminate TT collection.
{
	SimulatorBoardEvent boardEvent;
   IV_STATUS status = IVR_SUCCESS;
   bool terminate = false;
   LARGE_INTEGER waitTimeout;
   string digitString;
   char tt;

	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
      digits = new char[1];
      digits[0] = '\0';
   	return(IVR_FATAL);
   }

   // newTime is in ms
   waitTimeout.QuadPart = -(LONGLONG) newTime * 10000;

	// If there is an X anywhere in the string then all digits are acceptable
   if(strchr(ttOkay, 'X'))
   {
   	ttOkay = "0123456789ABCD*#";
   }

	// If there is an X anywhere in the string then all digits are acceptable
   if(strchr(ttDone, 'X'))
   {
   	ttOkay = "0123456789ABCD*#";
   }

	// Until enough digits or other terminating condition
   while(((count > 0) || (count == INFINITE)) && !terminate && !ttQueue.Empty())
   {
      // Get digit from software touch tone queue
  		ttQueue.Get(tt);

      // Check for termination digit
      if((tt != '\0') && strchr(ttDone, tt))
      {
         terminate = true;
      }

      // Keep touch tone only if it is in ttOkay
      if((tt != '\0') && strchr(ttOkay, tt))
      {
         digitString += tt;
         if(count != INFINITE)
         {
            count--;
         }
      }
   }

	// Until enough digits or other terminating condition
	while(((count > 0) || (count == INFINITE)) && !terminate)
   {
		// Set timeout timer
   	if(newTime != FOREVER)
	   {
   	   SetWaitableTimer(hGenericTimer, &waitTimeout, 0, 0, 0, false);
	   }

      // Wait for digit event from board event queue
      status = WaitForEvent(&boardEvent);

      switch(status)
      {
         case IVR_GENERIC_TIMEOUT:
         {
            // If we timed out with no digits return IVR_TIMEOUT
            status = digitString.length() == 0 ? IVR_TIMEOUT : IVR_SUCCESS;
            terminate = true;
            break;
         }

         case IVR_SUCCESS:
         {
				if(boardEvent.SimEvent == BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT)
            {
               // Wait for digit to show up in software queue
	   	   	WaitForSingleObject(ttQueue.GetSignal(), INFINITE);

               // Get digit from software touch tone queue
     		      ttQueue.Get(tt);

               // Check for termination digit
               if((tt != '\0') && strchr(ttDone, tt))
               {
                  terminate = true;
               }

               // Keep touch tone only if it is in ttOkay
               if((tt != '\0') && strchr(ttOkay, tt))
               {
                  digitString += tt;
                  if(count != INFINITE)
                  {
                     count--;
                  }
               }
            }

            break;
         }

         default:
         {
            terminate = true;
         }
      }
   }

   // Stop timeout timer
	CancelWaitableTimer(hGenericTimer);

   digits = new char[digitString.length() + 1];
   strcpy(digits, digitString.c_str());
   strupr(digits);

	return(status);
}
/*BoardSimulator::GetDigits*/

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::GenerateDigits                           (public)
   ----------------------------------------------------------------------- */
IV_STATUS BoardSimulator::GenerateDigits(const char* ttStr)
{
   char* touchtone = "TOUCHTONEX";
   char digit;

	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
   	return(IVR_FATAL);
   }

	// Dial digits... play touch tones
   for(unsigned int index = 0; ttStr[index] != '\0'; index++)
   {
      digit = (char) toupper(ttStr[index]);

      if(isdigit(digit) || ((digit >= 'A') && (digit <= 'D')))
      {
         touchtone[9] = digit;
         PlaySound(touchtone, pInfo->boardLibrary, SND_RESOURCE | SND_ASYNC);
      }

      if(digit == '*')
      {
         PlaySound("TOUCHTONESTAR", pInfo->boardLibrary, SND_RESOURCE | SND_ASYNC);
      }

      if(digit == '#')
      {
         PlaySound("TOUCHTONEPOUND", pInfo->boardLibrary, SND_RESOURCE | SND_ASYNC);
      }

      if((digit == '.') || (digit == ','))
      {
         Sleep(500);
      }
      else
      {
         Sleep(225);
      }
   }

	return(IVR_SUCCESS);
}
/*BoardSimulator::GenerateDigits*/

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::QueryDigit                               (public)

   Purpose:     Checks the ADI Touch-Tone queue to see if any tones are
                pending, and puts the first one into the 'ttStr'
                parameter.  The tones ARE NOT removed from the queue.
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::QueryDigit(char& digit)
{
	// Get a copy of the first digit in the queue (does not remove from queue!)
	if(!ttQueue.Peek(digit))
   {
      // No digit in queue so return NULL
   	digit = '\0';
   }

   return(IVR_SUCCESS);
}
/*BoardSimulator::QueryDigit*/

// ****************************************************************************
// *************************** AUDIO FUNCTIONS ********************************
// ****************************************************************************

/* -------------------------------------------------------------------------
   Name:        BoardSimulator::AudioBeep                                (public)
   ------------------------------------------------------------------------- */
IV_STATUS BoardSimulator::AudioBeep(bool interruptable)
{
   IV_STATUS status;
	SimulatorBoardEvent boardEvent;

	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
   	return(IVR_FATAL);
   }

   if(interruptable)
   {
      char ttDig;

      if((status = QueryDigit(ttDig)) != IVR_SUCCESS)
      {
         return(status);
      }

      if((ttDig != '\0') || ((pInfo->speechRecognition != NULL)
         && (pInfo->speechRecognition->SRIsReceivingAudio() || pInfo->speechRecognition->SRIsPendingAudio())))
      {
         ivLog->Print("      Interrupted!\n");
         return(IVR_SUCCESS);
      }
   }

   if(!PlaySound("BEEP", pInfo->boardLibrary, SND_RESOURCE | SND_SYNC))
   {
      ivLog->Print("      Failed to beep\n");
      return(IVR_FAIL);
   }

   eventQueue.Add(BoardSimulatorCommunicationPacket::BRD_SIMULATOR_PLAYRECORD_DONE);

	// Wait for SIMEVT_PLAYRECORD_DONE to process
   while((status = WaitForEvent(&boardEvent)) == IVR_SUCCESS)
   {
      switch(boardEvent.SimEvent)
      {
      	case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_PLAYRECORD_DONE:
         {
            return(IVR_SUCCESS);
         }

         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_BARGEIN:
         {
            break;
         }

         default:
         {
  	   	   ivLog->Print("      ERROR: Unexpected Simulator board event %d:'%s'\n",boardEvent.SimEvent, BoardSimulatorCommunicationPacket::GetEventName(boardEvent.SimEvent));
  	         return(IVR_FAIL);
         }
      }
   }

   return(status);
}
/*BoardSimulator::AudioBeep*/

IV_STATUS BoardSimulator::StartRecordFile(const char* filename, bool interruptable,
   SOUND_FORMAT, int, int maxRecTime, int, bool disableBeep, bool)
{
   IV_STATUS status;
   DWORD dwReturn;
   MCI_RECORD_PARMS mciRecordParms;
   char string[1024];
   DWORD dwFlags = MCI_NOTIFY;

   if(audioState->IsRecording())
   {
      ivLog->Print("      ERROR: Already recording\n");
      return(IVR_FATAL);
   }

   // Check for pending digit
   if(interruptable)
   {
      char ttDig;

      if((status = QueryDigit(ttDig)) != IVR_SUCCESS)
      {
         return(status);
      }

      if(ttDig != '\0')
      {
         ivLog->Print("      Interrupted!\n");
         return(IVR_SUCCESS);
      }
   }

   // Create 0 sample 8kHz 16-bit mono wav file in order to force windows MCI
   // to record at that rate
   ofstream file(filename);

   // RIFF header:
   //    struct
   //    {
   //       char id[4];    // identifier string "RIFF"
   //       DWORD length;  // remaining file length after this RIFF header
   //       char did[4];   // data identifier such as "WAVE"
   //    } riff_hdr;

   // RIFF chunk header:
   //    struct
   //    {
   //       char id[4];    // identifier string "fmt" or "data"
   //       DWORD length;  // remaining chunk length after chunk header
   //    } chunk_hdr;

   // WAVE format chunk:
   //    struct
   //    {
   //       WORD wFormatTag;        // Format category
   //       WORD wChannels;         // Number of channels
   //       DWORD dwSamplesPerSec;  // Sampling rate
   //       DWORD dwAvgBytesPerSec; // For buffer estimation
   //       WORD wBlockAlign;       // Data block size
   //       WORD wBitsPerSample;    // Sample size
   //    } format_chunk;

   char fileHeader[44] =
      {
         0x52, 0x49, 0x46, 0x46,    // "RIFF"
         0x24, 0x00, 0x00, 0x00,    // 32 more bytes after header
         0x57, 0x41, 0x56, 0x45,    // "WAVE"
         0x66, 0x6D, 0x74, 0x20,    // "fmt"
         0x10, 0x00, 0x00, 0x00,    // 16 more bytes in chunk
         0x01, 0x00,                // Microsoft Pulse Code Modulation (PCM)
         0x01, 0x00,                // mono
         0x40, 0x1F, 0x00, 0x00,    // 8000 samples per second
         0x80, 0x3E, 0x00, 0x00,    // 160000 bytes per second
         0x02, 0x00,                // 2 byte block
         0x10, 0x00,                // 16 bits per sample
         0x64, 0x61, 0x74, 0x61,    // "data"
         0x00, 0x00, 0x00, 0x00     // 0 more bytes in chunk
      };

   if(file == NULL)
   {
      ivLog->Print("      ERROR: Unable to open file '%s' to record\n", filename);
      return(IVR_FAIL);
   }

   file.write(fileHeader, 44);

   file.close();

   if(!disableBeep)
   {
      status = AudioBeep(false);
      if(status != IVR_SUCCESS)
      {
         return(status);
      }
   }

   if(maxRecTime == 0)
   {
      return(IVR_SUCCESS);
   }

   audioState->mciOpenParms.lpstrDeviceType = (LPCSTR) MCI_DEVTYPE_WAVEFORM_AUDIO;
   audioState->mciOpenParms.lpstrElementName = new char[strlen(filename) + 1];
   strcpy((char*) audioState->mciOpenParms.lpstrElementName, filename);

   if((dwReturn = mciSendCommand(0, MCI_OPEN,
      MCI_WAIT | MCI_OPEN_TYPE | MCI_OPEN_ELEMENT | MCI_OPEN_TYPE_ID,
      (DWORD)(LPVOID) &audioState->mciOpenParms))!= 0)
   {
   	// Failed to open element. Don't close it; just return error.
		ivLog->Print("      Failed to open MCI element: %d\n", dwReturn);
      mciGetErrorStringA(dwReturn, string, 1023);
     	ivLog->Print("         %s\n", string);
      delete[] audioState->mciOpenParms.lpstrElementName;
      audioState->mciOpenParms.lpstrElementName = NULL;
   	return(IVR_FAIL);
   }

   // The device opened successfully; Send device ID to simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_SET_MCI_DEVICE, audioState->mciOpenParms.wDeviceID);

   // Begin recording and record for the specified number of milliseconds.
   mciRecordParms.dwTo = maxRecTime;

   if(maxRecTime != FOREVER) dwFlags |= MCI_TO;

   // window will be notified with an MM_MCINOTIFY message when
   // playback is complete.
   mciRecordParms.dwCallback = (DWORD) pInfo->hSimulator;

   if((dwReturn = mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_RECORD, dwFlags, (DWORD)(LPVOID) &mciRecordParms)) != 0)
   {
   	// Failed to open element.
		ivLog->Print("      mciSendCommand MCI_RECORD failed: %d\n", dwReturn);
      mciGetErrorStringA(dwReturn, string, 1023);
     	ivLog->Print("         %s\n", string);
      mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_CLOSE, MCI_WAIT, NULL);
      delete[] audioState->mciOpenParms.lpstrElementName;
      audioState->mciOpenParms.lpstrElementName = NULL;
   	return(IVR_FAIL);
   }

   audioState->StartRecording(filename, TF_WAV_8KHZ_16BIT);
   audioState->SetInterruptable(interruptable);

   // Make StopRecordButton visible in Simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_SHOW_STOP_RECORDING_BUTTON);

   return(IVR_SUCCESS);
}

IV_STATUS BoardSimulator::AudioWaitForRecordStop(void)
{
   IV_STATUS status;
	SimulatorBoardEvent boardEvent;

   if(!audioState->IsRecording())
   {
      return(IVR_SUCCESS);
   }

	// Wait for file to stop recording
   while((status = WaitForEvent(&boardEvent)) == IVR_SUCCESS)
   {
      switch(boardEvent.SimEvent)
      {
      	case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_PLAYRECORD_DONE:
         {
            // Make StopRecordButton visible in Simulator
            SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_HIDE_STOP_RECORDING_BUTTON);
            return(IVR_SUCCESS);
         }

         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_INTERRUPTPLAYRECORD:
         {
            return(AudioStopRecord());
         }

         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
         {
            if(audioState->IsInterruptable())
            {
		   	   ivLog->Print("      Interrupted by touch tone\n");

               return(AudioStopRecord());
            }

            break;
         }

         default:
         {
  	   	   ivLog->Print("      ERROR: Unexpected Simulator board event %d:'%s'\n",boardEvent.SimEvent, BoardSimulatorCommunicationPacket::GetEventName(boardEvent.SimEvent));
  	         return(IVR_FAIL);
         }
      }
   }

   AudioStopRecord();

   return(status);
}

IV_STATUS BoardSimulator::AudioStopRecord(void)
{
   DWORD dwReturn;
   char string[1024];

   if(!audioState->IsRecording())
   {
      return(IVR_SUCCESS);
   }

   // Issue stop
   if((dwReturn = mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_STOP, MCI_WAIT, NULL)) != 0)
   {
      ivLog->Print("      mciSendCommand MCI_STOP failed: %d\n", dwReturn);
      mciGetErrorStringA(dwReturn, string, 1023);
      ivLog->Print("         %s\n", string);
      return(IVR_FAIL);
   }

   return(AudioWaitForRecordStop());
}

IV_STATUS BoardSimulator::StartPlayFile(const char* filename, bool interruptable,
   SOUND_FORMAT, int, unsigned int playFromTime, unsigned int playForTime)
{
	SimulatorBoardEvent boardEvent;
   IV_STATUS status;
   DWORD dwReturn;
   MCI_SET_PARMS mciSetParms;
   MCI_STATUS_PARMS mciStatusParms;
   MCI_PLAY_PARMS mciPlayParms;
   DWORD mciPlayFlags;
   char string[1024];

	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
   	return(IVR_FATAL);
   }

   if(access(filename, 00) != 0)
   {
		ivLog->Print("      File does not exist\n");
   	return(IVR_FAIL);
   }

   if(audioState->IsPlaying())
   {
      ivLog->Print("      ERROR: Already playing\n");
      return(IVR_FATAL);
   }

   if(interruptable)
   {
      char ttDig;

      if((status = QueryDigit(ttDig)) != IVR_SUCCESS)
      {
         return(status);
      }

      if((ttDig != '\0') || ((pInfo->speechRecognition != NULL)
         && (pInfo->speechRecognition->SRIsReceivingAudio() || pInfo->speechRecognition->SRIsPendingAudio())))
      {
         ivLog->Print("      Interrupted!\n");
         return(IVR_SUCCESS);
      }
   }
//   else
//   {
//      // Disable Barge-In button in Simulator
//      SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_DISABLE_BARGEIN_BUTTON);
//   }

   // Open the device by specifying the device and filename.
   // MCI will choose a device capable of playing the specified file.
   audioState->mciOpenParms.lpstrDeviceType = (LPCSTR) MCI_DEVTYPE_WAVEFORM_AUDIO;
   audioState->mciOpenParms.lpstrElementName = new char[strlen(filename) + 1];
   strcpy((char*) audioState->mciOpenParms.lpstrElementName, filename);

   if((dwReturn = mciSendCommand(0, MCI_OPEN,
         MCI_OPEN_TYPE | MCI_OPEN_ELEMENT | MCI_OPEN_TYPE_ID | MCI_WAIT,
         (DWORD)(LPVOID) &(audioState->mciOpenParms))) != 0)
   {
   	// Failed to open element. Don't close it; just return error.
		ivLog->Print("      Failed to open MCI element: %d\n", dwReturn);
      mciGetErrorStringA(dwReturn, string, 1023);
     	ivLog->Print("         %s\n", string);
   	return(IVR_FAIL);
   }

   // The device opened successfully; Send device ID to simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_SET_MCI_DEVICE, audioState->mciOpenParms.wDeviceID);

   // Set MCI time format to milliseconds
   mciSetParms.dwTimeFormat = MCI_FORMAT_MILLISECONDS;

   if((dwReturn = mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_SET, MCI_SET_TIME_FORMAT | MCI_WAIT, (DWORD)(LPVOID) &mciSetParms)) != 0)
   {
      ivLog->Print("      Failed to set MCI device\n");
      ivLog->Print("      mciSendCommand MCI_SET_TIME_FORMAT failed: %d\n", dwReturn);
      mciGetErrorStringA(dwReturn, string, 1023);
     	ivLog->Print("         %s\n", string);
      mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_CLOSE, MCI_WAIT, NULL);
      return(IVR_FAIL);
   }

   // Get length of audio in milliseconds
   mciStatusParms.dwItem = MCI_STATUS_LENGTH;

   if((dwReturn = mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_STATUS, MCI_STATUS_ITEM | MCI_WAIT, (DWORD)(LPVOID) &mciStatusParms)) != 0)
   {
   	// Failed to open element. Close it and return error.
		ivLog->Print("      Failed to get MCI status: %d\n", dwReturn);
      mciGetErrorStringA(dwReturn, string, 1023);
     	ivLog->Print("         %s\n", string);
      mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_CLOSE, MCI_WAIT, NULL);
   	return(IVR_FAIL);
   }

   mciPlayParms.dwTo = mciStatusParms.dwReturn;

   // window will be notified with an MM_MCINOTIFY message when playback is complete.
   mciPlayParms.dwCallback = (DWORD) pInfo->hSimulator;

   mciPlayFlags = MCI_NOTIFY | MCI_TO;

   if(playFromTime > 0)
   {
      mciPlayFlags |= MCI_FROM;
      mciPlayParms.dwFrom = playFromTime;
   }

   if((playForTime != FOREVER) && (playFromTime + playForTime < mciPlayParms.dwTo))
   {
      mciPlayParms.dwTo = playFromTime + playForTime;
   }

   if((shortAudioPromptLength > 0) && (playFromTime + shortAudioPromptLength < mciPlayParms.dwTo))
   {
      mciPlayParms.dwTo = playFromTime + shortAudioPromptLength;
   }

   if(mciPlayParms.dwFrom > mciPlayParms.dwTo)
   {
      mciPlayParms.dwFrom = mciPlayParms.dwTo;
   }

   if((dwReturn = mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_PLAY, mciPlayFlags, (DWORD)(LPVOID) &mciPlayParms)) != 0)
   {
 	   ivLog->Print("      mciSendCommand MCI_PLAY failed: %d\n", dwReturn);
      mciGetErrorStringA(dwReturn, string, 1023);
     	ivLog->Print("         %s\n", string);
      mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_CLOSE, MCI_WAIT, NULL);
    	return(IVR_FAIL);
   }

   audioState->SetInterruptable(interruptable);
   audioState->StartPlaying(TF_WAV_8KHZ_16BIT);

   if(playForTime == 0)
   {
      AudioStopPlay();
   }

   return(IVR_SUCCESS);
}

IV_STATUS BoardSimulator::AudioPlayBuffer(char* audioBuffer, unsigned,
	bool interruptable, SOUND_FORMAT, int)
{
   IV_STATUS status;
	SimulatorBoardEvent boardEvent;

	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
   	return(IVR_FATAL);
   }

   if(interruptable)
   {
      char ttDig;

      if((status = QueryDigit(ttDig)) != IVR_SUCCESS)
      {
         return(status);
      }

      if((ttDig != '\0') || ((pInfo->speechRecognition != NULL)
         && (pInfo->speechRecognition->SRIsReceivingAudio() || pInfo->speechRecognition->SRIsPendingAudio())))
      {
         ivLog->Print("      Interrupted!\n");
         return(IVR_SUCCESS);
      }
   }

	// @@@ This is a blocking function call.
   // @@@ Need to replace it with MCI API calls.
   if(!PlaySound(audioBuffer, NULL, SND_MEMORY | SND_SYNC))
   {
      ivLog->Print("      Failed to play audio buffer\n");
      return(IVR_FAIL);
   }

   eventQueue.Add(BoardSimulatorCommunicationPacket::BRD_SIMULATOR_PLAYRECORD_DONE);

	// Wait for SIMEVT_PLAYRECORD_DONE to process
   while((status = WaitForEvent(&boardEvent)) == IVR_SUCCESS)
   {
      switch(boardEvent.SimEvent)
      {
      	case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_PLAYRECORD_DONE:
         {
            return(IVR_SUCCESS);
         }

         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
         {
            break;
         }

         default:
         {
  	   	   ivLog->Print("      ERROR: Unexpected Simulator board event %d:'%s'\n",boardEvent.SimEvent, BoardSimulatorCommunicationPacket::GetEventName(boardEvent.SimEvent));
  	         return(IVR_FAIL);
         }
      }
   }

   return(status);
}
/*BoardSimulator::AudioPlayBuffer*/

IV_STATUS BoardSimulator::AudioWaitForPlayStop(void)
{
	SimulatorBoardEvent boardEvent;
   IV_STATUS status;

   if(!audioState->IsPlaying())
   {
      return(IVR_SUCCESS);
   }

   while((status = WaitForEvent(&boardEvent)) == IVR_SUCCESS)
   {
      switch(boardEvent.SimEvent)
      {
      	case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_PLAYRECORD_DONE:
         {
            // Re-enable Barge-In button in Simulator
//            SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_ENABLE_BARGEIN_BUTTON);

            audioState->SetPlayedEntireAudio();

         	return(IVR_SUCCESS);
         }

         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
         {
            if(audioState->IsInterruptable())
            {
               ivLog->Print("      Interrupted by touch tone\n");

               return(AudioStopPlay());
            }

            break;
         }

//         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_BARGEIN:
//         {
//            ivLog->Print("      Interrupted by speech\n");
//            if((status = AudioStopPlay()) != IVR_SUCCESS)
//            {
//               return(status);
//            }
//
//            return(StartBargeInRecord());
//         }

         default:
         {
  	   	   ivLog->Print("      ERROR: Unexpected Simulator board event %d:'%s'\n",boardEvent.SimEvent, BoardSimulatorCommunicationPacket::GetEventName(boardEvent.SimEvent));
  	         return(IVR_FAIL);
         }
      }
   }

   AudioStopPlay();

   return(status);
}

IV_STATUS BoardSimulator::AudioStopPlay(void)
{
   DWORD dwReturn;
   char string[1024];

   if((dwReturn = mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_STOP, MCI_WAIT, NULL)) != 0)
   {
      ivLog->Print("      mciSendCommand MCI_STOP failed: %d\n", dwReturn);
      mciGetErrorStringA(dwReturn, string, 1023);
      ivLog->Print("         %s\n", string);
      mciSendCommand(audioState->mciOpenParms.wDeviceID, MCI_CLOSE, MCI_WAIT, NULL);
      return(IVR_FAIL);
   }

   return(AudioWaitForPlayStop());
}

IV_STATUS BoardSimulator::AudioRecognise(int, int)
{
   return(IVR_NOTSUPPORTED);
}
/*
IV_STATUS BoardSimulator::AudioRecognise(int, int)
{
	IV_STATUS status;
   SimulatorBoardEvent boardEvent;
   char ttDig;

	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
   	return(IVR_FATAL);
   }

   if(pInfo->speechRecognition == NULL)
   {
      ivLog->Print("      ERROR: No speech recognition module loaded\n");
      ivLog->Print("         Make sure to use SR Acquire before SR steps\n");
      return(IVR_FATAL);
   }

   if((status = QueryDigit(ttDig)) != IVR_SUCCESS)
   {
      return(status);
   }

   if((ttDig != '\0') || pInfo->speechRecognition->SRIsPendingAudio())
   {
      ivLog->Print("      Interrupted!\n");
      return(ttDig != '\0' ? IVR_NO_VOICE : IVR_SUCCESS);
   }

   if(!audioState->IsBargeInActive() && !pInfo->speechRecognition->SRIsReceivingAudio())
   {
      StartBargeInRecord();
   }

	while((status = WaitForEvent(&boardEvent, FOREVER)) == IVR_SUCCESS)
   {
   	switch(boardEvent.SimEvent)
      {
         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_INTERRUPTPLAYRECORD:
         {
            if((status = StopBargeInRecord()) != IVR_SUCCESS)
            {
               return(status);
            }

            status = pInfo->speechRecognition->SRStopAudio();

            // Enable Barge-In button in Simulator
            if(audioState->IsBargeInPermitted())
            {
               ActivateBargeIn();
            }

            return(status);
         }

         case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
         {
            ivLog->Print("      Interrupted by touch tone\n");

            if((status = StopBargeInRecord()) != IVR_SUCCESS)
            {
               return(status);
            }

            if((status = pInfo->speechRecognition->SRStopAudio()) != IVR_SUCCESS)
            {
               return(status);
            }

            return(IVR_NO_VOICE);
         }

         default:
         {
  	   	   ivLog->Print("      ERROR: Unexpected Simulator board event %d:'%s'\n",boardEvent.SimEvent, BoardSimulatorCommunicationPacket::GetEventName(boardEvent.SimEvent));
  	         return(IVR_FAIL);
         }
      }
   }

   return(status);
}

IV_STATUS BoardSimulator::ActivateBargeIn(void)
{
   if(!lineOffHook || (pInfo->speechRecognition == NULL))
   {
      return(IVR_SUCCESS);
   }

   // Make Barge-In button visible in Simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_SHOW_BARGEIN_BUTTON);

   // Enable Barge-In button in Simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_ENABLE_BARGEIN_BUTTON);

   audioState->ActivateBargeIn();

   return(IVR_SUCCESS);
}

IV_STATUS BoardSimulator::StartBargeInRecord(void)
{
   MMRESULT mmResult;
   IV_STATUS status;
   char* audioChunk;

   if(pInfo->speechRecognition == NULL)
   {
      ivLog->Print("      ERROR: No speech recognition module loaded\n");
      ivLog->Print("         Make sure to use SR Acquire before SR steps\n");
      return(IVR_FATAL);
   }

   if((status = pInfo->speechRecognition->SRStartAudio()) != IVR_SUCCESS)
   {
      return(status);
   }

   audioChunk = pInfo->speechRecognition->SRGetNextAudioChunk();
   audioState->AddBuffer(audioChunk, pInfo->speechRecognition->SRGetAudioChunkSize());
   audioState->AddBuffer(pInfo->speechRecognition->SRGetNextAudioChunk(), pInfo->speechRecognition->SRGetAudioChunkSize());

   if((mmResult = waveInStart(audioState->hwi)) != MMSYSERR_NOERROR)
   {
      ivLog->Print("      ERROR: Unable to start record\n");
      ivLog->Print("         %s\n", MMError(mmResult));
      pInfo->speechRecognition->SRClearAudio();
      return(IVR_FAIL);
   }

   pInfo->speechRecognition->SRSetAudioStart(audioChunk);
   audioState->StartRecording(NULL, GetSRAudioFormat());

   // Hide Barge-In button in Simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_HIDE_BARGEIN_BUTTON);

   // Make StopRecordButton visible in Simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_SHOW_STOP_RECORDING_BUTTON);

   return(IVR_SUCCESS);
}

IV_STATUS BoardSimulator::DeactivateBargeIn(void)
{
   // Hide Barge-In button in Simulator
   SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_HIDE_BARGEIN_BUTTON);

   audioState->DeactivateBargeIn();

   if(audioState->IsBargeInActive())
   {
      return(StopBargeInRecord());
   }

   return(IVR_SUCCESS);
}

IV_STATUS BoardSimulator::StopBargeInRecord(void)
{
   MMRESULT mmResult;

   if(pInfo->speechRecognition == NULL)
   {
      ivLog->Print("      ERROR: No speech recognition module loaded\n");
      ivLog->Print("         Make sure to use SR Acquire before SR steps\n");
      return(IVR_FATAL);
   }

   mmResult = waveInStop(audioState->hwi);
   if(mmResult != MMSYSERR_NOERROR)
   {
      ivLog->Print("      ERROR: Unable to stop record\n");
      ivLog->Print("         %s\n", MMError(mmResult));
      return(IVR_FAIL);
   }

   mmResult = waveInReset(audioState->hwi);
   if(mmResult != MMSYSERR_NOERROR)
   {
      ivLog->Print("      ERROR: Unable to reset SR buffers\n");
      ivLog->Print("         %s\n", MMError(mmResult));
      return(IVR_FAIL);
   }

   pInfo->speechRecognition->SRStopAudio();

   return(IVR_SUCCESS);
}
*/
IV_STATUS BoardSimulator::AudioPlayEvalMessage(void)
{
	return(IVR_SUCCESS);
}

IV_STATUS BoardSimulator::HTTPRequest(HTTPTransaction& http, int timeout, int connectionTimeout,
   int connectionRetries, const char* filename, SOUND_FORMAT format, int volLevel)
{
   LARGE_INTEGER liTimeout;
   IV_STATUS status;
   SimulatorBoardEvent boardEvent;
   ESCTime startTime;
   ESCTime silenceStartTime;
   ESCTime currentTime;
   int ret;
   bool loopAgain;
   bool attemptConnect;
   int connectCounter = 0;

	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
   	return(IVR_FATAL);
   }

   // Start playing file here
   ivLog->Print("      Playing '%s'\n", filename);
   status = StartPlayFile(filename, false, format, volLevel);
   if(status != IVR_SUCCESS)
   {
      ivLog->Print("      ERROR: Failed to play file '%s'\n", filename);
      return(status);
   }

   startTime.Update();
   
	// Set connection timeout timer
   if(connectionTimeout != FOREVER)
   {
      liTimeout.QuadPart = -(LONGLONG) connectionTimeout*10000;
      SetWaitableTimer(hGenericTimer, &liTimeout, 0, 0, 0, false);
   }

   // Set Generic Event to socket handle
   SetGenericEvent(http.GetEventHandle());

   // Try up to iConnectionRetries times to connect
   loopAgain = true;
   attemptConnect = true;
   while(loopAgain)
   {
      // Connect to HTTP server
      if(attemptConnect)
      {
         ivLog->Print("      Connection attempt %d\n", ++connectCounter);

         if((ret = http.Connect()) != Socket::SUCCESS)
         {
            const char* errorMessage = Socket::WSAErrorString(ret);

            ivLog->Print("      ERROR: Unable to connect to host\n");
            ivLog->Print("         '%s'\n", errorMessage);
           	CancelWaitableTimer(hGenericTimer);
            ClearGenericEvent();
            http.Response()->SetText(errorMessage);
            // Wait for audio to stop playing
            if(audioState->IsPlaying() && (status = AudioWaitForPlayStop()) != IVR_SUCCESS)
            {
               return(status);
            }
            return(IVR_FAIL);
         }
      }

      status = WaitForEvent(&boardEvent);

      switch(status)
      {
         // Board event
         case IVR_SUCCESS:
         {
         	switch(boardEvent.SimEvent)
            {
               case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_PLAYRECORD_DONE:
               {
                  // Reset timer if necessary
                  if(connectionTimeout != FOREVER)
                  {
                     silenceStartTime.Update();

                     if(timeout < connectCounter*connectionTimeout - (int) (1000*silenceStartTime.GetSecondsSince(startTime)))
                     {
                        liTimeout.QuadPart = -(LONGLONG) timeout * 10000;
                        SetWaitableTimer(hGenericTimer, &liTimeout, 0, 0, 0, false);
                     }
                  }
                  else if(timeout != FOREVER)
                  {
                     liTimeout.QuadPart = -(LONGLONG) timeout * 10000;
                     SetWaitableTimer(hGenericTimer, &liTimeout, 0, 0, 0, false);
                  }

                  break;
   		   	}

               case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
               {
                  break;
               }

               default:
               {
        	   	   ivLog->Print("      ERROR: Unexpected Simulator board event %d:'%s'\n",boardEvent.SimEvent, BoardSimulatorCommunicationPacket::GetEventName(boardEvent.SimEvent));
                 	CancelWaitableTimer(hGenericTimer);
                  ClearGenericEvent();
                  return(IVR_FAIL);
               }
            }

            break;
         }

         // Connection signalled
         case IVR_GENERIC_EVENT:
         {
            // Check status
            if((ret = http.GetStatus()) == Socket::SUCCESS)
            {
               ivLog->Print("      Connection successful\n");

               http.SelectEvents(FD_READ);

               loopAgain = false;

               if(silenceStartTime.IsValid())
               {
                  liTimeout.QuadPart = -(LONGLONG) (timeout  - (int) (1000*silenceStartTime.GetSecondsSince(startTime))) * 10000;
                  SetWaitableTimer(hGenericTimer, &liTimeout, 0, 0, 0, false);
               }
               else
               {
                 	CancelWaitableTimer(hGenericTimer);
               }
            }
            else
            {
               const char* errorMessage = Socket::WSAErrorString(ret);

               ivLog->Print("      Connection attempt failed\n");
               ivLog->Print("         '%s'\n", errorMessage);

               if(connectCounter > connectionRetries)
               {
                  http.Response()->SetText(errorMessage);
                  // Wait for audio to stop playing
                  if(audioState->IsPlaying() && (status = AudioWaitForPlayStop()) != IVR_SUCCESS)
                  {
                     return(status);
                  }
                  return(IVR_FAIL);
               }

               http.Close();
               attemptConnect = true;
            }

            break;
         }

         // Timeout
         case IVR_GENERIC_TIMEOUT:
         {
            int updatedTimeout;

             ivLog->Print("      ERROR: Timed out trying to connect to host\n");

            if((timeout != FOREVER) && silenceStartTime.IsValid())
            {
               currentTime.Update();
               updatedTimeout = timeout - (int) (1000*currentTime.GetSecondsSince(silenceStartTime));
            }
            else
            {
               updatedTimeout = connectionTimeout;
            }

            // Silence timeout or maximum connection attempts
            if((updatedTimeout <= 0) || (connectCounter > connectionRetries))
            {
              	CancelWaitableTimer(hGenericTimer);
               ClearGenericEvent();
               // Wait for file to stop playing
               if(audioState->IsPlaying() && ((status = AudioWaitForPlayStop()) != IVR_SUCCESS))
               {
                  return(status);
               }
               return(IVR_TIMEOUT);
            }
            // Reset timeout timer
            else
            {
               liTimeout.QuadPart = -(LONGLONG) (updatedTimeout < connectionTimeout ? updatedTimeout : connectionTimeout)*10000;
               SetWaitableTimer(hGenericTimer, &liTimeout, 0, 0, 0, false);
            }

            http.Close();
            attemptConnect = true;
            break;
         }

         default:
         {
            if((status != IVR_HANGUP) && (status != IVR_SHUTDOWN))
            {
               ivLog->Print("      ERROR: Unexpected problem waiting for connection event\n");
            }
           	CancelWaitableTimer(hGenericTimer);
            ClearGenericEvent();
            if(audioState->IsPlaying())
            {
               AudioStopPlay();
            }
            return(status);
         }
      }
   }

   ivLog->Print("      Sending request: '%s'\n", http.Request()->GetText());

   // Send the request
   while((ret = http.Send()) == WSAEWOULDBLOCK)
   {
      // Allow 10 ms for socket to clear
      Sleep(10);
   }

   if(ret != Socket::SUCCESS)
   {
      const char* errorMessage = Socket::WSAErrorString(ret);

      ivLog->Print("      ERROR: Failed to send request\n");
      ivLog->Print("         '%s'\n", errorMessage);
      CancelWaitableTimer(hGenericTimer);
      ClearGenericEvent();
      http.Response()->SetText(errorMessage);
      if(audioState->IsPlaying() && ((status = AudioWaitForPlayStop()) != IVR_SUCCESS))
      {
         return(status);
      }
      return(IVR_FAIL);
   }

   ivLog->Print("      Request successful, waiting for response\n");

   loopAgain = true;
   while(loopAgain)
   {
      status = WaitForEvent(&boardEvent);

      switch(status)
      {
         // Board event
         case IVR_SUCCESS:
         {
         	switch(boardEvent.SimEvent)
            {
               case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_PLAYRECORD_DONE:
               {
               	// Set timeout timer
                  if(timeout != FOREVER)
                  {
                     liTimeout.QuadPart = -(LONGLONG) timeout * 10000;
                     SetWaitableTimer(hGenericTimer, &liTimeout, 0, 0, 0, false);
                  }

                  break;
   		   	}

               case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
               {
                  break;
               }

               default:
               {
        	   	   ivLog->Print("      ERROR: Unexpected Simulator board event %d:'%s'\n",boardEvent.SimEvent, BoardSimulatorCommunicationPacket::GetEventName(boardEvent.SimEvent));
                 	CancelWaitableTimer(hGenericTimer);
                  ClearGenericEvent();
                  return(IVR_FAIL);
               }
            }

            break;
         }

         // Socket signalled
         case IVR_GENERIC_EVENT:
         {
            if((ret = http.Receive()) == Socket::SUCCESS)
            {
               loopAgain = false;
            }
            else if(ret == WSAEWOULDBLOCK)
            {
               // Allow 10 ms for socket to clear
               Sleep(10);
            }
            else
            {
               const char* errorMessage = Socket::WSAErrorString(ret);

               ivLog->Print("      ERROR: Failed to receive response\n");
               ivLog->Print("         '%s'\n", errorMessage);
              	CancelWaitableTimer(hGenericTimer);
               ClearGenericEvent();
               http.Response()->SetText(errorMessage);
               if(audioState->IsPlaying() && ((status = AudioWaitForPlayStop()) != IVR_SUCCESS))
               {
                  return(status);
               }
               return(IVR_FAIL);
            }

            break;
         }

         // Timed out
         case IVR_GENERIC_TIMEOUT:
         {
            ivLog->Print("      ERROR: Timed out waiting for response\n");
           	CancelWaitableTimer(hGenericTimer);
            ClearGenericEvent();
            // Wait for audio to stop playing
            if(audioState->IsPlaying() && (status = AudioWaitForPlayStop()) != IVR_SUCCESS)
            {
               return(status);
            }
            return(IVR_TIMEOUT);
         }

         default:
         {
            if((status != IVR_HANGUP) && (status != IVR_SHUTDOWN))
            {
               ivLog->Print("      ERROR: Unexpected problem waiting for initial response event\n");
            }
           	CancelWaitableTimer(hGenericTimer);
            ClearGenericEvent();
            if(audioState->IsPlaying())
            {
               AudioStopPlay();
            }
            return(status);
         }
      }
   }

   // Check that response was OK (20x)
   // Bad return code: not 200 to 209
   if((http.Response()->GetStatusCode() < 200) || (http.Response()->GetStatusCode() > 209))
   {
      ivLog->Print("      ERROR: '%s'\n", http.Response()->GetReasonPhrase());
    	CancelWaitableTimer(hGenericTimer);
      ClearGenericEvent();
      if(audioState->IsPlaying() && ((status = AudioWaitForPlayStop()) != IVR_SUCCESS))
      {
         return(status);
      }
      return(IVR_FAIL);
   }

   // Don't have whole message yet
   while(!http.Response()->HaveCompleteMessage())
   {
      status = WaitForEvent(&boardEvent);

      switch(status)
      {
         // Board event
         case IVR_SUCCESS:
         {
         	switch(boardEvent.SimEvent)
            {
               case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_PLAYRECORD_DONE:
               {
               	// Set timeout timer
                  if(timeout != FOREVER)
                  {
                     liTimeout.QuadPart = -(LONGLONG) timeout * 10000;
                     SetWaitableTimer(hGenericTimer, &liTimeout, 0, 0, 0, false);
                  }

                  break;
   		   	}

               case BoardSimulatorCommunicationPacket::BRD_SIMULATOR_DIGIT:
               {
                  break;
               }

               default:
               {
        	   	   ivLog->Print("      ERROR: Unexpected Simulator board event %d:'%s'\n",boardEvent.SimEvent, BoardSimulatorCommunicationPacket::GetEventName(boardEvent.SimEvent));
                 	CancelWaitableTimer(hGenericTimer);
                  ClearGenericEvent();
                  return(IVR_FAIL);
               }
            }

            break;
         }

         // Socket signalled
         case IVR_GENERIC_EVENT:
         {
            if((ret = http.Receive()) == WSAEWOULDBLOCK)
            {
               // Allow 10 ms for socket to clear
               Sleep(10);
            }
            else if(ret != Socket::SUCCESS)
            {
               const char* errorMessage = Socket::WSAErrorString(ret);

               ivLog->Print("      ERROR: Failed to receive remainder of response\n");
               ivLog->Print("         '%s'\n", errorMessage);
              	CancelWaitableTimer(hGenericTimer);
               ClearGenericEvent();
               http.Response()->SetText(errorMessage);
               if(audioState->IsPlaying() && ((status = AudioWaitForPlayStop()) != IVR_SUCCESS))
               {
                  return(status);
               }
               return(IVR_FAIL);
            }

            break;
         }

         // Timeout
         case IVR_GENERIC_TIMEOUT:
         {
            ivLog->Print("      ERROR: Timed out waiting for remainder of response\n");
           	CancelWaitableTimer(hGenericTimer);
            ClearGenericEvent();
            // Wait for audio to stop playing
            if(audioState->IsPlaying() && (status = AudioWaitForPlayStop()) != IVR_SUCCESS)
            {
               return(status);
            }
            return(IVR_TIMEOUT);
         }

         default:
         {
            if((status != IVR_HANGUP) && (status != IVR_SHUTDOWN))
            {
               ivLog->Print("      ERROR: Unexpected problem waiting for complete response event\n");
            }
           	CancelWaitableTimer(hGenericTimer);
            ClearGenericEvent();
            if(audioState->IsPlaying())
            {
               AudioStopPlay();
            }
            return(status);
         }
      }
   }

   ivLog->Print("      Response received\n");

  	CancelWaitableTimer(hGenericTimer);
   ClearGenericEvent();

   // Wait for file to stop playing
   if(audioState->IsPlaying() && ((status = AudioWaitForPlayStop()) != IVR_SUCCESS))
   {
      return(status);
   }

   return(IVR_SUCCESS);
}

