/* -------------------------------------------------------------------------
                Copyright (c) Engage Software Corporation 1994 - 2000.
                All rights reserved.
   ------------------------------------------------------------------------- */
#ifndef _BRD_SIMULATOR_H
#define _BRD_SIMULATOR_H

#include <mmsystem.h>	// MCI stuff
#include "..\TFServer\signalevt.h"
#include "brd_generic.h"
#include "brd_simulator_communication.h"
#include "..\TFSimulator\TFSimulator_communication.h"

class SimulatorBoardEvent : public BoardEvent
{
	public:

 		SimulatorBoardEvent(void);
 		~SimulatorBoardEvent(void);

      bool SameEvent(BoardEvent* comparisonEvent);

   	BoardSimulatorCommunicationPacket::BRD_SIMULATOR_EVENT SimEvent;
};

class SimulatorAudioState : public AudioState
{
   public:

      SimulatorAudioState(CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>* commHub, HANDLE hSimulator);

      WAVEHDR* AddBuffer(char* audioBuffer, unsigned int audioBufferSize);

      MCI_OPEN_PARMS mciOpenParms;
      WAVEFORMATEX wfx;
      HWAVEIN hwi;

   protected:

      WAVEHDR bufferHeaders[10];
      unsigned int currentBufferHeader;
};

//IV_STATUS Wait (int timeout);     // Length of time to wait in 1/1000 seconds.

/* -------------------------------------------------------------------------
   Class:       BoardSimulator

   Purpose:     Implements all the functions defined in "BoardGeneric"
                class.
   ------------------------------------------------------------------------- */
class BoardSimulator : public BoardGeneric
{
	public:
	   // ---- CONSTRUCTOR / DESTRUCTOR -------------------------------------------
   	BoardSimulator(PIVRINFO pIVRInfo);
   	~BoardSimulator(void);
	   // Prevent compiler from creating default constructor, default copy
   	// constructor, and assignment operator.
	   BoardSimulator(void);
   	BoardSimulator(const BoardSimulator&);
	   BoardSimulator& operator=(const BoardSimulator&);

	   // ---- CLASS FUNCTIONS ----------------------------------------------------
      IV_STATUS ProcessSystemEvent(void);
      IV_STATUS StartReorderToneDetection(void);
      IV_STATUS StopReorderToneDetection(void);
      IV_STATUS Wait(int timeout, bool interruptable);

	   // ---- PARAMETER FUNCTIONS ------------------------------------------------
   	IV_STATUS SetParameter(const char* pszParm, DWORD dwParmValue);

	   // ---- LINE FUNCTIONS -----------------------------------------------------
   	IV_STATUS BlockCalls(void);

      // flashTime of 0 skips flash
	   IV_STATUS FlashHook(long flashTime);

	   IV_STATUS Hangup(bool returnHangup = false, char* pszCause = NULL);
		IV_STATUS Pickup(void);
	   IV_STATUS PlaceCall(const char *digits, const char* callerID, int numrings,
         callMode mode, int flashdur, bool connectOnCED, bool connectOnSIT, int* pVoiceLength,
         int silenceTimeout, int maxVoiceTimeout,
         ISDNNumberPlan callingNumPlan = UNKNOWN_NUMBER_PLAN,
         ISDNNumberType callingNumType = UNKNOWN_NUMBER_TYPE,
         ISDNNumberPlan calledNumPlan = UNKNOWN_NUMBER_PLAN,
         ISDNNumberType calledNumType = UNKNOWN_NUMBER_TYPE);
	   IV_STATUS UnblockCalls(void);
      IV_STATUS WaitForCall(long timeout, int numrings, bool pickup, char* calledDigits,
         char* callingDigits, char* handsetDisplay, int numdigits = 0);

	   // ---- T1 LINE FUNCTIONS --------------------------------------------------
      IV_STATUS Switch(int newPort, bool bListen);
      IV_STATUS UnSwitch(void);

	   // ---- TT FUNCTIONS -------------------------------------------------------
   	IV_STATUS FlushDigits(void);
	   IV_STATUS GenerateDigits(const char* ttStr);
   	IV_STATUS QueryDigit(char& digit);

      // timeout of 0 inidicates returns immediately with IVR_SUCCESS if digit
      // has already occurred or IVR_TIMEOUT if no digit has occurred
      // count must be > 0 or INFINITE
      // digits buffer is populated by GetDigits and the client is repsonible for
      // deleting the buffer
      // Digits A,B,C, and D are in upper case
      IV_STATUS GetDigits(
      	unsigned count,     // Number of digits to collect.
	      char*&   digits,    // Digit buffer returned from GetDigits
   	   int      newTime,   // Timeout between key presses.
      	const char*	ttOkay,    // Digits that are valid.
	      const char*	ttDone);   // Digits that terminate TT collection.

	   // ---- AUDIO FUNCTIONS ----------------------------------------------------
   	IV_STATUS AudioBeep(bool interruptable);

      // maxRecTime of 0 records an empty or short as possible sound file
      // maxRecTime of FOREVER indicates record until interrupt or silence timeout
      IV_STATUS StartRecordFile(const char* filename, bool interruptable,
         SOUND_FORMAT format, int volLevel, int maxRecTime, int maxSilence,
         bool disableBeep, bool automaticGainControl);

      IV_STATUS AudioStopRecord(void);
      IV_STATUS AudioWaitForRecordStop(void);

      // playForTime of 0 indicates do not play file
      // playForTime of FOREVER indicates play until end of file
	   IV_STATUS StartPlayFile(const char* filename, bool interruptable, SOUND_FORMAT format,
         int volLevel, unsigned int playFromTime = 0, unsigned int playForTime = FOREVER);

      IV_STATUS AudioPlayBuffer(char* audioBuffer, unsigned bufferSize, bool interruptable,
      	SOUND_FORMAT format, int volLevel);

	   IV_STATUS AudioStopPlay(void);
      IV_STATUS AudioWaitForPlayStop(void);

	   IV_STATUS AudioRecognise(int noVoiceTimeout, int endOfVoiceTimeout);
//      IV_STATUS ActivateBargeIn(void);
//      IV_STATUS StartBargeInRecord(void);
//      IV_STATUS DeactivateBargeIn(void);
//      IV_STATUS StopBargeInRecord(void);

      IV_STATUS AudioPlayEvalMessage(void);
      IV_STATUS HTTPRequest(HTTPTransaction& http, int timeout, int connectionTimeout,
         int connectionRetries, const char* filename, SOUND_FORMAT format, int volLevel);

	protected:

      MCIDEVICEID wDeviceID;
      SignalQueue<char> ttQueue;                                                     // Queue for touch tones;
	   SignalQueue<BoardSimulatorCommunicationPacket::BRD_SIMULATOR_EVENT> eventQueue;    // Queue for Simulator events.
      SimulatorAudioState* audioState;
      int shortAudioPromptLength;
      const char* startingSRBuffer;
      const char* currentSRBuffer;

      bool blocked;

      IV_STATUS ProcessBoardMessage(BoardCommunicationPacket* boardMessage);
		IV_STATUS ProcessBoardEvent(BoardEvent* boardEvent, bool& bFound);
      void SendEvent(TFSimulatorCommunicationPacket::TF_SIMULATOR_EVENT event, int eventInfo = 0);

   // **** INITIALIZE FUNCTIONS ********************************************
   bool QueryBoard(void);
   bool AddLines (unsigned boardNo, unsigned lineCount);
};

#endif
