#ifndef __STRINGSUPPORT_H__
#define __STRINGSUPPORT_H__

#include <string>
#include <vector>

// FormattedString returns the formatted string provided the format specified in
// fmt, and the variables in the following parameters.

const char *FormattedString( const char* fmt, ... );

// Removes the preceding and following characters padding from the string input.

void RemovePadding( std::string &input, char padding = ' ' );

// Tokenize the input string while using the separator char. Puts all individual
// tokens in the tokens vector. Removes the spaces from the tokens if removeSpaces
// is true, and may limit the amount of tokens looked for if maxTokens is positive.
// Will return the amount of tokens found in the return of the function.

int Tokenize( std::string input, const char *separator,
   std::vector<std::string> *tokens, bool removeSpaces = true, int maxTokens = -1 );

// Generates a random string in the return of the function with the specified
// length while asigning each individual character between useLower and useUpper.

std::string GenerateUniqueString( unsigned int length, char useLower,
   char useUpper );

// Searches for toReplace in source and replaces all occurence with replaceWith.

void SearchAndReplace( std::string& source, const std::string& toReplace,
   const std::string& replaceWith );

// Modifies input character so that it is between normalizeFromChar and
// normalizeToChar and returns the normalized value.

unsigned char NormalizeChar( int input, unsigned char normalizeFromChar,
   unsigned char normalizeToChar );

// Puts the source string in lowercase.

void LowercaseString( std::string& source );

// Returns the substring of source up to the delimitorString into source, or an
// empty string if delimitorString can't be found into source.

std::string SubstringUpTo( const std::string source,
   const std::string delimitorString );

#endif