#pragma hdrstop
#include "..\BoardDLLs\brd_generic.h"
#include "step_SetReorderTone.h"

StepSetReorderTone::StepSetReorderTone(STEPREC& stepRecord) : Step(stepRecord)
{
   frequency1 = stepRecord.s1.arr;
   frequency2 = stepRecord.s2.arr;
   minimumOnTime = stepRecord.s3.arr;
   maximumOnTime = stepRecord.s4.arr;
   minimumOffTime = stepRecord.s5.arr;
   maximumOffTime = stepRecord.s6.arr;
}

StepSetReorderTone::~StepSetReorderTone(void)
{
}

const char* StepSetReorderTone::GetStepName(void) const
{
	return("Set Reorder Tone");
}

IV_STATUS StepSetReorderTone::Execute(PIVRINFO pInfo)
{
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   int iFrequency1 = 0;
   int iFrequency2 = 0;
   int iMinimumOnTime;
   int iMaximumOnTime;
   int iMinimumOffTime;
   int iMaximumOffTime;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   if((value = pInfo->variables->ParseParameter("Frequency 1", frequency1.c_str(), "0")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iFrequency1 = atoi(value);
      delete[] value;

      if(iFrequency1 < 0)
      {
         pInfo->ivLog->Print("      ERROR: Frequency 1 '%s' cannot be < 0\n", frequency1.c_str());
         bParmsValid = false;
      }
   }

   if((value = pInfo->variables->ParseParameter("Frequency 2", frequency2.c_str(), "0")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iFrequency2 = atoi(value);
      delete[] value;

      if(iFrequency2 < 0)
      {
         pInfo->ivLog->Print("      ERROR: Frequency 2 '%s' cannot be < 0\n", frequency2.c_str());
         bParmsValid = false;
      }
   }

   if((iFrequency1 == 0) && (iFrequency2 == 0))
   {
      pInfo->ivLog->Print("      ERROR: One frequency, '%s' or '%s', must be > 0\n", frequency1.c_str(), frequency2.c_str());
      bParmsValid = false;
   }

   if((value = pInfo->variables->ParseParameter("Minimum On Time", minimumOnTime.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iMinimumOnTime = atoi(value);
      delete[] value;

      if(iMinimumOnTime <= 0)
      {
         pInfo->ivLog->Print("      ERROR: Minimum On Time '%s' must be > 0\n", minimumOnTime.c_str());
         bParmsValid = false;
      }
   }

   if((value = pInfo->variables->ParseParameter("Maximum On Time", maximumOnTime.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iMaximumOnTime = atoi(value);
      delete[] value;

      if(iMaximumOnTime <= 0)
      {
         pInfo->ivLog->Print("      ERROR: Maximum On Time '%s' must be > 0\n", maximumOnTime.c_str());
         bParmsValid = false;
      }
   }

   if((value = pInfo->variables->ParseParameter("Minimum Off Time", minimumOffTime.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iMinimumOffTime = atoi(value);
      delete[] value;

      if(iMinimumOffTime <= 0)
      {
         pInfo->ivLog->Print("      ERROR: Minimum Off Time '%s' must be > 0\n", minimumOffTime.c_str());
         bParmsValid = false;
      }
   }

   if((value = pInfo->variables->ParseParameter("Maximum Off Time", maximumOffTime.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iMaximumOffTime = atoi(value);
      delete[] value;

      if(iMaximumOffTime <= 0)
      {
         pInfo->ivLog->Print("      ERROR: Maximum Off Time '%s' must be > 0\n", maximumOffTime.c_str());
         bParmsValid = false;
      }
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }
   
   pInfo->ivLog->Print("      Setting frequency 1 to %d Hz\n", iFrequency1);
   pInfo->ivLog->Print("      Setting frequency 2 to %d Hz\n", iFrequency2);
   pInfo->ivLog->Print("      Setting minimum on time to %d ms\n", iMinimumOnTime);
   pInfo->ivLog->Print("      Setting maximum on time to %d ms\n", iMaximumOnTime);
   pInfo->ivLog->Print("      Setting minimum off time to %d ms\n", iMinimumOffTime);
   pInfo->ivLog->Print("      Setting maximum off time to %d ms\n", iMaximumOffTime);

   return(pInfo->ivBoard->SetReorderTone(iFrequency1, iFrequency2, iMinimumOnTime,
      iMaximumOnTime, iMinimumOffTime, iMaximumOffTime));
 }
