#include "..\inet\Socket.h"
#include "..\BoardDLLs\brd_generic.h"  // ProcessSystemEvent()
#include "step_UDPIPBind.h"

StepUDPIPBind::StepUDPIPBind(STEPREC& stepRecord) : Step(stepRecord)
{
   port = stepRecord.s2.arr;
   socketHandleVariable = stepRecord.s1.arr;
}

StepUDPIPBind::~StepUDPIPBind(void)
{
}

const char* StepUDPIPBind::GetStepName(void) const
{
	return("UDP Bind");
}

IV_STATUS StepUDPIPBind::Execute(PIVRINFO pInfo)
{
   char* value;
   unsigned short uPort;
   UDPSocket socket(pInfo->ivLog, Socket::ANY_PORT, true);
   int ret;
   char number[14];
   ACE_INET_Addr localAddr;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Port", port.c_str(),
      "", "Empty Port defaults to any available port")) == NULL)
   {
      return(IVR_FATAL);
   }
   else
   {
      uPort = (unsigned short) atoi(value);

      if((uPort == Socket::ANY_PORT) && (strlen(value) != 0))
      {
         pInfo->ivLog->Print("      Port '0' evaluates to any available port\n");
      }

      delete[] value;

      if((ret = socket.Bind(uPort)) != Socket::SUCCESS)
      {
         pInfo->ivLog->Print("      ERROR: Failed to bind socket to port %d\n", uPort);
         pInfo->ivLog->Print("         %s\n", Socket::WSAErrorString(ret));
         socket.Close();
         return(IVR_FAIL);
      }
   }

   socket.get_local_addr(localAddr);
   pInfo->ivLog->Print("      Bound to port %d\n", localAddr.get_port_number());

   sprintf(number, "%d", socket.get_handle());

   if(!pInfo->variables->Set(socketHandleVariable.c_str(), number))
   {
      pInfo->ivLog->Print("      ERROR: Failed to set Socket Handle variable '%s'\n", socketHandleVariable.c_str());
      socket.Close();
      return(IVR_FATAL);
   }

   return(IVR_SUCCESS);
 }
