#include <string.h>
#include "step_audiocopy.h"
#include "..\escutil\iv_str.h"				// strtrim
#include "..\BoardDLLs\brd_generic.h"

StepAudioCopy::StepAudioCopy(STEPREC& stepRecord) : Step(stepRecord)
{
   fromFileNm = stepRecord.s6.arr;
   toFileNm = stepRecord.s3.arr;
}

StepAudioCopy::~StepAudioCopy(void)
{
}

const char* StepAudioCopy::GetStepName(void) const
{
	return("Copy File");
}

// -------------------------------------------------------------------------
//   Name:        StepAudioCopy::Execute
//
//   Questions:   - If the user specifies a play code, why try and read in the
//                  file name?
// ------------------------------------------------------------------------- 
IV_STATUS StepAudioCopy::Execute(PIVRINFO pInfo)
{
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   string parsedFromFileNm;
   string parsedToFileNm;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Source Filename", fromFileNm.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedFromFileNm = value;
      delete[] value;

      parsedFromFileNm = value = pInfo->variables->FilePath(parsedFromFileNm.c_str());
      delete[] value;

      pInfo->ivLog->Print("      Source path is '%s'\n", parsedFromFileNm.c_str());
   }

   if((value = pInfo->variables->ParseParameter("Destination Filename", toFileNm.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedToFileNm = value;
      delete[] value;

      parsedToFileNm = value = pInfo->variables->FilePath(parsedToFileNm.c_str());
      delete[] value;

      pInfo->ivLog->Print("      Destination path is '%s'\n", parsedToFileNm.c_str());
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }
   
   if(!CopyFile(parsedFromFileNm.c_str(), parsedToFileNm.c_str(), false))
   {
      DWORD errorID = GetLastError();
      char MsgBuf[1024];
      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, errorID, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), MsgBuf, 1024, 0);
      pInfo->ivLog->Print("      ERROR: WaitForEvent returned: %x: %s", errorID, MsgBuf);
      return(IVR_FAIL);
   }

   return(IVR_SUCCESS);
}
// StepAudioCopy::Execute
