#include "step_filewrite.h"

StepFileWrite::StepFileWrite(STEPREC& stepRecord) : Step(stepRecord)
{
   fileName = stepRecord.s1.arr;
   varName = stepRecord.s2.arr;
   bBase64Decode = stepRecord.u1;
}

StepFileWrite::~StepFileWrite(void)
{
}

const char* StepFileWrite::GetStepName(void) const
{
	return("Write File");
}

IV_STATUS StepFileWrite::Execute(PIVRINFO pInfo)
{
   IV_STATUS status;
   char* value;
   char* pathName;
   bool bParmsValid = true;   // to evaluate all parameters before exiting

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("File", fileName.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      pathName = pInfo->variables->FilePath(value);
      delete[] value;
   }

   if(!pInfo->variables->IsVariable(varName.c_str()))
   {
      pInfo->ivLog->Print("      ERROR: Variable '%s' is undefined\n", varName.c_str());
      bParmsValid = false;
   }
   else
   {
      pInfo->ivLog->Print("      Write contents of %s\n", varName.c_str());
   }

   if(bParmsValid == false)
   {
      return(IVR_FATAL);
   }

   if(bBase64Decode)
   {
      pInfo->ivLog->Print("      Decode from Base64\n");
   }

   status = pInfo->variables->WriteFile(pathName, varName.c_str(), bBase64Decode);

   delete[] pathName;

   return(status);
}
