#include "..\BoardDLLs\brd_generic.h"
#include "step_getcallinfo.h"

StepGetCallInfo::StepGetCallInfo(STEPREC& stepRecord) : Step(stepRecord)
{
   variable1 = stepRecord.s1.arr;
   field1    = stepRecord.s2.arr;

   variable2 = stepRecord.s3.arr;
   field2    = stepRecord.s4.arr;

   variable3 = stepRecord.s5.arr;
   field3    = stepRecord.s6.arr;

   variable4 = stepRecord.s7.arr;
   field4    = stepRecord.s8.arr;

   variable5 = stepRecord.s9.arr;
	field5    = stepRecord.s10.arr;
}

StepGetCallInfo::~StepGetCallInfo(void)
{
}

const char* StepGetCallInfo::GetStepName(void) const
{
	return("Get Call Info");
}

IV_STATUS StepGetCallInfo::Execute(PIVRINFO pInfo)
{
   char* value;                     // temporary value used to parse field name parameters
   string field1_parsed;            // field identifiers after parsing for TeleFlow variables
   string field2_parsed;            //   "
   string field3_parsed;            //   "
   string field4_parsed;            //   "
   string field5_parsed;            //   "
   bool bParmsValid = true;         // validate all parameters before exiting
   IV_STATUS status = IVR_SUCCESS;  // status returned from board-level function(s)


   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }


   // Validate all parameters
   // -----------------------

   // Parse first field name
   if((value = pInfo->variables->ParseParameter("First Field Identifier", field1.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      field1_parsed = value;
      delete[] value;
   }

   // Parse second field name
   if((value = pInfo->variables->ParseParameter("Second Field Identifier", field2.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      field2_parsed = value;
      delete[] value;
   }

   // Parse third field name
   if((value = pInfo->variables->ParseParameter("Third Field Identifier", field3.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      field3_parsed = value;
      delete[] value;
   }

   // Parse fourth field name
   if((value = pInfo->variables->ParseParameter("Fourth Field Identifier", field4.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      field4_parsed = value;
      delete[] value;
   }

   // Parse fifth field name
   if((value = pInfo->variables->ParseParameter("Fifth Field Identifier", field5.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      field5_parsed = value;
      delete[] value;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }


   // Retrieve values from Board level
   // --------------------------------

   // Retrieve first field name into first variable
   if(!variable1.empty())
   {
      if(!field1_parsed.empty())
      {
         value = pInfo->ivBoard->GetCallInfo(field1_parsed.c_str());
         if(value != NULL)
         {
            if(!pInfo->variables->Set(variable1.c_str(), value))
            {
               pInfo->ivLog->Print("      ERROR: Failed to set variable '%s'\n", variable1.c_str());
               status = IVR_FATAL;
            }
            delete[] value;
         }
      }
      else
      {
         if(!pInfo->variables->Set(variable1.c_str(), ""))
         {
            pInfo->ivLog->Print("      ERROR: Failed to set variable '%s'\n", variable1.c_str());
            status = IVR_FATAL;
         }
      }
   }

   // Retrieve second field name into second variable
   if(!variable2.empty())
   {
      if(!field2_parsed.empty())
      {
         value = pInfo->ivBoard->GetCallInfo(field2_parsed.c_str());
         if(value != NULL)
         {
            if(!pInfo->variables->Set(variable2.c_str(), value))
            {
               pInfo->ivLog->Print("      ERROR: Failed to set variable '%s'\n", variable2.c_str());
               status = IVR_FATAL;
            }
            delete[] value;
         }
      }
      else
      {
         if(!pInfo->variables->Set(variable2.c_str(), ""))
         {
            pInfo->ivLog->Print("      ERROR: Failed to set variable '%s'\n", variable2.c_str());
            status = IVR_FATAL;
         }
      }
   }

   // Retrieve third field name into third variable
   if(!variable3.empty())
   {
      if(!field3_parsed.empty())
      {
         value = pInfo->ivBoard->GetCallInfo(field3_parsed.c_str());
         if(value != NULL)
         {
            if(!pInfo->variables->Set(variable3.c_str(), value))
            {
               pInfo->ivLog->Print("      ERROR: Failed to set variable '%s'\n", variable3.c_str());
               status = IVR_FATAL;
            }
            delete[] value;
         }
      }
      else
      {
         if(!pInfo->variables->Set(variable3.c_str(), ""))
         {
            pInfo->ivLog->Print("      ERROR: Failed to set variable '%s'\n", variable3.c_str());
            status = IVR_FATAL;
         }
      }
   }

   // Retrieve fourth field name into fourth variable
   if(!variable4.empty())
   {
      if(!field4_parsed.empty())
      {
         value = pInfo->ivBoard->GetCallInfo(field4_parsed.c_str());
         if(value != NULL)
         {
            if(!pInfo->variables->Set(variable4.c_str(), value))
            {
               pInfo->ivLog->Print("      ERROR: Failed to set variable '%s'\n", variable4.c_str());
               status = IVR_FATAL;
            }
            delete[] value;
         }
      }
      else
      {
         if(!pInfo->variables->Set(variable4.c_str(), ""))
         {
            pInfo->ivLog->Print("      ERROR: Failed to set variable '%s'\n", variable4.c_str());
            status = IVR_FATAL;
         }
      }
   }

   // Retrieve fifth field name into fifth variable
   if(!variable5.empty())
   {
      if(!field5_parsed.empty())
      {
         value = pInfo->ivBoard->GetCallInfo(field5_parsed.c_str());
         if(value != NULL)
         {
            if(!pInfo->variables->Set(variable5.c_str(), value))
            {
               pInfo->ivLog->Print("      ERROR: Failed to set variable '%s'\n", variable5.c_str());
               status = IVR_FATAL;
            }
            delete[] value;
         }
      }
      else
      {
         if(!pInfo->variables->Set(variable5.c_str(), ""))
         {
            pInfo->ivLog->Print("      ERROR: Failed to set variable '%s'\n", variable5.c_str());
            status = IVR_FATAL;
         }
      }
   }

   return(status);
}
