#include <string>
using std::string;

#pragma hdrstop
#include "..\inet\inet_code.h"
#include "..\BoardDLLs\brd_generic.h"
#include "..\escutil\iv_str.h"   // FindCaseInsensitiveString
#include "step_audioplay.h"      // StepAudioPlay
#include "step_loadwebdoc.h"

/* -------------------------------------------------------------------------
   Name:        StepLoadWebDoc::StepLoadWebDoc                         (Constructor)
   ------------------------------------------------------------------------- */
StepLoadWebDoc::StepLoadWebDoc(void) : Step()
{
   pszResultVar = NULL;
   pszTimeout = NULL;
   pszURL = NULL;
   pszMsgCd = NULL;
   pszFileNm = NULL;
   format = TF_BY_EXTENSION;
   pszConnectionTimeout = NULL;
   pszConnectionRetries = NULL;
}
/*StepLoadWebDoc::StepLoadWebDoc*/


/* -------------------------------------------------------------------------
   Name:        StepLoadWebDoc::~StepLoadWebDoc                         (Destructor)
   ------------------------------------------------------------------------- */
StepLoadWebDoc::~StepLoadWebDoc(void)
{
   delete[] pszResultVar;
   delete[] pszTimeout;
   delete[] pszURL;
   delete[] pszMsgCd;
   delete[] pszFileNm;
   delete[] pszConnectionTimeout;
   delete[] pszConnectionRetries;
}
/*StepLoadWebDoc::~StepLoadWebDoc*/


/* -------------------------------------------------------------------------
   Name:        StepLoadWebDoc::GetStepName                             (Public)
   ------------------------------------------------------------------------- */
const char* StepLoadWebDoc::GetStepName(void) const
{
	return("Load Web Document");
}

IV_STATUS StepLoadWebDoc::SetResultVar(const char* value)
{
   return(SetString(value, &pszResultVar));
}

IV_STATUS StepLoadWebDoc::SetTimeout(const char* value)
{
   return(SetString(value, &pszTimeout));
}

IV_STATUS StepLoadWebDoc::SetURL(const char* value)
{
   return(SetString(value, &pszURL));
}

IV_STATUS StepLoadWebDoc::SetMessageCode(const char* value)
{
   return(SetString(value, &pszMsgCd));
}

IV_STATUS  StepLoadWebDoc::SetFileName(const char* value)
{
   return(SetString(value, &pszFileNm));
}

void StepLoadWebDoc::SetFileType(int value)
{
   format = (SOUND_FORMAT) value;
}

IV_STATUS StepLoadWebDoc::SetConnectionTimeout(const char* value)
{
   return(SetString(value, &pszConnectionTimeout));
}

IV_STATUS StepLoadWebDoc::SetConnectionRetries(const char* value)
{
   return(SetString(value, &pszConnectionRetries));
}

/* -------------------------------------------------------------------------
   Name:        StepLoadWebDoc::Execute
   ------------------------------------------------------------------------- */
IV_STATUS StepLoadWebDoc::Execute(PIVRINFO pInfo)
{
   IV_STATUS status;
   char* pszURLString = NULL;
   char* value = NULL;
   char* pszFileNameString = NULL;
   char* pszResult = NULL;
   VoiceLink* vLink;
   int iTimeout;
   int iConnectionTimeout;
   int iConnectionRetries;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if(pInfo->variables->IsParsedEmpty(pszURL))
   {
      pInfo->ivLog->Print("      ERROR: URL address '%s' is undefined\n", pszURL);
      return(IVR_FATAL);
   }

   // --- Retrieve timeout pszTimeoutString (in milliseconds)
   if(pInfo->variables->IsParsedInvalid(pszTimeout))
   {
      pInfo->ivLog->Print("      ERROR: Silence TIMEOUT '%s' undefined\n", pszTimeout);
      return(IVR_FATAL);
   }
   else if(pInfo->variables->IsParsedEmpty(pszTimeout))
   {
      iTimeout = atoi(pInfo->variables->Get("@TIMEOUT"));
   }
   else
   {
      value = pInfo->variables->Parse(pszTimeout);
      iTimeout = atoi(value);
      delete[] value;
   }

   if((iTimeout < 0) && (iTimeout != FOREVER))
   {
		pInfo->ivLog->Print("      ERROR: Silence TIMEOUT '%s' must be >= 0\n",
         pInfo->variables->IsParsedEmpty(pszTimeout) ? "@TIMEOUT" : pszTimeout);
      return(IVR_FATAL);
   }

   // Retrieve connection timeout
   if(pInfo->variables->IsParsedInvalid(pszConnectionTimeout))
   {
      pInfo->ivLog->Print("      ERROR: HTTP Initial Connection TIMEOUT '%s' undefined\n", pszConnectionTimeout);
      return(IVR_FATAL);
   }
   else if(pInfo->variables->IsParsedEmpty(pszConnectionTimeout))
   {
      iConnectionTimeout = 500;
   }
   else
   {
      value = pInfo->variables->Parse(pszConnectionTimeout);
      iConnectionTimeout = atoi(value);
      delete[] value;
   }

   if((iConnectionTimeout <= 0) && (iTimeout != FOREVER))
   {
		pInfo->ivLog->Print("      ERROR: HTTP Initial Connection TIMEOUT '%s' must be > 0\n", pszConnectionTimeout);
      return(IVR_FATAL);
   }

   // Retrieve connection retries
   if(pInfo->variables->IsParsedInvalid(pszConnectionRetries))
   {
      pInfo->ivLog->Print("      ERROR: HTTP Initial Connection Retries '%s' undefined\n", pszConnectionRetries);
      return(IVR_FATAL);
   }
   else if(pInfo->variables->IsParsedEmpty(pszConnectionRetries))
   {
      iConnectionRetries = 2;
   }
   else
   {
      value = pInfo->variables->Parse(pszConnectionRetries);
      iConnectionRetries = atoi(value);
      delete[] value;
   }

   if(iConnectionRetries < 0)
   {
		pInfo->ivLog->Print("      ERROR: HTTP Initial Connection Retries '%s' must be >= 0\n", pszConnectionRetries);
      return(IVR_FATAL);
   }

   // --- Retrieve URL
   pszURLString = pInfo->variables->Parse(pszURL);

   // --- Retrieve filename
   // Message code
   if(!pInfo->variables->IsEmpty(pszMsgCd))
   {
      char* vFile;

      if((pszFileNameString = pInfo->variables->Parse(pszMsgCd)) == NULL)
      {
         pInfo->ivLog->Print("      ERROR: Message Code variable '%s' undefined\n", pszMsgCd);
         return(IVR_FATAL);
      }

      strtrim(pszFileNameString);

      vLink = pInfo->voiceList->Get(pszFileNameString);

      if(vLink == NULL)
      {
         pInfo->ivLog->Print("      ERROR: Failed to find message file for script code '%s'\n", pszFileNameString);
        	delete[] pszURLString;
         delete[] pszFileNameString;
         return(IVR_FATAL);
      }

      delete[] pszFileNameString;
      pszFileNameString = NULL;

      if(pInfo->simulator != NULL)
      {
         vFile = new char[strlen (vLink->GetWaveFile()) + 1];
         strcpy(vFile, vLink->GetWaveFile());
      }
      else
      {
         vFile = new char[strlen (vLink->GetVoiceFile()) + 1];
         strcpy(vFile, vLink->GetVoiceFile());
      }

      format = (SOUND_FORMAT) vLink->GetFileType();

      // Voice DB entries *may* contain variables->
      if((pszFileNameString = pInfo->variables->Parse(vFile)) == NULL)
      {
         pInfo->ivLog->Print("      ERROR: Failed to parse file name '%s'\n", vFile);
        	delete[] pszURLString;
         delete[] vFile;
         return(IVR_FATAL);
      }

      delete[] vFile;
   }
   // File name
   else if(!pInfo->variables->IsEmpty(pszFileNm))
   {
      if((pszFileNameString = pInfo->variables->Parse(pszFileNm)) == NULL)
      {
         pInfo->ivLog->Print("      ERROR: File name '%s' undefined\n", pszFileNm);
         return(IVR_FATAL);
      }
   }

   // Plays a file
   if(!pInfo->variables->IsEmpty(pszFileNameString))
   {
      char* pathName;

   	if(PortRequired(pInfo) != IVR_SUCCESS)
      {
         delete[] pszURLString;
         delete[] pszFileNameString;
         return(IVR_FATAL);
      }

      strtrim(pszFileNameString);
      pathName = pInfo->variables->FilePath(pszFileNameString);
      delete[] pszFileNameString;

      status = pInfo->ivBoard->LoadWebDocument(pszURLString, iTimeout,
         iConnectionTimeout, iConnectionRetries, pathName, format,
         atoi(pInfo->variables->Get("@VOLUME")), pszResult);

     	delete[] pszURLString;
      delete[] pathName;

      pInfo->variables->Set(pszResultVar, (pszResult != NULL) ? pszResult : "");

      delete[] pszResult;
   }
   // No file played
   else
   {
      delete[] pszFileNameString;

      LARGE_INTEGER liTimeout;
      HANDLE waitObjs[3];
      HANDLE hTimer = CreateWaitableTimer(0, false, 0);
      DWORD ret;
      bool firstSend = true;
      NET_STORAGE socket;
      string hostName;
      const char* parser;
      string::size_type index;
      const char* hostNameStart;
      int hostNameLength;
      bool loopAgain;
      string requestMessage;
      unsigned int totalBytesSent = 0;
      int bytesSent;
      string returnMessage;
      char returnCode[4];
      string number;
      string::size_type contentStart;
      string::size_type contentEnd = 0;
      string::size_type contentLength = 0;
      char buffer[128];
      int bytesReceived;
      int connectCounter = 1;

      // Initialise value to empty string
      pInfo->variables->Set(pszResultVar, "");

      // We don't support "https:" at this time
      index = FindCaseInsensitiveString(pszURLString, "https:");
      if(index != string::npos)
      {
         pInfo->ivLog->Print("      ERROR: Secure connections not supported\n");
         delete[] pszURLString;
         CloseHandle(hTimer);
         return(IVR_NOTSUPPORTED);
      }

      // Find beginning of host name ignoring "http:" prefix and/or "//" if in URL
      hostNameStart = pszURLString;

      index = FindCaseInsensitiveString(pszURLString, "http:");
      if(index != string::npos)
      {
         hostNameStart += index;
      }
      parser = strstr(hostNameStart, "//");
      if(parser != NULL)
      {
         hostNameStart = parser + 2;
      }

      // Find end of host name and get the length of host name
      parser = strchr(hostNameStart, '/');

      if(parser == NULL)
      {
         pInfo->ivLog->Print("      ERROR: No document specified on host '%s'\n", hostNameStart);
         delete[] pszURLString;
         CloseHandle(hTimer);
         return(IVR_FATAL);
      }

   	// Set timeout timer
      if(iTimeout != FOREVER)
      {
         liTimeout.QuadPart = -(LONGLONG) iTimeout * 10000;
         SetWaitableTimer(hTimer, &liTimeout, 0, 0, 0, false);
      }

      hostNameLength = parser - hostNameStart;

      // copy hostname
      hostName.assign(hostNameStart, 0, hostNameLength);

      pInfo->ivLog->Print("      Connecting to host '%s'\n", hostName.c_str());

      // Initialises socket and connects to the host
      // HTTP is always transferred on port 80
      if(!NetConnect(&socket, hostName.c_str(), 80, FD_CONNECT | FD_READ))// | FD_WRITE))
      {
         const char* errorMessage = WSAErrorMsg(WSAGetLastError());
         pInfo->ivLog->Print("      ERROR: Unable to connect to host\n");
         pInfo->ivLog->Print("         '%s'\n", errorMessage);
         pInfo->variables->Set(pszResultVar, errorMessage);
         delete[] pszURLString;
         CloseHandle(hTimer);
         return(IVR_FAIL);
      }

      // Set up events array
      waitObjs[0] = pInfo->messages->GetMessageSignal();  // system event such as shutdown
      waitObjs[1] = hTimer;                        // timer for the timeout value
      waitObjs[2] = socket.hEvent;                 // signal from socket

      // Try up to iConnectionRetries times to connect
      loopAgain = true;
      while(loopAgain && (connectCounter <= iConnectionRetries))
      {
         switch(WaitForMultipleObjects(3, waitObjs, false, iConnectionTimeout))
         {
            // System event
            case WAIT_OBJECT_0:
            {
               status = pInfo->ivBoard->ProcessSystemEvent();

               if(status != IVR_SUCCESS)
               {
                  NetCloseSocket(&socket);
                  delete[] pszURLString;
                  CloseHandle(hTimer);
                  return(status);
               }

               break;
            }

            // Timeout
            case WAIT_OBJECT_0 + 1:
            {
               pInfo->ivLog->Print("      ERROR: Timed out trying to connect to host\n");
               NetCloseSocket(&socket);
               delete[] pszURLString;
               CloseHandle(hTimer);
               return(IVR_TIMEOUT);
            }

            // Connection signalled
            case WAIT_OBJECT_0 + 2:
            {
               loopAgain = false;
               break;
            }

            // 500 ms connection timeout
            case WAIT_TIMEOUT:
            {
               if(connectCounter < iConnectionRetries)
               {
                  pInfo->ivLog->Print("      Connection retry %d\n", connectCounter++);
                  NetSendConnect(&socket, hostName.c_str(), 80);
               }
               else
               {
                  pInfo->ivLog->Print("      ERROR: Timed out trying to connect to host\n");
                  delete[] pszURLString;
                  CloseHandle(hTimer);
                  return(IVR_TIMEOUT);
               }

               break;
            }

            // WaitForMultipleObjects failed
            case WAIT_FAILED:
            {
               DWORD errorID = GetLastError();
               char MsgBuf[1024];

               FormatMessage(
                  FORMAT_MESSAGE_FROM_SYSTEM,
                  0,
                  errorID,
                  MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
                  MsgBuf,
                  1024,
                  0
               );

               pInfo->ivLog->Print("      ERROR: WaitForMultipleObjects returned: %x: %s", errorID, MsgBuf);
               pInfo->variables->Set(pszResultVar, MsgBuf);
               NetCloseSocket(&socket);
               delete[] pszURLString;
               CloseHandle(hTimer);
               return(IVR_FAIL);
            }
         }
      }

      // Create the HTTP request
      requestMessage = "GET ";
      requestMessage += (string) (hostNameStart + hostNameLength);
      requestMessage += " HTTP/1.0\r\nUser-Agent: TeleFlow\r\n\Host: ";
      requestMessage += hostName;
      requestMessage += ":";
      requestMessage += "80";
      requestMessage += "\r\n\Accept: */*\r\n\r\n";

      do
      {
         // Send the request
         bytesSent = NetSend(&socket, requestMessage.c_str() + totalBytesSent, requestMessage.length() - totalBytesSent);

         if(bytesSent != SOCKET_ERROR)
         {
            if(firstSend)
            {
               // If the send was successful then we know the connection actually happened
               // The connection signal on the socket is not a guarantee of a connection...
               // only that the socket has finished setting up
               pInfo->ivLog->Print("      Connection successful\n");
               firstSend = false;
            }

            totalBytesSent += bytesSent;
         }
         else if(WSAGetLastError() != WSAEWOULDBLOCK)
         {
            int lastError = WSAGetLastError();
            const char* errorMessage = WSAErrorMsg(lastError);

            if(lastError == WSAENOTCONN)
            {
               pInfo->ivLog->Print("      ERROR: Connection failed\n");
            }
            else
            {
               pInfo->ivLog->Print("      ERROR: failed trying to send message '%s'\n", requestMessage.c_str());
               pInfo->ivLog->Print("         '%s'\n", errorMessage);
            }

            pInfo->variables->Set(pszResultVar, errorMessage);
            NetCloseSocket(&socket);
            CloseHandle(hTimer);
            return(IVR_FAIL);
         }

         // Allow 10 ms for socket to clear
         Sleep(10);
      } while(totalBytesSent < requestMessage.length());

      delete[] pszURLString;

      pInfo->ivLog->Print("      Request successful, waiting for reponse\n");

      loopAgain = true;
      while(loopAgain)
      {
         switch(WaitForMultipleObjects(3, waitObjs, false, INFINITE))
         {
            // System event
            case WAIT_OBJECT_0:
            {
               status = pInfo->ivBoard->ProcessSystemEvent();

               if(status != IVR_SUCCESS)
               {
                  NetCloseSocket(&socket);
                  CloseHandle(hTimer);
                  return(status);
               }

               break;
            }

            // Timed out
            case WAIT_OBJECT_0 + 1:
            {
               pInfo->ivLog->Print("      ERROR: Timed out waiting for initial response\n");
               NetCloseSocket(&socket);
               CloseHandle(hTimer);
               return(IVR_TIMEOUT);
            }

            // Socket signalled
            case WAIT_OBJECT_0 + 2:
            {
               // Receive as many packets as have arrived
               do
               {
                  bytesReceived = NetRecv(&socket, buffer, 127);

                  if(bytesReceived != SOCKET_ERROR)
                  {
                     buffer[bytesReceived] = '\0';
                     returnMessage += (string) buffer;
                     ret = WaitForSingleObject(socket.hEvent, 0);
                  }
                  else if(WSAGetLastError() == WSAEWOULDBLOCK)
                  {
                     Sleep(10);
                  }
               } while(((bytesReceived == SOCKET_ERROR) && (WSAGetLastError() == WSAEWOULDBLOCK)) || (ret == WAIT_OBJECT_0));

               loopAgain = false;

               // Sometimes an WSAEWOULDBLOCK error occurs for unknown reasons but
               // it does not appear to be harmful so we ignore it and keep waiting
               if((bytesReceived == SOCKET_ERROR) && (WSAGetLastError() != WSAEWOULDBLOCK))
               {
                  int lastError = WSAGetLastError();
                  const char* errorMessage = WSAErrorMsg(lastError);

                  pInfo->ivLog->Print("      ERROR: Failed trying to receive initial response\n");
                  pInfo->ivLog->Print("         '%s'\n", errorMessage);
                  pInfo->variables->Set(pszResultVar, errorMessage);
                  NetCloseSocket(&socket);
                  CloseHandle(hTimer);
                  return(IVR_FAIL);
               }

               break;
            }

            // WaitForMultipleObjects failed
            case WAIT_FAILED:
            {
               DWORD errorID = GetLastError();
               char MsgBuf[1024];

               FormatMessage(
                  FORMAT_MESSAGE_FROM_SYSTEM,
                  0,
                  errorID,
                  MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
                  MsgBuf,
                  1024,
                  0
               );

               pInfo->ivLog->Print("      ERROR: WaitForMultipleObjects returned: %x: %s", errorID, MsgBuf);
               pInfo->variables->Set(pszResultVar, MsgBuf);
               NetCloseSocket(&socket);
               delete[] pszURLString;
               CloseHandle(hTimer);
               return(IVR_FAIL);
            }
         }
      }

      // Find the beggining of the content of the response
      // HTTP headers end with "\r\n\r\n"
      contentStart = returnMessage.find("\r\n\r\n") + 4;
      if(contentStart == string::npos)
      {
         pInfo->ivLog->Print("      ERROR: Incomplete header error\n");
         NetCloseSocket(&socket);
         CloseHandle(hTimer);
         return(IVR_FAIL);
      }

      // Check that response was OK (200)
      // Format: "HTTP1.1/ 200 OK\n"
      index = returnMessage.find(" ");
      returnCode[0] = returnMessage[++index];
      returnCode[1] = returnMessage[++index];
      returnCode[2] = returnMessage[++index];
      returnCode[3] = '\0';

      // Bad return code: not 200 to 209
      if((atoi(returnCode) < 200) || (atoi(returnCode) > 209))
      {
         index = returnMessage.find("\n");
         returnMessage.assign(returnMessage, 0, index - 1);
         // Store the error code in the result variable
         pInfo->variables->Set(pszResultVar, returnMessage.c_str());
         NetCloseSocket(&socket);
         CloseHandle(hTimer);
         return(IVR_FAIL);
      }

      //Check for content range
      // Format: "Content-Range: bytes 409 - 1354"
      if((index = FindCaseInsensitiveString(returnMessage.c_str(), "Content-Range:")) != string::npos)
      {
         // Find start of content
         for(; !isdigit(returnMessage[index]); index++);
         for(; isdigit(returnMessage[index]); index++);
         // Get end of content
         for(index++; !isdigit(returnMessage[index]); index++);
         for(number = ""; isdigit(returnMessage[index]); index++)
         {
            number += returnMessage[index];
         }
         contentEnd = atoi(number.c_str());
      }
      // Check for content length
      // Format: "Content-Length: 3969"
      else if((index = FindCaseInsensitiveString(returnMessage.c_str(), "Content-Length:")) != string::npos)
      {
         // Get start of content
         for(; !isdigit(returnMessage[index]); index++);
         for(number = ""; isdigit(returnMessage[index]); index++)
         {
            number += returnMessage[index];
         }
         contentLength = atoi(number.c_str());
      }

      // Don't have whole message yet
      while((returnMessage.length() < contentEnd) || (returnMessage.length() < contentStart + contentLength))
      {
         switch(WaitForMultipleObjects(3, waitObjs, false, INFINITE))
         {
            // System event
            case WAIT_OBJECT_0:
            {
               status = pInfo->ivBoard->ProcessSystemEvent();

               if(status != IVR_SUCCESS)
               {
                  NetCloseSocket(&socket);
                  CloseHandle(hTimer);
                  return(status);
               }

               break;
            }

            // Timed out
            case WAIT_OBJECT_0 + 1:
            {
               pInfo->ivLog->Print("      ERROR: Timed out waiting for complete response\n");
               NetCloseSocket(&socket);
               // Store as much of the message as we have received in the result variable
               pInfo->variables->Set(pszResultVar, returnMessage.c_str());
               CloseHandle(hTimer);
               return(IVR_TIMEOUT);
            }

            // Socket signalled
            case WAIT_OBJECT_0 + 2:
            {
               // Receieve as many packets as have arrived
               do
               {
                  bytesReceived = NetRecv(&socket, buffer, 127);

                  if(bytesReceived != SOCKET_ERROR)
                  {
                     buffer[bytesReceived] = '\0';
                     returnMessage += (string) buffer;
                     ret = WaitForSingleObject(socket.hEvent, 0);
                  }
                  else if(WSAGetLastError() == WSAEWOULDBLOCK)
                  {
                     Sleep(10);
                  }
               } while(((bytesReceived == SOCKET_ERROR) && (WSAGetLastError() == WSAEWOULDBLOCK)) || (ret == WAIT_OBJECT_0));

               // Sometimes an WSAEWOULDBLOCK error occurs for unknown reasons but
               // it does not appear to be harmful so we ignore it and keep waiting
               if((bytesReceived == SOCKET_ERROR) && (WSAGetLastError() != WSAEWOULDBLOCK))
               {
                  const char* errorMessage = WSAErrorMsg(WSAGetLastError());
                  pInfo->ivLog->Print("      ERROR: Failed trying to receive complete response\n");
                  pInfo->ivLog->Print("         '%s'\n", errorMessage);
                  pInfo->variables->Set(pszResultVar, errorMessage);
                  NetCloseSocket(&socket);
                  CloseHandle(hTimer);
                  return(IVR_FAIL);
               }

               break;
            }

            // WaitForMultipleObjects failed
            case WAIT_FAILED:
            {
               DWORD errorID = GetLastError();
               char MsgBuf[1024];

               FormatMessage(
                  FORMAT_MESSAGE_FROM_SYSTEM,
                  0,
                  errorID,
                  MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
                  MsgBuf,
                  1024,
                  0
               );

               pInfo->ivLog->Print("      ERROR: WaitForMultipleObjects returned: %x: %s", errorID, MsgBuf);
               pInfo->variables->Set(pszResultVar, MsgBuf);
               NetCloseSocket(&socket);
               delete[] pszURLString;
               CloseHandle(hTimer);
               return(IVR_FAIL);
            }
         }
      }

      pInfo->ivLog->Print("      Response received\n");

      // Close the socket
      NetCloseSocket(&socket);

      CloseHandle(hTimer);

      // Strip away the HTTP header
      returnMessage.assign(returnMessage, contentStart, returnMessage.length());

      if(!pInfo->variables->Set(pszResultVar, returnMessage.c_str()))
      {
         pInfo->ivLog->Print("      ERROR: Failed to set URL Text variable '%s'\n", pszResultVar);
         return(IVR_FATAL);
      }

      status = IVR_SUCCESS;
   }

   return(status);
 }
/*StepLoadWebDoc::Execute*/
