#include "step_savelog.h"

StepSaveLog::StepSaveLog(STEPREC& stepRecord) : Step(stepRecord)
{
   bSpecifiedLogFolder = (bool) stepRecord.u1;
   bSpecifiedLogName = (bool) stepRecord.u2;
   folderName = stepRecord.s1.arr;
   fileName = stepRecord.s2.arr;
}

StepSaveLog::~StepSaveLog(void)
{
}

const char* StepSaveLog::GetStepName(void) const
{
	return("Save Log");
}

// -------------------------------------------------------------------------
//   Name:        StepSaveLog::Execute
//
//   Purpose:     Flush the TeleFlow log to disk.
// -------------------------------------------------------------------------
IV_STATUS StepSaveLog::Execute(PIVRINFO pInfo)
{
   bool bParmsValid = true;      // Evaluate all parameters before exiting
   char* value;
   string parsedFolder;
   string parsedFileName;
   string parsedFullPath;
   string parsedTeleFlowPath;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if(bSpecifiedLogFolder)
   {
      if((value = pInfo->variables->ParseParameter("Folder name", folderName.c_str(), "")) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         parsedFolder = value;
         delete[] value;
         if(parsedFolder[parsedFolder.length()-1] != '\\')
         {
            parsedFolder += '\\';
         }
         parsedFullPath = pInfo->variables->FilePath(parsedFolder.c_str());
      }
   }
   else
   {
      // Use TeleFlow log folder
      parsedFolder = pInfo->pszLogDir;
      parsedFullPath = pInfo->variables->FilePath(parsedFolder.c_str());
      pInfo->ivLog->Print("      Save to TeleFlow log folder\n");
   }

   if(bSpecifiedLogName)
   {
      if((value = pInfo->variables->ParseParameter("File name", fileName.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         parsedFileName = value;
         delete[] value;

         // append port number to file name
         value = new char[36];
         sprintf(value, "%i", pInfo->iPort);    // if zero padding desired, change to "%03i"
         parsedFileName += "_";
         parsedFileName += value;
         delete[] value;
      }
   }
   else
   {
      // Use TeleFlow application name
      parsedFileName = pInfo->pszAppNm;
      pInfo->ivLog->Print("      Use TeleFlow application name\n");
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   // If Teleflow currently set to log all events, and we have not supplied our
   // own path and file name, then don't bother to log (we already have all the
   // data we need in the log up until this point), just output a notification
   if((pInfo->bLogOutput) && (!bSpecifiedLogName))
   {
      parsedTeleFlowPath = pInfo->variables->FilePath(pInfo->pszLogDir);
      if(stricmp(parsedFullPath.c_str(), parsedTeleFlowPath.c_str()) == 0)
      {
         pInfo->ivLog->Print("      ---- Skipping Save Log - Log Output currently set to 'All events' ---- \n");
         return(IVR_SUCCESS);
      }
   }
   parsedFullPath += parsedFileName;
   pInfo->ivLog->Print("      Saving log to '%s'\n", parsedFullPath.c_str());

   if(!pInfo->ivLog->SaveLogToFile(parsedFullPath.c_str(), true))
   {
      pInfo->ivLog->Print("      ERROR: Failed to open file\n");
      return(IVR_FAIL);
   }
   
   return(IVR_SUCCESS);
}
