#include <stdlib.h>             // Needed for "atoi".
#include <sys\timeb.h>          // Needed for "ftime"
#include "..\escutil\odbc_db.h"
#include "step_sqlbeginstmt.h"

StepSQLBeginStmt::StepSQLBeginStmt(STEPREC& stepRecord) : Step(stepRecord)
{
   odbcConnection = stepRecord.s2.arr;
   sqlHandle = stepRecord.s1.arr;
   stmtText = stepRecord.s6.arr;
   timeWarning = stepRecord.s3.arr;
}

StepSQLBeginStmt::~StepSQLBeginStmt(void)
{
}

const char* StepSQLBeginStmt::GetStepName(void) const
{
	return("SQL Statement");
}

IV_STATUS StepSQLBeginStmt::Execute(PIVRINFO pInfo)
{
   char* valueConnectHandle = NULL;
   char* valueStmtHandle = NULL;
   IV_STATUS  status;
   timeb      startTime;
   timeb      endTime;
   double      fStartTime;
   double      fEndTime;
   double      fElapsedTime;
   int        iTimeWarningThreshold;
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   SQLList *connection;
   SQL* stmt;

   // capture start time
   ftime(&startTime);
   fStartTime = startTime.time + (float)(startTime.millitm) / 1000;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if(pInfo->odbcManager == NULL)
   {
      pInfo->ivLog->Print("      ERROR: SQL not initialized.\n");
      return(IVR_FATAL);
   }

   if((valueStmtHandle = pInfo->variables->ParseParameter("Statement Handle", sqlHandle.c_str())) == NULL)
   {
      bParmsValid = false;
   }

   if((value = pInfo->variables->ParseParameter("Warning time threshold", timeWarning.c_str(), "2000")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iTimeWarningThreshold = atoi(value);
      delete[] value;
   }

   // Note: stmtText is actually parsed in BeginStmt
   if(pInfo->variables->IsParsedEmpty(stmtText.c_str()))
   {
   	pInfo->ivLog->Print("      ERROR: Statement '%s' is empty\n", stmtText.c_str());
      bParmsValid = false;
   }

   // Parse ODBC connection handle
   if((valueConnectHandle = pInfo->variables->ParseParameter("ODBC Connection", odbcConnection.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      if(strcmp(valueConnectHandle,"") == 0)
      {
         pInfo->ivLog->Print("      Using default ODBC handle\n");
      }

      // Find ODBC connection by its handle
      if((connection = pInfo->odbcManager->FindConnection(valueConnectHandle)) == NULL)
      {
         pInfo->ivLog->Print("      ERROR: No current connection matching handle '%s'\n", valueConnectHandle);
         bParmsValid = false;
      }
   }

   if(!bParmsValid)
   {
      delete[] valueConnectHandle;  // safe to delete, pointer was initialized to NULL
      delete[] valueStmtHandle;     //   "
      return(IVR_FATAL);
   }

   // --- Parameters validated - execution begins here ---

   // Find this SQL statement handle within the ODBC connection
   if((stmt = connection->FindSQL(valueStmtHandle)) != NULL)
   {
      pInfo->ivLog->Print("      WARNING: Handle '%s' still open.  Closing handle\n", stmt->GetReferenceName());
      delete stmt;
   }

   stmt = new SQL;
   stmt->SetReferenceName(valueStmtHandle);
   stmt->SetDBHandle(connection->GetDBHandle());
   connection->AddLast((Link*)stmt);

   status = stmt->BeginStmt(pInfo, stmtText.c_str());

   // capture end time, calculate difference, display in log
   ftime(&endTime);
   fEndTime = endTime.time + (float)endTime.millitm / 1000;
   fElapsedTime = fEndTime - fStartTime;

   if(fElapsedTime < ((float)iTimeWarningThreshold/1000))
   {
      pInfo->ivLog->Print("      Elapsed Time: %.3f \n", fElapsedTime);
   }
   else
   {
      pInfo->ivLog->Print("      Elapsed Time: %.3f  -  TIME WARNING\n", fElapsedTime);
   }

   delete[] valueConnectHandle;
   delete[] valueStmtHandle;
   return(status);
}
