#include <stdlib.h>             // Needed for "atoi".
#include "..\escutil\odbc_db.h"
#include "step_sqlendstmt.h"

StepSQLEndStmt::StepSQLEndStmt(STEPREC& stepRecord) : Step(stepRecord)
{
   sqlHandle = stepRecord.s1.arr;
   odbcConnection = stepRecord.s2.arr;
}

StepSQLEndStmt::~StepSQLEndStmt(void)
{
}

const char* StepSQLEndStmt::GetStepName(void) const
{
	return("SQL End");
}

IV_STATUS StepSQLEndStmt::Execute (PIVRINFO pInfo)
{
   char* valueConnectHandle = NULL;
   char* valueStmtHandle = NULL;
   bool bParmsValid = true;   // Evaluate all parameters before exiting
   SQLList *connection;
   SQL* stmt;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if(pInfo->odbcManager == NULL)
   {
      pInfo->ivLog->Print("      ERROR: SQL not initialized.\n");
      return(IVR_FATAL);
   }

   if((valueStmtHandle = pInfo->variables->ParseParameter("Statement Handle", sqlHandle.c_str())) == NULL)
   {
      bParmsValid = false;
   }

   // Parse ODBC connection handle
   if((valueConnectHandle = pInfo->variables->ParseParameter("ODBC Connection", odbcConnection.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      if(strcmp(valueConnectHandle,"") == 0)
      {
         pInfo->ivLog->Print("      Using default ODBC handle\n");
      }

      // Find ODBC connection by its handle
      if((connection = pInfo->odbcManager->FindConnection(valueConnectHandle)) == NULL)
      {
         pInfo->ivLog->Print("      ERROR: No current connection matching handle '%s'\n", valueConnectHandle);
         bParmsValid = false;
      }
   }

   if(!bParmsValid)
   {
      delete[] valueConnectHandle;  // safe to delete, pointer was initialized to NULL
      delete[] valueStmtHandle;     //   "
      return(IVR_FATAL);
   }

   // --- Parameters validated - execution begins here ---

   // 'connection' points to SQLList object for this ODBC connection handle

   if((stmt = connection->FindSQL(valueStmtHandle)) != NULL)
   {
      delete stmt;
      // Note: the 'Link' base class from which this SQL object is derived will
      //       remove this element from the the SQLList linked-list
   }

   delete[] valueConnectHandle;
   delete[] valueStmtHandle;

   // Reset the SQL State system variables
   pInfo->variables->SetSystem("@SYS_LAST_SQL_STATE", "");
   pInfo->variables->SetSystem("@SYS_LAST_SQL_STATE_CODE", "");
   pInfo->variables->SetSystem("@SYS_LAST_SQL_STATE_DESC", "");

   // Don't fail if no statement defined.
   return(IVR_SUCCESS);
}
