#include <stdlib.h>             // Needed for "atoi".
#include "..\BoardDLLs\brd_generic.h"
#include "wait.h"
#include "step_wait.h"

StepWait::StepWait(STEPREC& stepRecord) : Step(stepRecord)
{
   timeout = stepRecord.s1.arr;
   interruptable = (bool) stepRecord.u1;
}

StepWait::~StepWait(void)
{
}

const char* StepWait::GetStepName(void) const
{
	return("Wait");
}

IV_STATUS StepWait::Execute(PIVRINFO pInfo)
{
   int iTimeout;
   char* value;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   // Get timeout
   if((value = pInfo->variables->ParseParameter("Delay", timeout.c_str())) == NULL)
   {
      return(IVR_FATAL);
   }

   iTimeout = atoi(value);
   delete[] value;

   if(iTimeout < 0)
   {
		pInfo->ivLog->Print("      ERROR: TIMEOUT '%s' must be >= 0\n", timeout.c_str());
      return(IVR_FATAL);
   }

   pInfo->ivLog->Print("      Waiting %d milliseconds\n", iTimeout);

   if(iTimeout == 0)
   {
      return(IVR_SUCCESS);
   }

   return(pInfo->iPort > 0 ? pInfo->ivBoard->Wait(iTimeout, interruptable) : Wait(pInfo, iTimeout));
}
