#ifndef IVRINFO_H
#define IVRINFO_H

#include "ivr_defs.h"
#include "gui_log.h"
#include "..\TFServer\TFServer_communication.h"
#include "license_read.h"  // TeleFlow license information

// These declarations are used outside their respective .h files to minimise module coupling.
// GUI_Log.h and VarController.h were included because they are used almost everywhere.
class VarController;
class VoiceList;
class BoardGeneric;
class ODBCManager;
class XMLList;
class LanguageModule;
class Voice;
class TFEngineCommunicationDestination;
class TCPClientManager;

/* -------------------------------------------------------------------------
   Structure:   IVRINFO

   Purpose:     This structure is passed to the Engine thread and almost all
                functions called by the Engine thread.  This structure
                contains information needed to control the thread execution.

   Notes:       - Some of the variables in this structure are retained
                  between Engine shutdowns and restarts.

   Questions:   - If the Engine thread fails to start due to an
                  unrecoverable error, how do I prevent other parts of the
                  system from trying to restart it or send it messages?
   ------------------------------------------------------------------------- */
#define MAX_PARAMETERS 9

typedef struct
{
   // Used to define engine thread.
   char*         pszInstallationPath;
   char         *pszAppNm;          // Name of app. should be unique.
   int           iAppID;            // ID of app. should be unique.
   char         *pszAppFile;        // Name of .TAP file.
   char         *pszProtocol;
	int           iPort;             // Port number the app. intends to use - TF Feb.19/02
   int           iParamCount;       // Number of parameters in next struct.
   char         *pszParams[MAX_PARAMETERS];      // Optional parameters.

   char         *pszLogDir;         // Log directory.
   char*         pszVoiceSearchPath; // (optional) path to search for voice files
   LICENSE_STATUS portLicense;      // whether or not the port is licensed to run

   // Used to store call state info
   char         calledNumber[MAX_DIGIT_BUFFER_SIZE];  // called number (DNIS)
   char         callingNumber[MAX_DIGIT_BUFFER_SIZE]; // calling number (ANI or CLID)

   // Used to control restarts.
   DWORD         restartMax;        // Maximum restart count.
   DWORD         restartCnt;        // Remaining number of times to restart.
   DWORD         restartDelay;      // Time to delay before restarting thread.
   bool          bRestartInfinite;  // If true, restartCnt is ignored.
   bool          bLogOutput;        // Output should be sent to a log.
   bool          bLogAppend;        // Append to end of existing log file.
   bool          bRestart;          // Restart Engine thread if it terminates.
   bool          bTerminate;        // Set when the thread should terminate.
   bool			  reloadRequested;	// Set when the thread should reload

   // Information needed in thread.
   bool          shutdownRequested; // Set when application should shutdown
                                    // Thread may reload application later
   bool          safeToShutdown;    // Application not in a critical section
   LINE_STATUS   status;
   VarController* variables;         // Variable controller
   VoiceList    *voiceList;         // Maps msg # to file names.
   BoardGeneric *ivBoard;           // Board class.
   int          iLastNonSIPPort;    // Highest numbered traditionaly telephony port
   ODBCManager  *odbcManager;      // ODBC connection manager, also contains all 'SQLList' objects
   XMLList      *xmlList;           // Pointer to linked list of XML documents
   GUILog       *ivLog;             // Engine output.
   bool logSRAudio;

   // Sync loading.  Prevent multiple threads from loading .TAM files
   // at the same time.  If too many threads attempt to load, the Server
   // fails to open files causing the thread to crash when it can't find
   // a particular TAM.
   Mutex* loadingMutex;

   // Used to communicate with Service.
   HANDLE        hSimulator;
   HANDLE        hResume;
   CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>* commHub;
   TFEngineCommunicationDestination* messages;
   TCPClientManager* tcpClientManager;

   HINSTANCE boardLibrary;
   // Used for multi-language 'Say' support
   HINSTANCE languageLibrary;
   LanguageModule* language;
   HINSTANCE speechRecognitionLibrary;
   Voice* speechRecognition;
   HINSTANCE voiceAuthenticationLibrary;
   Voice* voiceAuthentication;
   HINSTANCE textToSpeechLibrary;
   Voice* textToSpeech;
} IVRINFO, *PIVRINFO;

#endif
