/* -------------------------------------------------------------------------
   File:        link.h

   Copyright (c) 1997-2001 enGenic. All rights reserved.
   ------------------------------------------------------------------------- */
#ifndef LINK_H
#define LINK_H

#if defined (__DLL__)
#define LINKLIBAPI __declspec(dllexport)
#else
#define LINKLIBAPI __declspec(dllimport)
#endif
/* ------------------------ FORWARD REFERENCE ------------------------------ */
class Link;

/* -------------------------------------------------------------------------
   Notes:       - Pure virtual class.
   ------------------------------------------------------------------------- */
class LINKLIBAPI Linkage
{
   public:
      virtual ~Linkage();

      virtual Link* Suc() const = 0;
      virtual Link* Pred() const = 0;

   protected:
      Linkage();           // Can only derive from this class.
};

/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
class LINKLIBAPI Head : public Linkage
{
   friend Link;

   public:
      Head();
      virtual ~Head();

      virtual Link* Suc() const;
      virtual Link* Pred() const;

      Link* First() const;
      Link* Last() const;

      void AddFirst (Link*);
      void AddLast  (Link*);

      long Cardinal() const;
      bool Empty() const;

      void Clear();

   private:
      Link *first;                     // First "Link" in the list.
      Link *last;                      // Last "Link" in the list.
};

/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
class LINKLIBAPI Link : public Linkage
{
   friend Head;

   public:
      virtual ~Link();

      virtual Link* Suc() const;
      virtual Link* Pred() const;

      Link* Out();
      void  InTo (Head* list);

      void Precede (Link* toPrecede);
      void Precede (Head* list);

      void Follow (Link* toFollow);
      void Follow (Head* list);

      bool inList() const;

   protected:
      Link();             // Can only derive from this class.

   private:
      void RemoveElement();

      void addAfter (Link*);
      void addBefore (Link*);

      Link    *prev;
      Link    *next;
      Head    *theList;
};

#endif
