#include <winsock2.h>

// Warnings disabled to avoid warnings generated by Microsoft code
#pragma warn -8027
#pragma warn -8060
#pragma warn -8061
#include <ws2tcpip.h>
// Warnings re-enabled
#pragma warn +8027
#pragma warn +8060
#pragma warn +8061

#include "ace\Time_Value.h"
#include "ace\OS_NS_sys_time.h"
#include "Socket.h"
#include "..\escutil\iv_str.h"
#include "SDPInfo.h"

SDPInfo::SDPInfo(void)
{
   Reset();
}

void SDPInfo::AddHostPayload(SDPInfo::PAYLOAD payload)
{
   AddPayload(hostPayloads, payload);
}

void SDPInfo::AddPayload(deque<SDPInfo::PAYLOAD>& payloadList, SDPInfo::PAYLOAD payload)
{
   deque<SDPInfo::PAYLOAD>::iterator pos;

   // Search for payload in payloads
   for(pos = payloadList.begin(); (pos != payloadList.end()) && (*pos != payload); pos++);

   // Add payload only if it isn't already in the list
   if(pos == payloadList.end())
   {
      payloadList.push_back(payload);
   }
}

void SDPInfo::AddPeerPayload(SDPInfo::PAYLOAD payload)
{
   AddPayload(peerPayloads, payload);
}

void SDPInfo::ClearHostPayloads(void)
{
   hostPayloads.clear();
}

void SDPInfo::ClearPeerPayloads(void)
{
   peerPayloads.clear();
}

const char* SDPInfo::GenerateSessionID(void)
{
   ACE_Time_Value timeOfDay = ACE_OS::gettimeofday();
   char number[14];

   sprintf(number, "%d", timeOfDay.sec());
   sessionID = number;
   sprintf(number, "%d", timeOfDay.usec());
   sessionID += (string) number;

   return(sessionID.c_str());
}

const char* SDPInfo::GetHostAddress(void) const
{
   return(hostAddress.c_str());
}

SDPInfo::PAYLOAD SDPInfo::GetHostPayload(unsigned int index) const
{
   return(index >= hostPayloads.size() ? INVALID_PAYLOAD : hostPayloads[index]);
}

unsigned short SDPInfo::GetHostPort(void) const
{
   return(hostPort);
}

const char* SDPInfo::GetHostUserName(void) const
{
   return(hostUserName.c_str());
}

SDPInfo::MEDIA SDPInfo::GetMediaType(void) const
{
   return(mediaType);
}

const char* SDPInfo::GetPeerAddress(void) const
{
   return(peerAddress.c_str());
}

SDPInfo::PAYLOAD SDPInfo::GetPeerPayload(unsigned int index) const
{
   return(index >= peerPayloads.size() ? INVALID_PAYLOAD : peerPayloads[index]);
}

unsigned short SDPInfo::GetPeerPort(void) const
{
   return(peerPort);
}

const char* SDPInfo::GetPeerUserName(void) const
{
   return(peerUserName.c_str());
}

SDPInfo::PAYLOAD SDPInfo::GetPrimaryPayload(void) const
{
   const deque<SDPInfo::PAYLOAD>* primaryPayloads = prioritiseHostPayloads ? &hostPayloads : &peerPayloads;
   const deque<SDPInfo::PAYLOAD>* secondaryPayloads = prioritiseHostPayloads ? &peerPayloads : &hostPayloads;
   deque<SDPInfo::PAYLOAD>::const_iterator primaryPayload;
   deque<SDPInfo::PAYLOAD>::const_iterator secondaryPayload;

   // Find first primary payload match in secondary payloads
   for(primaryPayload = primaryPayloads->begin(); primaryPayload != primaryPayloads->end(); primaryPayload++)
   {
      for(secondaryPayload = secondaryPayloads->begin(); secondaryPayload != secondaryPayloads->end(); secondaryPayload++)
      {
         if(*secondaryPayload == *primaryPayload)
         {
            return(*primaryPayload);
         }
      }
   }

   return(INVALID_PAYLOAD);
}

SDPInfo::PROTOCOL SDPInfo::GetProtocol(void) const
{
   return(protocol);
}

const char* SDPInfo::GetSessionID(void) const
{
   return(sessionID.c_str());
}

const char* SDPInfo::GetSessionName(void) const
{
   return(sessionName.c_str());
}

const char* SDPInfo::GetVersion(void) const
{
   return(version.c_str());
}

void SDPInfo::Print(Log* ivLog) const
{
   if(ivLog == NULL)
   {
      return;
   }

   ivLog->Print("         SDP\n");
   ivLog->Print("            Session Name: '%s'\n", sessionName.c_str());
   ivLog->Print("            Session ID: '%s'\n", sessionID.c_str());
   ivLog->Print("            Media Type: %d\n", mediaType);
   ivLog->Print("            Protocol: %d\n", protocol);
   ivLog->Print("            Version: '%s'\n", version.c_str());
   ivLog->Print("            Host User Name: '%s'\n", hostUserName.c_str());
   ivLog->Print("            Host Address: '%s'\n", hostAddress.c_str());
   ivLog->Print("            Host Port: %d\n", hostPort);
   ivLog->Print("            Peer User Name: '%s'\n", peerUserName.c_str());
   ivLog->Print("            Peer Address: '%s'\n", peerAddress.c_str());
   ivLog->Print("            Peer Port: %d\n", peerPort);
   // not printed:           prioritiseHostPayloads (bool)
   ivLog->Print("            Host Payloads: ");
   for(unsigned int index = 0; index < hostPayloads.size(); index++)
   {
      ivLog->Print("%d ", hostPayloads[index]);
   }
   ivLog->Print("\n");
   ivLog->Print("            Peer Payloads: ");
   for(unsigned int index = 0; index < peerPayloads.size(); index++)
   {
      ivLog->Print("%d ", peerPayloads[index]);
   }
   ivLog->Print("\n");
   ivLog->Print("            Primary Payload: %d\n", GetPrimaryPayload());
}

void SDPInfo::PrioritiseHostPayloads(void)
{
   prioritiseHostPayloads = true;
}

void SDPInfo::PrioritisePeerPayloads(void)
{
   prioritiseHostPayloads = false;
}

void SDPInfo::Reset(void)
{
   sessionName.clear();
   sessionID.clear();
   hostUserName.clear();
   peerUserName.clear();
   hostAddress.clear();
   peerAddress.clear();
   mediaType = INVALID_MEDIA;
   hostPort = Socket::INVALID_PORT;
   peerPort = Socket::INVALID_PORT;
   protocol = INVALID_PROTOCOL;
   version.clear();
   PrioritisePeerPayloads();
   hostPayloads.clear();
   peerPayloads.clear();
   peerInfoChanged = false;
   networkType = INTERNET;   // only value supported by TeleFlow
   addressType = IP4;        // only value supported by TeleFlow
}

void SDPInfo::SetHostAddress(const char* address)
{
   hostAddress = address;
}

void SDPInfo::SetHostPayloads(SDPInfo::PAYLOAD* payloads, int count)
{
   SetPayloads(hostPayloads, payloads, count);
}

void SDPInfo::SetHostPort(unsigned short port)
{
   hostPort = port;
}

void SDPInfo::SetHostUserName(const char* name)
{
   string::size_type userNameStart;
   string::size_type userNameEnd;

   if(name == NULL)
   {
      hostUserName = "";
      return;
   }
   else
   {
      hostUserName = name;
   }

   if((userNameStart = FindCaseInsensitiveString(name, "sip:")) != string::npos)
   {
      userNameStart += 4;
   }
   else if((userNameStart = FindCaseInsensitiveString(name, "sips:")) != string::npos)
   {
      userNameStart += 5;
   }
   else
   {
      userNameStart = 0;
   }

   if(((userNameEnd = FindCaseInsensitiveString(name, "@")) != string::npos) && (userNameEnd >= userNameStart))
   {
      hostUserName = hostUserName.substr(userNameStart, userNameEnd - userNameStart);
   }
   else
   {
      hostUserName = hostUserName.substr(userNameStart);
   }
}

void SDPInfo::SetMediaType(SDPInfo::MEDIA mediaType)
{
   this->mediaType = mediaType;
}

void SDPInfo::SetPayloads(deque<SDPInfo::PAYLOAD>& payloadList, SDPInfo::PAYLOAD* payloads, int count)
{
   payloadList.clear();

   if(payloads == NULL)
   {
      return;
   }

   for(int index = 0; index < count; index++)
   {
      AddPayload(payloadList, payloads[index]);
   }
}

void SDPInfo::SetPeerAddress(const char* address)
{
   peerAddress = address;
}

void SDPInfo::SetPeerPayloads(SDPInfo::PAYLOAD* payloads, int count)
{
   SetPayloads(peerPayloads, payloads, count);
}

void SDPInfo::SetPeerPort(unsigned short port)
{
   peerPort = port;
}

void SDPInfo::SetPeerUserName(const char* name)
{
   peerUserName = name;
}

void SDPInfo::SetProtocol(SDPInfo::PROTOCOL protocol)
{
   this->protocol = protocol;
}

void SDPInfo::SetProtocol(DWORD protocol)
{
   SDPInfo::PROTOCOL newProtocol = (SDPInfo::PROTOCOL)protocol;
   this->protocol = newProtocol;
}

void SDPInfo::SetSessionID(const char* ID)
{
   sessionID = ID;
}

void SDPInfo::SetSessionName(const char* name)
{
   sessionName = name;
}

void SDPInfo::SetVersion(const char* version)
{
   this->version = version;
}

bool SDPInfo::ParseRemoteInfo(const char* buffer, size_t bufsize)
{
   // Set of local variables to temporarily store data until we know the buffer is valid SDP
   string localSessionID;
   string localPeerUserName;
   string localPeerAddress;
   unsigned short localPeerPort = 0;
   SDPInfo::MEDIA localMediaType = INVALID_MEDIA;
   SDPInfo::PROTOCOL localProtocol = INVALID_PROTOCOL;
   deque<SDPInfo::PAYLOAD> localPeerPayloads;

   // Variables to perform parsing
   bool bEOL;
   unsigned int offset = 0;
   bool bValidSDP = true;
   char sdpType;
   string sdpValue;

   if((buffer == NULL) || (bufsize == 0))
   {
      return(false);
   }

   while(buffer[offset] && ((bufsize == 0) || (offset < bufsize)))
   {
      // 1st 2 character of each line must always be <type>=
      sdpType = buffer[offset];
      offset++;
      if ((buffer[offset] == NULL) || (buffer[offset] != '='))
      {
         return(false);
      }
      offset++;

      // the rest of the line is the <value>
      sdpValue = "";
      bEOL = false;
      while(buffer[offset] && !bEOL)
      {
         // Line can end in CRLF or just LF
         if((buffer[offset] == '\r') || (buffer[offset] == '\n'))
         {
            bEOL = true;
            offset++;
            if(buffer[offset] == '\n')
            {
               offset++;
            }
         }
         else
         {
            sdpValue += buffer[offset];
            offset++;
         }
      }

      switch(sdpType)
      {
         case 'o':           // owner/creator and session identifier
         {
            localPeerUserName = GetValueToken(sdpValue, 1);   // 1. <username>
            localSessionID = GetValueToken(sdpValue, 2);      // 2. <session_id>
            // skip 3. <version>
            // skip 4. <network_type>
            // skip 5. <address_type>
            // skip 6. <address> - use 'c'onnection instead
            if((localPeerUserName == "") || (localSessionID == ""))
            {
               bValidSDP = false;
            }
            break;
         }

         case 'v':           // protocol version
         case 's':           // session name
         {
            // skipped, TeleFlow retains the initial settings
            break;
         }

         case 'c':           // connection information (not req. if incl. in all media)
         {
            // skip 1. <network type>
            // skip 2. <address type>
            localPeerAddress = GetValueToken(sdpValue, 3);   // 3. <connection address>
            if(localPeerAddress == "")
            {
               bValidSDP = false;
            }
            break;
         }

         case 't':           // time the session is active
         case 'r':           // zero or more repeat times (optional)
         {
            // SDPInfo class does not support 'time' type
            // Most call control is handled by 3rd party SIP stack (eg: NMS SIP for NCC), which
            // presumably, handles this for us.  At the time it was written, this parse() method
            // needed to handle re-invites only; because NMS SIP did not handle them natively.
            break;
         }

         case 'm':           // media name and transport address
         {
            string temp;
            SDPInfo::PAYLOAD payload;
            unsigned int valuecount;
            SDPInfo::MEDIA tempMediaType = INVALID_MEDIA;
            SDPInfo::PROTOCOL tempProtocol = INVALID_PROTOCOL;

            // 1. <media>
            temp = GetValueToken(sdpValue, 1);
            if(temp == "audio")
            {
               tempMediaType = AUDIO;
            }

            // 3. <transport>
            temp = GetValueToken(sdpValue, 3);
            if(temp == "RTP/AVP")
            {
               tempProtocol = RTP;
            }

            // Make sure this is a media format TeleFlow can handle
            if((tempMediaType == AUDIO) && (tempProtocol == RTP))
            {
               localMediaType = AUDIO;
               localProtocol = RTP;

               // 2. <port>
               temp = GetValueToken(sdpValue, 2);
               localPeerPort = (unsigned short) atoi(temp.c_str());

               // 4. <fmt list>
               valuecount = 3;
               while(temp != "")
               {
                  valuecount++;
                  temp = GetValueToken(sdpValue, valuecount);
                  if(temp != "")
                  {
                     payload = (SDPInfo::PAYLOAD) atoi(temp.c_str());
                     AddPayload(localPeerPayloads, payload);
                  }
               }
            }

            break;
         }

         case 'i':           // session information
         case 'u':           // URI of description
         case 'e':           // email address
         case 'p':           // phone number
         case 'b':           // bandwidth information
         case 'z':           // time zone adjustments
         case 'k':           // encryption key
         case 'a':           // zero or more session attribute lines
         {
            // Optional types, ignored by TeleFlow, but still valid SDP
            break;
         }

         default:
         {
            // Unknown type, must ignore announcement (RFC 2327, section 6)
            bValidSDP = false;
         }

      } // switch

   } // loop

   if((bValidSDP))
   {
      // Replace object properties with local values...

      if(sessionID.compare(localSessionID) != 0)
      {
         sessionID = localSessionID;
         peerInfoChanged = true;
      }

      if(peerUserName.compare(localPeerUserName) != 0)
      {
         peerUserName = localPeerUserName;
         peerInfoChanged = true;
      }

      if(peerAddress.compare(localPeerAddress) != 0)
      {
         peerAddress = localPeerAddress;
         peerInfoChanged = true;
      }

      if(mediaType != localMediaType)
      {
         mediaType = localMediaType;
         peerInfoChanged = true;
      }

      if(peerPort != localPeerPort)
      {
         peerPort = localPeerPort;
         peerInfoChanged = true;
      }

      if(protocol != localProtocol)
      {
         protocol = localProtocol;
         peerInfoChanged = true;
      }

      deque<SDPInfo::PAYLOAD>::size_type numOldPayloads = peerPayloads.size();
      deque<SDPInfo::PAYLOAD>::size_type numNewPayloads = localPeerPayloads.size();
      if(numOldPayloads != numNewPayloads)
      {
         peerPayloads = localPeerPayloads;
         peerInfoChanged = true;
      }
      else
      {
         for(unsigned i=0; i < numOldPayloads; i++)
         {
            if(peerPayloads[i] != localPeerPayloads[i])
            {
               peerPayloads = localPeerPayloads;
               peerInfoChanged = true;
               break;   // end loop early, data change detected, so we are done
            }
         }
      }
   }

   return(bValidSDP);
}
//SDPInfo::ParseRemoteInfo

bool SDPInfo::IsPeerInfoChanged(void)
{
   return(peerInfoChanged);
}

void SDPInfo::ResetPeerInfoChange(void)
{
   peerInfoChanged = false;
}

string SDPInfo::GetValueToken(string values, unsigned int n)
{
   size_t first;
   size_t next;
   unsigned int tokencount;
   string token;

   if(!values.empty() && (values != ""))
   {
      tokencount = 1;
      first = 0;
      next = values.find(' ');

      while((tokencount < n) && (next != string::npos))
      {
         tokencount++;
         first = next + 1;
         next = values.find(' ', first);
      }

      if(tokencount == n)
      {
         token =  values.substr(first, next - first);
      }
   }

   return(token);
}

const char* SDPInfo::ComposeMessageBody()
{
   char temp[32];
   SDPInfo::PAYLOAD primaryPayload = GetPrimaryPayload();

   // SDP version
   messageBody =  "v=0\r\n";

   // origin
   messageBody += "o=";
   messageBody += GetHostUserName();
   messageBody += " ";
   messageBody += GetSessionID();
   messageBody += " ";
   messageBody += GetVersion();
   messageBody += " ";
   messageBody += TranslateNetworkType();
   messageBody += " ";
   messageBody += TranslateAddressType();
   messageBody += " ";
   messageBody += GetHostAddress();
   messageBody += "\r\n";

   // session name
   messageBody += "s=";
   messageBody += GetSessionName();
   messageBody += "\r\n";

   // connection
   messageBody += "c=";
   messageBody += TranslateNetworkType();
   messageBody += " ";
   messageBody += TranslateAddressType();
   messageBody += " ";
   messageBody += GetHostAddress();
   messageBody += "\r\n";

   // time
   messageBody += "t=0 0";
   messageBody += "\r\n";

    // media
   messageBody += "m=audio ";
   sprintf(temp, "%i", GetHostPort());
   messageBody += temp;
   messageBody += " RTP/AVP ";
   sprintf(temp, "%i", primaryPayload);
   messageBody += temp;
   messageBody += " 101";
   messageBody += "\r\n";

   // attributes for media
   messageBody += "a=sendrecv";
   messageBody += "\r\n";
   messageBody += "a=rtpmap:";
   sprintf(temp, "%i", primaryPayload);
   messageBody += temp;
   messageBody += " ";
   messageBody += TranslateEncodingName(primaryPayload);
   messageBody += "/8000";
   messageBody += "\r\n";
   messageBody += "a=ptime:20";
   messageBody += "\r\n";
   messageBody += "a=rtpmap:101 telephone-event/8000\r\n";
   messageBody += "a=fmtp:101 0-15\r\n";

   return(messageBody.c_str());
}
//SDPInfo::ComposeMessageBody

const char* SDPInfo::TranslateNetworkType(void)
{
   networkTypeString = "";

   // Currently TeleFlow only supports Internet
   switch(networkType)
   {
      case(INTERNET):
      default:
      {
         networkTypeString += "IN";
      }
   }

   return(networkTypeString.c_str());
}
// SDPInfo::TranslateNetworkType

const char* SDPInfo::TranslateAddressType(void)
{
   addressTypeString = "";

   // Currently TeleFlow only supports IP4
   switch(addressType)
   {
      case(IP4):
      default:
      {
         addressTypeString = "IP4";
      }
   }
   return(addressTypeString.c_str());
}
//SDPInfo::TranslateAddressType


const char* SDPInfo::TranslateEncodingName(SDPInfo::PAYLOAD payload)
{
   // Return the encoding name for a specific payload ID, for use in the rtpmap attribute
   switch(payload)
   {
      case(G711_U):
      {
         return("PCMU");
      }
      case(G711_A):
      {
         return("PCMA");
      }
      case(GSM):
      {
         return("GSM");
      }
      case(G723):
      {
         return("G723");
      }
      case(G729):
      {
         return("G729");
      }
      case(TELEPHONE_EVENT):
      {
         return("telephone-event");
      }
      default:
      {
         return("");
      }
   }
}
//SDPInfo::TranslateMediaEncoding

